﻿/*******************************************************************
* CMDatasetGeoJSON Class
* 
* @module CMDatasetGeoJSON
* @Copyright HSU, Jim Graham, 2019
******************************************************************/
//******************************************************************
// CMDatasetGeoJSON Constructor
//******************************************************************
/*
* Constructor
* @protected, @constructs
*/
function CMDatasetGeoJSON() 
{
	CMDatasetVector.call(this);
}
CMDatasetGeoJSON.prototype=Object.create(CMDatasetVector.prototype); // inherit prototype functions from PanelBase()

CMDatasetGeoJSON.prototype.contructor=CMDatasetGeoJSON; // override the constructor to go to ours
//**************************************************************
// Functions specifically for vector data
//**************************************************************

/*
* Called to obtain the data for the layer from a URL.
* Currently,only GeoJSON is supported as the transfer type.
*/
CMDatasetGeoJSON.prototype.SetURL=function(URL) 
{
	if (this.URL!=URL)
	{
		this.URL=URL;
		
		var TheRequest=new XMLHttpRequest(); // wait staff at the resturant
		TheRequest.open("GET",URL,true); // the URL is what we ordered
		TheRequest.TheURL=URL;
		TheRequest.TheDataset=this;
		//TheRequest.ZoomToBounds=ZoomToBounds;
				
		TheRequest.onreadystatechange=function() 
		{
			if (this.readyState == 4)  // done
			{
				if (this.status == 200) // OK
				{
					var TheText=this.responseText;
	
					// clip the data if ClipBoundary is set
					var TheGeoJSONObject=JSON.parse(TheText);
				
					this.TheDataset.SetGeoJSON(TheGeoJSONObject);
					
					// update the debuging panel if set
	
					if (this.TheDataset.GetParent(CMMainContainer)!=null)
					{
					}
					
					// call the layer's specified on load function if provided
					{
	//					this.TheDataset.SendMessageToListeners(CMDataset.MESSAGE_DATASET_LOADED);
					}
					
				}
				else alert("HTTP error "+this.status+" "+this.statusText+" ("+this.TheURL+")");
			}
		}
		TheRequest.send();
	}
};

CMDatasetGeoJSON.prototype.SaveDataToURL=function(URL)
{
	var TheGeoJSON=this.GetGeoJSONFeatures();
	
	var TheGeoJSONText=JSON.stringify(TheGeoJSON);
	
	URL=URL+encodeURI("?Data="+TheGeoJSONText);
	
	var TheRequest=new XMLHttpRequest(); // wait staff at the resturant
	//TheRequest.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
	TheRequest.open("PUT",URL,true); // the URL is what we ordered
	TheRequest.TheURL=URL;
	TheRequest.TheDataset=this;
			
	TheRequest.onreadystatechange=function() 
	{
		if (this.readyState == 4)  // done
		{
			if (this.status == 200) // OK
			{
				var TheText=TheRequest.responseText;

				var TheGeoJSONObject=JSON.parse(TheText);
				
			}
			else alert("HTTP error "+this.status+" "+this.statusText+" ("+this.TheURL+")");
		}
	}
	TheRequest.send(TheGeoJSONText);
}
//**********************************************************************
// CMDatasetGeoJSON functions to get and set the GeoJSON-based data
//**********************************************************************

/**
* Sets just the features within this dataset.
* @public
* @param - TheGeoJSONObject a valid GeoJSON object containing features.  The
*			object passed in must have a "features" array containing valid
*			GeoJSON features).
*/
CMDatasetGeoJSON.prototype.SetGeoJSONFeatures=function(TheGeoJSONObject)
{
	this.TheRegions=[];
	
	this.Type=CMDatasetGeoJSON.GetRegionsFromGeoJSON(TheGeoJSONObject,this.TheRegions);
	
	// project the data if needed
	
	var TheProjector=this.GetProjector();
	
	if ((TheProjector!=null)&&(true)) // project the data without saving original - jjg should we ave the original for a later projection?
	{
		this.TheRegions=TheProjector.ProjectRegionsFromGeographic(this.TheRegions,this.Type);
	}
	
	// save the data in the data object
	
	this.SetupBoundingBoxesFromRegions(this.TheRegions);
	
	// zoom to the data
	
/*	if (ZoomToBounds)
	{
		var TheMainContainer=this.GetParent(CMMainContainer);
		
		TheMainContainer.ZoomToBounds(this.GetBounds());
	}
*/
	this.SendMessageToListeners(CMDataset.MESSAGE_DATASET_LOADED);
}
/**
* Set this dataset to match a GeoJSON object.  This will replace any existing data
* @public
* @param TheGeoJSONObject - A GeoJSON object witha "features" object and optional "properties" object.
* @param ZoomToBounds - optional flag 
*/
CMDatasetGeoJSON.prototype.SetGeoJSON=function(TheGeoJSONObject)
{
	// get the attribute headings
	
	var NumColumns=0;
	var Headings=[];
	
	var Properties=TheGeoJSONObject.features[0].properties;
	
	for (var key in Properties) 
	{
		Headings.push(key);
		NumColumns++;
	}
	// setup the attribute columns
	
	var NumRows=TheGeoJSONObject.features.length;
	var Columns=[];
	for (var ColumnIndex=0;ColumnIndex<NumColumns;ColumnIndex++)
	{
		var Cells=[];
		var Heading=Headings[ColumnIndex];
		
		for (var RowIndex=0;RowIndex<NumRows;RowIndex++)
		{
			var Properties=TheGeoJSONObject.features[RowIndex].properties;
			
			Cells.push(Properties[Heading]);
		}
		Columns.push(Cells);
	}
	
	this.SetAttributes(Headings,Columns);
	
	// set the features and supress messages until we process the attributes
	this.SetGeoJSONFeatures(TheGeoJSONObject);
}

//**********************************************************************
// Public static functions to convert GeoJSON to CanvasMap vector types
//**********************************************************************
CMDatasetGeoJSON.GetAreaFromGeoJSON=function(GeoJSONArea,Closed)
{
	var NewArea=[];
	
	if (GeoJSONArea!=undefined)
	{
		for (var i=0;i<GeoJSONArea.length;i++)
		{
			var ThePoly=GeoJSONArea[i];
			
			var Xs=[];
			var Ys=[];
			
			var NumCoordinates=ThePoly.length;
			if (Closed) NumCoordinates-=1; // remove last coordinate from closing GeoJSON poly
			
			for (var j=0;j<NumCoordinates;j++)
			{
				Xs.push(ThePoly[j][0]);
				Ys.push(ThePoly[j][1]);
			}
			NewArea.push([Xs,Ys]);
		}
	}
	return(NewArea);
}

/*
* Paint a geometry that is in reference coordinates (i.e. GeoJSON)
* @private
* @param TheGeometry
*/
CMDatasetGeoJSON.GetRegionFromGeoJSONGeometry=function(TheGeometry,Region)
{
	var Type=null;
	
	//if (Region==undefined) Region=[];
	
	if (TheGeometry.type=="Point")
	{
		var GeoJSONArea=[[TheGeometry.coordinates]]; // x and y
		
		var NewArea=CMDatasetGeoJSON.GetAreaFromGeoJSON(GeoJSONArea,false);
	
		if (NewArea!=null) Region.push(NewArea);
		
		Type=CMDatasetVector.TYPE_POINTS;
	}
	else if (TheGeometry.type=="LineString")
	{
		var GeoJSONArea=[TheGeometry.coordinates];
		
		var NewArea=CMDatasetGeoJSON.GetAreaFromGeoJSON(GeoJSONArea,false);
	
		if (NewArea!=null) Region.push(NewArea);
		
		Type=CMDatasetVector.TYPE_POLYLINES;
	}
	else if (TheGeometry.type=="MultiLineString")
	{
		// for a multiline string, each string is a set of line segments in an area (i.e. a connected stream or road network)
		for (var j=0;j<TheGeometry.coordinates.length;j++)
		{
			var GeoJSONArea=[TheGeometry.coordinates[j]];
		
			var NewArea=CMDatasetGeoJSON.GetAreaFromGeoJSON(GeoJSONArea,false);
		
			if (NewArea!=null) Region.push(NewArea);
		}
		
		Type=CMDatasetVector.TYPE_POLYLINES;
	}
	else if (TheGeometry.type=="Polygon") // really an array of polygons (an area with optional holes)
	{
		var GeoJSONArea=[];
		
		for (var j=0;j<TheGeometry.coordinates.length;j++) // first polygon is exterior, others are interior (this needs to be cached)
		{
			GeoJSONArea.push(TheGeometry.coordinates[j]);
		}
		var NewArea=CMDatasetGeoJSON.GetAreaFromGeoJSON(GeoJSONArea,true);
	
		if (NewArea!=null) Region.push(NewArea);
		
		Type=CMDatasetVector.TYPE_POLYGONS;
	}
	else if (TheGeometry.type=="MultiPolygon") // multiple areas each which can contain multiple polygons
	{
		for (var i=0;i<TheGeometry.coordinates.length;i++) // each loop is a separate area
		{
			var GeoJSONArea=[];
		
			var TheCoordinateArrays=TheGeometry.coordinates[i];
			
			for (var j=0;j<TheCoordinateArrays.length;j++) // first polygon is exterior, others are interior (this needs to be cached)
			{
				GeoJSONArea.push(TheCoordinateArrays[j]);
			}
			var NewArea=CMDatasetGeoJSON.GetAreaFromGeoJSON(GeoJSONArea,true);
		
			if (NewArea!=null) Region.push(NewArea);
		}
		
		Type=CMDatasetVector.TYPE_POLYGONS;
	}
	else if (TheGeometry.type=="GeometryCollection")
	{
		for (var j=0;j<TheGeometry.geometries.length;j++)
		{
			Type=CMDatasetGeoJSON.GetRegionFromGeoJSONGeometry(TheGeometry.geometries[j],Region);
		}
	}
	return(Type);
}
/**
* @private
*/
CMDatasetGeoJSON.GetRegionsFromGeoJSON=function(GeoJSONData,Regions)
{
	var Result=null;
	
	var TheFeatures=GeoJSONData.features;

	// Loop through each feature
	for (var i=0; i < TheFeatures.length; i++) 
	{
		var TheGeometry=TheFeatures[i].geometry;
		
		var TheRegion=[];
		
		var Type=CMDatasetGeoJSON.GetRegionFromGeoJSONGeometry(TheGeometry,TheRegion);
		
		if (Type!=null) Result=Type;
		
		Regions.push(TheRegion);
	}
	return(Result);
}
//**********************************************************************
// Public static functions to convert CanvasMap vector types to GeoJSON
//**********************************************************************
CMDatasetGeoJSON.GetGeoJSONCoordinatesFromPoly=function(ThePoly,TheType)
{
	var GeoJSONPoly=[];
	
	if (ThePoly!=undefined)
	{
		var Xs=ThePoly[0];
		var Ys=ThePoly[1];
		
		for (var j=0;j<Xs.length;j++)
		{
			GeoJSONPoly.push([Xs[j],Ys[j]]);
		}
		// have to loop back to the first point for a GeoJSON polygon
		if (TheType==CMDatasetVector.TYPE_POLYGONS) GeoJSONPoly.push([Xs[0],Ys[0]]);
	}
	return(GeoJSONPoly);
}
CMDatasetGeoJSON.GetGeoJSONCoordinatesFromArea=function(TheArea,TheType)
{
	var NewGeoJSONArea=[];
	
	if (TheArea!=undefined)
	{
		for (var i=0;i<TheArea.length;i++) // go through the polys
		{
			var ThePoly=TheArea[i];
			
			var GeoJSONPoly=CMDatasetGeoJSON.GetGeoJSONCoordinatesFromPoly(ThePoly,TheType);
			
			NewGeoJSONArea.push(GeoJSONPoly);
		}
	}
	return(NewGeoJSONArea);
}
CMDatasetGeoJSON.GetGeoJSONCoordinatesFromRegion=function(TheRegion,TheType)
{
	var NewGeoJSONRegion=[];
	
	if (TheRegion!=undefined)
	{
		for (var i=0;i<TheRegion.length;i++) // go through the polys
		{
			var TheArea=TheRegion[i];
			
			var GeoJSONPoly=CMDatasetGeoJSON.GetGeoJSONCoordinatesFromArea(TheArea,TheType);
			
			NewGeoJSONRegion.push(GeoJSONPoly);
		}
	}
	return(NewGeoJSONRegion);
}
CMDatasetGeoJSON.GetGeoJSONGeometryFromRegion=function(TheRegion,TheType)
{
	var GeoJSONGeometry="";
	
	switch (TheType)
	{
	case CMDatasetVector.TYPE_POINTS:
		{
			TheCoordinates=[];
			
			//for (var i=0;i<TheRegion.length;i++)
			{
				var TheArea=TheRegion[0];
				
				var ThePoly=TheArea[0];
				
				var X=ThePoly[0][0];
				var Y=ThePoly[1][0];
				
				TheCoordinates=[X,Y];
			}
			GeoJSONGeometry={
				"type":"Point",
				"coordinates":TheCoordinates
			}
		}
		break;
	case CMDatasetVector.TYPE_POLYLINES:
		{
			if (TheRegion.length==1) // Line string
			{
				var TheArea=TheRegion[0];
				
				var ThePoly=TheArea[0];
				
				var TheCoordinates=CMDatasetGeoJSON.GetGeoJSONCoordinatesFromPoly(ThePoly,TheType);
				
				GeoJSONGeometry={
					"type":"LineString",
					"coordinates":TheCoordinates
				}
			}
			else // MultilineString
			{
				var TheArea=TheRegion[0];
				
				var TheCoordinates=CMDatasetGeoJSON.GetGeoJSONCoordinatesFromArea(TheArea,TheType);
				
				GeoJSONGeometry={
					"type":"MultiLineString",
					"coordinates":TheCoordinates
				}
			}
		}
		break;
	case CMDatasetVector.TYPE_POLYGONS:
		{
			if (TheRegion.length==1) // polygon 
			{
				var TheArea=TheRegion[0];
				
				var ThePoly=TheArea[0];
				
				var TheCoordinates=CMDatasetGeoJSON.GetGeoJSONCoordinatesFromArea(TheArea,TheType);
				
				GeoJSONGeometry={
					"type":"Polygon",
					"coordinates":TheCoordinates
				}
			}
			else // MultilineString
			{
				var TheCoordinates=CMDatasetGeoJSON.GetGeoJSONCoordinatesFromRegion(TheRegion,TheType);
				
				GeoJSONGeometry={
					"type":"MultiPolygon",
					"coordinates":TheCoordinates
				}
			}
		}
		break;
	}
	return(GeoJSONGeometry);

}
/**
* Converts the contents of the Dataset to a GeoJSON object
*/ 
CMDatasetGeoJSON.prototype.GetGeoJSONFeatures=function()
{
	var TheFeatures=[];
	
	var TheRegions=this.GetRegions();
	var TheType=this.GetType();
	
	var TheHeadings=this.GetAttributeHeadings();
	
	for (var i=0;i<TheRegions.length;i++)
	{
		var TheRegion=TheRegions[i];
		
		var TheGeoJSONGeometry=CMDatasetGeoJSON.GetGeoJSONGeometryFromRegion(TheRegion,TheType);
		
		var TheProperties={};
		
		for (var j=0;j<TheHeadings.length;j++)
		{
			TheProperties[TheHeadings[j]]=this.GetAttributeCell(j,i);
		}
		
		var TheFeature={
			"type":"Feature",
			"geometry":TheGeoJSONGeometry,
			"properties":TheProperties
		}
		
		TheFeatures.push(TheFeature);
	}
	var Result={
		  "type": "FeatureCollection",
 		 "features": TheFeatures
	};
	
	return(Result);
}
 