/*******************************************************************
* CMDatasetPyramid Class
* Represnts a pyramid of raster tiles. 
*
* member Variables:
*  NumRows - undefined if there are no tiles
* 
* @module CMDatasetPyramid
* @Copyright HSU, Jim Graham, 2019
******************************************************************/
//******************************************************************
// CMDatasetPyramid Class
//
// Represnts a pyramid of raster tiles. 
//
// member Variables:
//  NumRows - undefined if there are no tiles
//
//******************************************************************
//******************************************************************
// Global Variables
//******************************************************************
/**
* Types of data sets
* @protected
*/
CMDatasetPyramid.MESSAGE_DATASET_TILE_LOADED=CMBase.GetUniqueNumber(); // includes TheTile in the parameters

//******************************************************************
// CMDatasetPyramid Constructor
//******************************************************************
/*
* Constructor
* @protected, @constructs
*/
function CMDatasetPyramid() 
{
	CMDataset.call(this);

	// properties
	this.AttributeInfo=null; // Attribute table for pyramid data
	this.HeadingIndex=-1; // index into the headings array (found from HTMLAttribute in parent);
	
	this.TileWidthInPixels=256;
	
	this.PaintDebugTileInfo=false; // could be a setting
	this.DebugZoomLevel=-1;
	this.DebugGlobalColumn=0;
	this.DebugGlobalRow=0;
	
	this.Bounds=null; // bounds for the entire pyramid
}
CMDatasetPyramid.prototype=Object.create(CMDataset.prototype); // inherit prototype functions from PanelBase()

CMDatasetPyramid.prototype.contructor=CMDatasetPyramid; // override the constructor to go to ours

//******************************************************************
// Functions used by subclasses and not overriden
//*****************************************************************
CMDatasetPyramid.prototype.SetDebugTile=function(ZoomLevel,GlobalColumn,GlobalRow) 
{
	this.PaintDebugTileInfo=true;
	this.DebugZoomLevel=ZoomLevel;
	this.DebugGlobalColumn=GlobalColumn;
	this.DebugGlobalRow=GlobalRow;
}

/*
* Called to obtain the data for the layer from a URL.
* @protected
* @param URL - null to reset the tiles and display nothing
*/
CMDatasetPyramid.prototype.SetURL=function(URL) 
{
	if (this.URL!=URL)
	{
		this.URL=URL;
		
		if (URL==null)
		{
			this.Tiles=undefined;
			this.NumRows=undefined;
			this.NumColumns=undefined;
		}
		else
		{
			var URL=this.URL+"Info.js";
		
			var TheRequest=new XMLHttpRequest();
			
			TheRequest.open("GET",URL,true);
			TheRequest.TheURL=URL;
			TheRequest.TheDataset=this;
					
			TheRequest.onreadystatechange=function() 
			{
				if( this.readyState == 4)  // done
				{
					if( this.status == 200) // OK
					{
						var TheText=this.responseText;
		
						var TheInfo=JSON.parse(TheText);
						
						//*******************
						
						if (this.TheDataset.GetName()===null) this.TheDataset.SetName(TheInfo.Title);
						
						var TheBounds = TheInfo.Bounds;
						this.TheDataset.SetBounds(TheBounds);
						
						this.TheDataset.OriginalBounds=TheInfo.OriginalBounds;
						
						this.TheDataset.TileRefWidth=TheInfo.TileRefWidth;
						this.TheDataset.MinColumn=Math.round(TheInfo.MinColumn) ;
						this.TheDataset.MaxColumn=Math.round(TheInfo.MaxColumn) ;
						this.TheDataset.NumColumns=Math.round(TheInfo.NumColumns) ;
						this.TheDataset.MinRow=Math.round(TheInfo.MinRow) ;
						this.TheDataset.MaxRow=Math.round(TheInfo.MaxRow) ;
						this.TheDataset.NumRows=Math.round(TheInfo.NumRows) ;
					
						this.TheDataset.Type=TheInfo.Type;
						
						this.TheDataset.TopZoomLevel=Math.round(TheInfo.TopZoomLevel) ;
						
						if ((this.TheDataset.Type=="REGIONS")||(this.TheDataset.Type=="POINTS"))
						{
							this.TheDataset.AttributeInfo=TheInfo.AttributeInfo;
							//this.LoadAttributeInfo(this.TheURL);
						}
						if (this.TheDataset.Type=="RASTER_DATA")
						{
							this.TheDataset.MinPixelValues=TheInfo.MinPixelValues;
							this.TheDataset.MaxPixelValues=TheInfo.MaxPixelValues;
						}
						
						//*************************************************************************
						// load the first step of child tiles
						
						this.TheDataset.Tiles=[];
					
						var TheMainContainer=this.TheDataset.GetParent(CMMainContainer);
						
	//					var TheView=TheMainContainer.GetScene().GetView(0); // jjg= fix later
						
						for (Row=0;Row<this.TheDataset.NumRows;Row++)
						{
							this.TheDataset.Tiles[Row]=[];
								
							for (Column=0;Column<this.TheDataset.NumColumns;Column++)
							{
								var GlobalRow=this.TheDataset.MaxRow-Row;
								var GlobalColumn=Column+this.TheDataset.MinColumn;
								var ZoomLevel=this.TheDataset.TopZoomLevel;
								
								this.TheDataset.Tiles[Row][Column]=new CMTile(this.TheDataset,ZoomLevel,GlobalColumn,GlobalRow);
								
								if (this.PaintDebugTileInfo) TheChildTile.SetPaintTileInfo(this.PaintDebugTile);
								
								this.TheDataset.Tiles[Row][Column].LoadTile();
							}
						}
						this.TheDataset.SendMessageToListeners(CMDataset.MESSAGE_DATASET_LOADED);
					}
					else alert("HTTP error "+this.status+" "+this.statusText+" ("+this.TheURL+")");
				}
			}
			TheRequest.send();
		}
	}
}
CMDatasetPyramid.prototype.CMDataset_GetIcon=CMDataset.prototype.GetIcon;


CMDatasetPyramid.prototype.GetIcon=function(TheLayer,Default)
{
	var TheIcon=this.CMDataset_GetIcon(TheLayer,Default);
	
//	if ((this.TheRegions!=null)) //
	{
		switch (this.Type)
		{
		case "REGIONS":
			{
				TheIcon=document.createElement('CANVAS');
				TheIcon.className="CM_LayerListIconClass";
				TheIcon.TheLayer=this;
				TheIcon.style.borderColor="rgba(0,0,0,0)";
				
				TheIcon.width=16;
				TheIcon.height=16;
				
				var TheView=new CMView2D();
				TheView.Setup(TheIcon);
					
				var StrokeStyle=TheLayer.GetSetting("Style","strokeStyle");
				var LineWidth=TheLayer.GetSetting("Style","lineWidth");
	
				var TheContext = TheIcon.getContext("2d");
				TheContext.strokeStyle=StrokeStyle;
				TheContext.lineWidth=LineWidth;
				
				TheContext.moveTo(0,0);
				TheContext.lineTo(16,16);
				TheContext.moveTo(0,16);
				TheContext.lineTo(7,7);
				TheContext.stroke();
			}
			break;
		case "POINTS":
			{
				var TheType=this.GetSetting("Mark","Type",undefined);
		
				if (TheType==undefined) TheType=CMLayer.MARK_CIRCLE;
				
				var StrokeStyle=TheLayer.GetSetting("Style","strokeStyle");
				var FillStyle=TheLayer.GetSetting("Style","fillStyle");
				
				TheIcon=TheLayer.GetMarkIcon(TheType,FillStyle,StrokeStyle);
			}
			break;
		}
	}
	return(TheIcon);
}
//******************************************************************
// Mouse event handling
//******************************************************************
CMDatasetPyramid.prototype.GetNumFeatures=function() 
{
	var Result=0;
	
	if (this.AttributeInfo!=null) // have vector data
	{
		Result=this.AttributeInfo.NumRows;
	}
	return(Result);
}
CMDatasetPyramid.prototype.InFeature=function(TheView,RefX,RefY,TargetFeatureIndex,RefTolerance) 
{
	var Result=false;
	
//	if (this.GetVisible())//&&(this.TheData!=null))
	{
		for (var Row=0;(Row<this.NumRows)&&(Result==false);Row++)
		{
			for (Column=0;(Column<this.NumColumns)&&(Result===false);Column++)
			{
				var TheTile=this.Tiles[Row][Column];
				
				var FeatureIndex=TheTile.In(TheView,RefX,RefY,RefTolerance);
				
				if (FeatureIndex==TargetFeatureIndex) Result=true;
			}
		}
	}
 	return(Result);
}
/*
* returns the feature index for the coordinate in projected space
* returns -1 if the coordinate is not in a feature
*/
CMDatasetPyramid.prototype.In=function(TheView,RefX,RefY,RefTolerance) 
{
	var FeatureIndex=-1;
	
//	if (this.GetVisible())//&&(this.TheData!=null))
	{
		for (var Row=0;(Row<this.NumRows)&&(FeatureIndex==-1);Row++)
		{
			for (Column=0;(Column<this.NumColumns)&&(FeatureIndex==-1);Column++)
			{
				var TheTile=this.Tiles[Row][Column];
				
				FeatureIndex=TheTile.In(TheView,RefX,RefY,RefTolerance);
			}
		}
	}
 	return(FeatureIndex);
};
CMDatasetPyramid.prototype.GetHeadingIndex=function()
{
	if (this.HeadingIndex==-1)
	{
		for (var i=0;i<this.AttributeInfo.Attributes.Headings.length;i++)
		{
			var Heading=this.AttributeInfo.Attributes.Headings[i];
			
			if (Heading==this.HTMLAttribute)
			{
				this.HeadingIndex=i;
			}
		}
	}
}
/**
* Display the info window if the user clicked on a feature
*/
CMDatasetPyramid.prototype.ShowInfoWindow=function(FeatureIndex,TheView,RefX,RefY) 
{
	if (this.HTMLAttribute!=null)
	{
		this.GetHeadingIndex();
		
		var TheColumn=this.Attributes.Values[this.HeadingIndex]; // jjg - need to reconcild headings with column indexes
		var TheHTML=TheColumn[FeatureIndex];
		
		var InfoWindow=TheView.CreateInfoWindow("CMDatasetPyramid.InfoWindow",RefX,RefY,200,30,TheHTML);
		
		CMMainContainer.SetPopupWindow(InfoWindow);
	}
}

CMDatasetPyramid.prototype.MouseDown=function(TheView,RefX,RefY,RefTolerance) 
{
	var Used=false;
	
	if ((this.IsVisible(TheView))&&
		((TheView.GetTool()==CMView.TOOL_INFO)&&(TheView.GetTool()==CMView.TOOL_SELECT))) // check if we where clicked in
	{
		for (var Row=0;(Row<this.NumRows)&&(Used==false);Row++)
		{
			for (Column=0;(Column<this.NumColumns)&&(Used==false);Column++)
			{
				var TheTile=this.Tiles[Row][Column];
				
				Used=TheTile.MouseDown(TheView,RefX,RefY,RefTolerance);
			}
		}
	}
	return(Used);
};
//**************************************************************
// Protected functions
//**************************************************************
CMDatasetPyramid.prototype.SetBounds=function(NewBounds) 
{
	this.TheBounds=NewBounds;
}
CMDatasetPyramid.prototype.GetBounds=function() 
{
	return(this.TheBounds);
}

//**************************************************************
// CMDataset Attribute functions
//**************************************************************
CMDatasetPyramid.prototype.GetNumAttributeRows=function() 
{ 
	var Result=0;
	if (this.AttributeInfo.Attributes!=null)
	{
		Result=this.AttributeInfo.NumRows;
	}
	return(Result); 
}
CMDatasetPyramid.prototype.GetNumAttributeColumns=function() 
{ 
	var Result=0;
	if (this.AttributeInfo!=null)
	{
		Result=this.AttributeInfo.NumRows;
	}
	return(Result); 
}
CMDatasetPyramid.prototype.GetAttributeHeading=function(ColumnIndex) 
{ 
	var Result="";
	if (this.AttributeInfo!=null)
	{
		Result=this.AttributeInfo.Attributes[ColumnIndex];
	}
	return(Result); 
}
CMDatasetPyramid.prototype.GetAttributeCellByHeading=function(Heading,RowIndex)  //jjg
{ 
	var Result=null;
	
	var ColumnIndex=this.GetAttributeIndexFromHeading(Heading);
	
	if (ColumnIndex==-1) 
	{
		throw("Sorry, the heading "+Heading+" was not found");
	}
	else
	{
		Result=this.GetAttributeCell(ColumnIndex,RowIndex);
	}
	return(Result);
}

CMDatasetPyramid.prototype.GetAttributeIndexFromHeading=function(Heading) 
{ 
	var Result=-1;
	if (this.AttributeInfo!=null)
	{
		for (var i=0;i<this.AttributeInfo.Attributes.length;i++)
		{
			if (this.AttributeInfo.Attributes[i].Heading==Heading)
			{
				Result=i;
			}
		}
	}
	return(Result); 
}
CMDatasetPyramid.prototype.GetAttributeCell=function(ColumnIndex,RowIndex) 
{ 
	var Result="";
	if (this.Attributes!=null)
	{
		var TheColumn=this.Attributes[ColumnIndex]; 
		Result=TheColumn[RowIndex];
	}
	return(Result); 
}
/*
* Called to load a column of attributes.
* @protected
*/

CMDatasetPyramid.prototype.LoadAttributeColumn=function(ColumnIndex) 
{
	var URL=this.URL+"Attributes/AttributeColumn_"+ColumnIndex+".js";

	var TheRequest=new XMLHttpRequest();
	
	TheRequest.open("GET",URL,true);
	TheRequest.TheURL=URL;
	TheRequest.TheDataset=this;
	TheRequest.ColumnIndex=ColumnIndex;
			
	TheRequest.onreadystatechange=function() 
	{
		if( this.readyState == 4)  // done
		{
			if( this.status == 200) // OK
			{
				var TheText=this.responseText;

				var TheInfo=JSON.parse(TheText);
				
				//*******************
				
				if (this.TheDataset.Attributes==undefined) this.TheDataset.Attributes=[];
				
				this.TheDataset.Attributes[this.ColumnIndex]=TheInfo.Values;
				
				this.TheDataset.SendMessageToListeners(CMDataset.MESSAGE_ATTRIBUTES_CHANGED);
			}
			else alert("HTTP error "+this.status+" "+this.statusText+" ("+this.TheURL+")");
		}
	}
	TheRequest.send();
}
CMDatasetPyramid.prototype.GetRasterFileExtension=function() 
{
	return(".png");
}
//******************************************************************
// Functions used by subclasses and not overriden
//******************************************************************

/**
* Gets the bounds for a specific feature, if any
*/
CMDatasetPyramid.prototype.GetFeatureBounds=function(FeatureIndex) 
{
	var Result=null;
	
	for (var Row=0;Row<this.NumRows;Row++)
	{
		for (Column=0;Column<this.NumColumns;Column++)
		{
			var TheTile=this.Tiles[Row][Column];
			
			Result=TheTile.AddToFeatureBounds(FeatureIndex,Result);
		}
	}
	return(Result);
}
//******************************************************************
// Textured 3d Tiles
//******************************************************************

/**
* Gets the bounds for a specific feature, if any
*/
CMDatasetPyramid.prototype.SetVisible=function(TheLayer,Flag) 
{
	var Result=null;
	
	if (Flag==false)
	{
		for (var Row=0;Row<this.NumRows;Row++)
		{
			for (Column=0;Column<this.NumColumns;Column++)
			{
				var TheTile=this.Tiles[Row][Column];
				
				TheTile.HideTiles(TheLayer,true);
			}
		}
	}
	this.Visible=Flag;
	return(Result);
}

//******************************************************************
// Textured 3d Tiles
//******************************************************************
/**
*
*/
CMDatasetPyramid.prototype.GetDataForTextureTile=function(TheView,TheTextureTile,Exaggeration)
{
	var Result=null;
	
	for (var Row=0;(Row<this.NumRows)&&(Result==null);Row++)
	{
		for (Column=0;(Column<this.NumColumns)&&(Result==null);Column++)
		{
			var TheTile=this.Tiles[Row][Column];
			
			Result=TheTile.GetDataForTextureTile(TheView,TheTextureTile,Exaggeration);
		}
	}
	return(Result);
}

//******************************************************************
// Painting
//******************************************************************
/*
* Paints a layer into the canvas
*/
CMDatasetPyramid.prototype.Paint=function(TheLayer,TheView,SelectedOnly,MouseOverOnly) 
{
	if ((this.NumRows!=undefined)&&(SelectedOnly==false)&&((MouseOverOnly==undefined)||(MouseOverOnly==false)))
	{
		// Get the drawing context from our <canvas> and
		// set the fill to determine what color our map will be.
		var TheContext=TheView.GetContext();
		
		for (var Row=0;Row<this.NumRows;Row++)
		{
			for (Column=0;Column<this.NumColumns;Column++)
			{
				var TheTile=this.Tiles[Row][Column];
				
				TheTile.PaintTile(TheLayer,TheView,SelectedOnly,this.TileRefWidth);
			}
		}
	}
}
//******************************************************************
// Searching
//******************************************************************

/*
* Requests search results from a layer.  The scene calls this function
* @protected
*/
CMDatasetPyramid.prototype.GetSearchResults=function(SearchPhrase,ResultsPanel) 
{
	var TheText="";
	
	if ((this.AttributeInfo!=null))
	{
		var NumRows=this.GetNumAttributeRows();
		var NumColumns=this.GetNumAttributeColumns();
		
		// draw each feature
		for (var ColumnIndex=0;ColumnIndex < NumColumns; ColumnIndex++) 
		{
			var TheColumn=this.Attributes[ColumnIndex]; // needs request
			
			if (TheColumn!=undefined) // jjg - need a way to indicate which columsn to search
			{
				for (var RowIndex=0;RowIndex < NumRows; RowIndex++) 
				{
					var TheValue=TheColumn[RowIndex];
					
					TheValue=TheValue.toLowerCase();
					
					if (TheValue.indexOf(SearchPhrase)!=-1)
					{
						var ThisResult=document.createElement("DIV");
						ThisResult.innerHTML=TheColumn[RowIndex];
						
						ThisResult.TheLayer=this;
						ThisResult.FeatureIndex=RowIndex;
						
						ThisResult.onclick=function()
						{
							this.TheLayer.TheScene.UnselectAll();
							this.TheLayer.SetSelectedFeature(this.FeatureIndex);
							
							var TheBounds=this.TheLayer.GetFeatureBounds(this.FeatureIndex);
							var TheView=this.TheLayer.TheScene.GetView(0);
							
							this.className="CM_SearchResultSelected";
							
							TheView.ZoomToBounds(TheBounds);
						}
						ResultsPanel.appendChild(ThisResult);
					}
				}
			}
		}
	}
	return(TheText);
}
