/******************************************************************************************************************
* CMDatasetRaster
* Manages a raster that is represnted by a JSON object.  This allows
* for any type (N-Band, RGB, etc.) and any data type (INT32, FLOAT 32)
* rasters to be managed in the client.d
*
* The drawing functions maps the raster to the screen based on
* the overall min max values.
*
* @module CMDatasetRaster
* @Copyright HSU, Jim Graham, 2019
******************************************************************************************************************/
//******************************************************************
// Constructor
//******************************************************************
/*
* Constructor
* @protected, @constructs
*/
function CMDatasetRaster() 
{
	CMDataset.call(this);

	// properties
	this.TheMatrix=null; // data for the raster
}
CMDatasetRaster.prototype=Object.create(CMDataset.prototype); // inherit prototype functions from PanelBase()

CMDatasetRaster.prototype.contructor=CMDatasetRaster; // override the constructor to go to ours
//******************************************************************
// CMLayer Class
//******************************************************************
/*
* Called to obtain the data for the layer from a URL.
* Currently,only GeoJSON is supported as the transfer type.
*/
CMDatasetRaster.prototype.SetURL=function(URL) 
{
	if (this.URL!=URL)
	{
	
		this.URL=URL;
		
		var TheRequest=new XMLHttpRequest(); // wait staff at the resturant
		TheRequest.open("GET",URL,true); // the URL is what we ordered
		TheRequest.TheURL=URL;
		TheRequest.TheDataset=this;
		//TheRequest.ZoomToBounds=ZoomToBounds;
				
		TheRequest.onreadystatechange=function() 
		{
			if (this.readyState == 4)  // done
			{
				if (this.status == 200) // OK
				{
					var TheText=TheRequest.responseText;
	
					var TheGeoJSONObject=JSON.parse(TheText);
					
					this.TheDataset.SetData(TheGeoJSONObject); // sends MESSAGE_DATASET_LOADED message
					
					//this.TheDataset.SendMessageToListeners(CMDataset.MESSAGE_DATASET_LOADED);
					
					// repaint last so the layer has the chance to chagne the painting settings (listeners made this obsolete)
				}
				else alert("HTTP error "+this.status+" "+this.statusText+" ("+this.TheURL+")");
			}
		}
		TheRequest.send();
	}
}
/*
* Set the data for this dataset.  Called by SetURL() and by CMLayerPyramid for
* raster data that is embedded in a tile.
*/
CMDatasetRaster.prototype.CMDataset_SetData=CMDataset.prototype.SetData;

CMDatasetRaster.prototype.SetData=function(TheData) 
{
	this.CMDataset_SetData(TheData);
	
	this.TheGeoJSONObject=TheData;
	this.NumBands=TheData.NumBands;
	this.WidthInPixels=TheData.WidthInPixels;
	this.HeightInPixels=TheData.HeightInPixels;
	this.NumBands=TheData.NumBands;
	this.MinPixelValues=TheData.MinPixelValues;
	this.MaxPixelValues=TheData.MaxPixelValues;
	
	this.SetBounds(TheData.Bounds);
	
	this.TheMatrix=TheData.Data;
	
	this.SendMessageToListeners(CMDataset.MESSAGE_DATASET_LOADED);
}

//******************************************************************
// CMDatasetRaster Functions
//******************************************************************
/**
* SetBounds must be called before SetImage or SetURL are called so the raster can be 
*/

CMDatasetRaster.prototype.SetImage=function(TheImage) 
{
	this.TheImage=TheImage;
}

CMDatasetRaster.prototype.In=function(TheView,RefX,RefY,RefTolerance) 
{
	return(-1);
};

//***************************************************************************
// reference functions
//*************************************************************************
/**
* Check if the feature is visible in the view.
* This should be called by subclasses but can also be called to limit a layer's bounds after loading.
* @public
* @param NewBounds - Bounds with format {XMin,XMax,YMin,YMax}
*/
CMDatasetRaster.prototype.SetBounds=function(NewBounds) 
{
	this.TheBounds=NewBounds;
}
/**
* Returns the bounds of the data within the layer.  Computed after loading the data.
* @public
* @returns Bounds - with format {XMin,XMax,YMin,YMax}
*/
CMDatasetRaster.prototype.GetBounds=function() 
{
	return(this.TheBounds);
}
//***************************************************************************
// reference functions
//*************************************************************************
CMDatasetRaster.prototype.GetRefY=function() 
{
	return(this.TheBounds.YMax);
}
CMDatasetRaster.prototype.GetRefX=function() 
{
	return(this.TheBounds.XMin);
}
CMDatasetRaster.prototype.GetRefWidth=function() 
{
	return(this.TheBounds.XMax-this.TheBounds.XMin);
}
CMDatasetRaster.prototype.GetRefHeight=function() 
{
	return(this.TheBounds.YMin-this.TheBounds.YMax);
}
//***************************************************************************
// reference functions
//*************************************************************************
CMDatasetRaster.prototype.GetPixelRefWidth=function()
{
	var Result=1;
	if (this.GetBounds()!=null) Result=this.GetRefWidth()/this.WidthInPixels;
	return(Result);
}
CMDatasetRaster.prototype.GetPixelRefHeight=function()
{
	var Result=-1;
	if (this.GetBounds()!=null) Result=this.GetRefHeight()/this.HeightInPixels;
	return(Result);
}

/**
 * Returns the horizontal pixel location of a horizontal reference value.
 * @param RefX - the horizontal reference value
 * @return - the horizontal pixel location in the raster
 */
CMDatasetRaster.prototype.GetPixelXFromRefX=function(RefX)
{
	var RefInRaster=RefX-this.GetRefX();
	
	var PixelX=(RefInRaster/this.GetRefWidth())*this.WidthInPixels;
	
	// we subtract one half of a pixel width to move the reference from the left
	// of the left-most pixel to the center of the pixel.
//	var PixelX=(((RefInRaster-(this.GetPixelRefWidth()/2))/this.GetRefWidth())*this.WidthInPixels);

	return(PixelX);
}
/**
 * Returns the vertical pixel location of a vertical reference value.
 * @param RefY - the vertical reference value
 * @return - the vertical pixel location in the raster
 */
CMDatasetRaster.prototype.GetPixelYFromRefY=function(RefY)
{
	var RefInRaster=RefY-this.GetRefY();

	var PixelY=(RefInRaster/this.GetRefHeight())*this.HeightInPixels;

//	var PixelY=(((RefInRaster-(this.GetPixelRefHeight()/2))/this.GetRefHeight())*this.HeightInPixels);

	return(PixelY);
}
/**
 * Gets the specified reference width (map units) to a width in pixels.
 * @param RefWidth
 * @return
 */
CMDatasetRaster.prototype.GetPixelWidthFromRefWidth=function(RefWidth)
{
	var PixelWidth=((RefWidth/this.GetRefWidth())*this.WidthInPixels);

	return(PixelWidth);
}
/**
 * Gets the specified reference height (map units) to a height in pixels.
 * @param RefHeight
 * @return
 */
CMDatasetRaster.prototype.GetPixelHeightFromRefHeight=function(RefHeight)
{
	var PixelHeight=((RefHeight/this.GetRefHeight())*this.HeightInPixels);

	return(PixelHeight);
}
/**
 * Gets a x reference value (map units) from a x pixel locatoin in this raster.
 * @param PixelX
 * @return
 */
CMDatasetRaster.prototype.GetRefXFromPixelX=function(PixelX)
{
	var RefInRaster=(PixelX/this.WidthInPixels)*this.GetRefWidth();//+(this.GetPixelRefWidth()/2);
	
	var RefX=RefInRaster+this.GetRefX();

	return(RefX);
}
/**
 * Gets a y reference value (map units) from a y pixel locatoin in this raster.
 * @param PixelY
 * @return
 */
CMDatasetRaster.prototype.GetRefYFromPixelY=function(PixelY)
{
	var RefInRaster=(PixelY/this.HeightInPixels)*this.GetRefHeight();//+(this.GetPixelRefHeight()/2);
	
	var RefY=RefInRaster+this.GetRefY();

	return(RefY);
}
/**
 * Converts a width in pixels to a a width in reference units (map units)
 * @param PixelWidth 
 * @return
 */
CMDatasetRaster.prototype.GetRefWidthFromPixelWidth=function(PixelWidth)
{
	var RefWidth=(PixelWidth/this.WidthInPixels)*this.GetRefWidth();

	return(RefWidth);
}
/**
 * Converts a height in pixels to a height in reference units (map units)
 * @param PixelHeight
 * @return
 */
CMDatasetRaster.prototype.GetRefHeightFromPixelHeight=function(PixelHeight)
{
	var RefHeight=(PixelHeight/this.HeightInPixels)*this.GetRefHeight();

	return(RefHeight);
}
//***************************************************************************
// Painting functions
//*************************************************************************

/*
* Paints a layer into the canvas
*/
CMDatasetRaster.prototype.Paint=function(TheView) 
{
	var TheImageData=null;
	
	if ((this.TheMatrix!=null))
	{
		// Get the drawing context from our <canvas> and
		// set the fill to determine what color our map will be.
		var TheContext=TheView.GetContext();
		
		var TheImageBounds=this.GetBounds();
		
	/*	var ImageRefX=TheImageBounds.XMin;
		var ImageRefY=TheImageBounds.YMax;
		var ImageRefWidth=TheImageBounds.XMax-TheImageBounds.XMin;
		var ImageRefHeight=TheImageBounds.YMin-TheImageBounds.YMax;
		
		// find the area of overlap between the view and the raster
		
		var TheViewBounds=TheView.GetBounds();
		
		var RefLeft=TheViewBounds.MinX-ImageMinX;
		var RefRight=TheViewBounds.MaxX-ImageMaxX;
		
		var Result=this.GetPixelFromRef(RefX,RefY);
		var XInPixels1=Math.round(Result.PixelX);
		var YInPixels1=Math.round(Result.PixelY);
		
		var PixelWidth=this.GetPixelWidthFromRefWidth(RefWidth);
		var PixelHeight=this.GetPixelHeightFromRefHeight(RefHeight);
		
		PixelWidth=Math.round(PixelWidth);
		PixelHeight=Math.round(PixelHeight);
*/	
		// get the raster bounds
		var RasterBounds=this.GetBounds();
		
		var XMin=RasterBounds.XMin;
		var XMax=RasterBounds.XMax;
		var YMin=RasterBounds.YMin;
		var YMax=RasterBounds.YMax;
		
		// assume we are drawing the entire raster
		var RasterPixelX=0;
		var RasterPixelY=0;
		
		var RasterRefWidth=RasterBounds.XMax-RasterBounds.XMin; 
		var RasterRefHeight=RasterBounds.YMin-RasterBounds.YMax; // ref height is negative
		
		// setup the image data to match the raster exactly
		var ImageDataHeightInPixels=this.HeightInPixels;
		var ImageDataWidthInPixels=this.WidthInPixels;
		
		//***************************************************************
		// find the location of the raster in the view (upper left pixel)
		var ViewBounds=TheView.GetBounds();
		
		// assume we are drawing from the upper left corner of the raster
		var ViewPixelX=TheView.GetPixelXFromRefX(XMin);
		var ViewPixelY=TheView.GetPixelYFromRefY(YMax);
		
		if (XMin<ViewBounds.XMin) // raster is off the left of the view
		{
			XMin=ViewBounds.XMin;
			ViewPixelX=0;
			RasterPixelX=this.GetPixelXFromRefX(XMin);
		}
		if (YMax>ViewBounds.YMax) // raster is off the top of the view
		{
			YMax=ViewBounds.YMax;
			ViewPixelY=0;
			RasterPixelY=this.GetPixelYFromRefY(YMax);
		}
		if (ViewBounds.XMax<XMax) // raster is off the right of the view
		{
			XMax=ViewBounds.XMax;
		}
		if (ViewBounds.YMin>YMin) // raster is off the bottom of the view
		{
			YMin=ViewBounds.YMin;
		}
		// find the ref width for the area that is common to the raster and the view
		var RefWidth=XMax-XMin;
		var RefHeight=YMin-YMax;
		
		// find the dimensions for the raster
		var RasterDataWidthInPixels=this.GetPixelWidthFromRefWidth(RefWidth);
		var RasterDataHeightInPixels=this.GetPixelHeightFromRefHeight(RefHeight);
		
		var ImageDataWidthInPixels=TheView.GetPixelWidthFromRefWidth(RefWidth);
		var ImageDataHeightInPixels=TheView.GetPixelHeightFromRefHeight(RefHeight);
		
		var SampleRateX=RasterDataWidthInPixels/ImageDataWidthInPixels;
		var SampleRateY=RasterDataHeightInPixels/ImageDataHeightInPixels;
		
		// make sure all hte values are intgers
		
		RasterPixelX=Math.round(RasterPixelX);
		RasterPixelY=Math.round(RasterPixelY);
		
		ImageDataHeightInPixels=Math.round(ImageDataHeightInPixels);
		ImageDataWidthInPixels=Math.round(ImageDataWidthInPixels);
		
		if ((ImageDataWidthInPixels>0)&&(ImageDataHeightInPixels>0))
		{
			// get the original data from the canvas
			TheImageData=TheContext.getImageData(ViewPixelX,ViewPixelY,ImageDataWidthInPixels,ImageDataHeightInPixels);
			
			//TheImageData=TheContext.createImageData(this.WidthInPixels,this.HeightInPixels);
			
			var Factor=256/(this.MaxPixelValues-this.MinPixelValues);
			
			var TheMask=this.TheGeoJSONObject.Mask;
			
			var ImageIndex=0;
			for (var y=0;y<ImageDataHeightInPixels;y++)
			{
				var RasterY=RasterPixelY+y*SampleRateY;
				RasterY=Math.round(RasterY);
				
				if (RasterY<0) RasterY=0;
				if (RasterY>=this.HeightInPixels) RasterY=this.HeightInPixels-1;
				
				for (var x=0;x<ImageDataWidthInPixels;x++)
				{
					var RasterX=RasterPixelX+x*SampleRateX;
					RasterX=Math.round(RasterX);
						
					if (RasterX<0) RasterX=0;
					if (RasterX>=this.WidthInPixels) RasterX=this.WidthInPixels-1;
				
					// setup the mask value
					var MaskValue=255;
					
					if (TheMask!=null) 
					{
						try
						{
							MaskValue=TheMask[RasterY][RasterX];
						}
						catch (error)
						{
							throw(error);
						}
					}
					// if not transparent, just copy the pixel values into the canvas
					if (MaskValue!=0)
					{
						var Value=this.TheGeoJSONObject.Data[RasterY][RasterX];
						
						// scale the value to go from 0 to 255
						Value=(Value-this.MinPixelValues)*Factor;
						
						Value=Math.round(Value);
						
						if (Value<0) Value=0;
						if (Value>255) Value=255;
					
						// if there is no mask, replace the pixels in the view with the raster pixel
						if (MaskValue==255)
						{
							TheImageData.data[ImageIndex+0]=Value;
							TheImageData.data[ImageIndex+1]=Value;
							TheImageData.data[ImageIndex+2]=Value;
						}
						else // there is some transparency
						{
							var Red=TheImageData.data[ImageIndex+0];
							var Green=TheImageData.data[ImageIndex+1];
							var Blue=TheImageData.data[ImageIndex+2];
							
							Red=(Red/255.0*(255-MaskValue))+(Value/255.0*MaskValue);
							if (Red<0) Red=0;
							if (Red>255) Red=255;
							
							Green=(Green/255.0*(255-MaskValue))+(Value/255.0*MaskValue);
							if (Green<0) Green=0;
							if (Green>255) Green=255;
							
							Blue=(Blue/255.0*(255-MaskValue))+(Value/255.0*MaskValue);
							if (Blue<0) Blue=0;
							if (Blue>255) Blue=255;
					
							
							TheImageData.data[ImageIndex+0]=Math.round(Red);
							TheImageData.data[ImageIndex+1]=Math.round(Green);
							TheImageData.data[ImageIndex+2]=Math.round(Blue);
							
							// don't change the transparency as this is the transparency for the canvas element!
							//TheImageData.data[ImageIndex+3]=MaskValue; 
						}
					}
					ImageIndex+=4;
				}
			}
			if (TheImageData!=null)
			{
				//TheContext.clearRect(ViewPixelX,ViewPixelY, TheImageData.width, TheImageData.height);
				
				TheContext.putImageData(TheImageData,ViewPixelX,ViewPixelY);
			}
		}
		return(TheImageData);
	}
}
//***************************************************************************
// Public CMDatasetRaster functions
//*************************************************************************

/*
* Returns the value of a single pixel.  
* @public
*/
CMDatasetRaster.prototype.GetSampleFromRef=function(RefX,RefY,Band) 
{
	var Result=undefined;
	
	if (this.TheGeoJSONObject!=undefined)
	{
		if (Band==undefined) Band=0;
		
		var PixelX=Math.floor(this.GetPixelXFromRefX(RefX));
		var PixelY=Math.floor(this.GetPixelYFromRefY(RefY));
		
		if ((PixelX>=0)&&(PixelX<this.WidthInPixels)&&(PixelY>=0)&&(PixelY<this.HeightInPixels))
		{
			Result=this.TheGeoJSONObject.Data[PixelY][PixelX];
		}
	}
	return(Result);
}
/**
* returns an array of THREE.Vector3 objects with the z values set from the 
* associated DEM.  New points will be added to the array when the line segments
* cross a vertical or horizontal boundary between pixels.
* 
* @public
*/
CMDatasetRaster.prototype.GetPathArray=function(Xs,Ys,Zs,ZOffset,ZMultiplier) 
{
	if (ZOffset==undefined) ZOffset=0;
	if (ZMultiplier==undefined) ZMultiplier=1;
	
	var PathArray=[];
	
	if (this.TheMatrix==undefined)
	{
		for (var j=0;j<Xs.length;j++)
		{
			var RefX=Xs[j];
			var RefY=Ys[j];
			var RefZ=0;
			
			if (Zs!=null) // get the z value from the points
			{
				RefZ=Zs[j];
			}
			else // see if there is a DEM for a z value
			{
				var DEM_ZOffset=this.GetSampleFromRef(RefX,RefY);
				
				if (DEM_ZOffset!=undefined) RefZ=DEM_ZOffset;
			}
			// apply the mulitplier and offset
			RefZ=RefZ*ZMultiplier+ZOffset;
			
			PathArray.push(new THREE.Vector3(RefX,RefY,RefZ));
		}
	}
	else
	{
		// new approach to find coordinates at each break between the cells of the raster
		var TheData=this.TheMatrix;
		
		var PixelWidth=this.WidthInPixels/TheData.length;
		var PixelHeight=this.HeightInPixels/TheData[0].length;
		
		var StartRefX=0;
		var StartRefY=0;
		
		// setup the first coordinate which is always at the start of the polyline
		for (var j=0;j<Xs.length;j++)
		{
			var EndRefX=Xs[j];
			var EndRefY=Ys[j];
			
			// 
			
			if ((j!=0)&&(false)) // have another coordinate
			{
				var RowInc=1;
				var ColInc=1;
				
				if (StartRefX<EndRefX) // going left to right
				{
					if (StartRefY<EndRefY) // going from left up to the right
					{
					}
					else // going from left down to the right
					{
						RowInc=-1;
					}
				}
				else // going right to left
				{
					ColInc=-1;
					
					if (StartRefY<EndRefY) // going from right up to the left
					{
					}
					else // going from right down to the left
					{
						RowInc=-1;
					}
					
				}
				//*********************************************
				// find the start row and column
				
				// assume y is increasing with x
				var StartRow=Math.ceil((StartRefY-this.TheBounds.YMin)/PixelHeight);
				var EndRow=Math.floor((EndRefY-this.TheBounds.YMin)/PixelHeight);
				
				if (RowInc<0) // y decreasing with x increasing
				{
					StartRow=Math.floor((StartRefY-this.TheBounds.YMin)/PixelHeight);
					EndRow=Math.ceil((EndRefY-this.TheBounds.YMin)/PixelHeight);
				}
				
				var StartRowY=(StartRow*PixelHeight)+this.TheBounds.YMin;
				var EndRowY=(EndRow*PixelHeight)+this.TheBounds.YMin;
				
				// skip the first and last row if they are the same as the start and/or last
				if (StartRowY==StartRefY) StartRow+=RowInc;
				if (EndRowY==EndRefY) EndRow-=RowInc;
				
				var Result=CMUtilities.GetLineFactors( StartRefX, StartRefY, EndRefX, EndRefY);
				var m=Result[0];
				var b=Result[1];
				
				// 
				var LastIndex=Xs.length-1;
				var NumRows=EndRow-StartRow+1;
				
				for (var RowCount=0;RowCount<NumRows;RowCount++)
				{
					// find the row number in the raster
					var Row=StartRow+(RowInc*RowCount);
					
					// find the ref of the top of the row
					var RowRefTopY=this.TheBounds.YMin+(Row*PixelHeight);
					
					// find the intersection with the top of this row
					
					var RowRefTopIntersectionX=(RowRefTopY-b)/m;
					
					EndRefZ=this.GetSampleFromRef(RowRefTopIntersectionX,RowRefTopY)*ZMultiplier+ZOffset;
					PathArray.push(new THREE.Vector3(RowRefTopIntersectionX,RowRefTopY,EndRefZ));
					
					var StartCol=Math.floor((StartRefX-this.TheBounds.XMin)/PixelWidth);
					var ColRemainder=RefX-(StartCol*PixelWidth);
				}
			}
			// setup the coordinate's z value
			var EndRefZ=0;
			if (Zs!=null) EndRefZ=Zs[j];
			else 
			{
				EndRefZ=this.GetSampleFromRef(EndRefX,EndRefY);
				if (EndRefZ==undefined) EndRefZ=0;
			}
			EndRefZ=EndRefZ*ZMultiplier+ZOffset;
			PathArray.push(new THREE.Vector3(EndRefX,EndRefY,EndRefZ));
			
			StartRefX=EndRefX;
			StartRefY=EndRefY;
		}
	}
	return(PathArray);
}
		