/******************************************************************************************************************
* CMItem
* An item is something that is displayed and typically can be edited
* by a user.  Items includeobjects such as arrows, boxes, labels, and ovals.
* A CMLayer (geospatial layer) is a subclass of this class.
*
* Adds:
* - Tweening
* - Styles, Painting
* - Visilbity
*
* @module CMDatasetPyramidOpenFormat
* @Copyright HSU, Jim Graham, 2019
******************************************************************************************************************/
//******************************************************************
// Definitions
//******************************************************************
 
CMItem.STATUS_HIDDEN=0;
CMItem.STATUS_VISIBLE=1;
CMItem.STATUS_SELECTABLE=2;
CMItem.STATUS_EDITABLE=3;
CMItem.StatusNames=["Hidden","Visible","Selectable","Editable"]; // jjg - need to resolve ints versus names for enumberated settings

/**
* Below are the settings definitions.
* @public, @settings
*/
CMItem.SettingDefintions=
{
	Item:
	{
		Name: { Name:"Name",Type:CMBase.DATA_TYPE_STRING, Default:"" }, // Name that appears in the item and layer lists
		
		Status: { Name:"Status",Type:CMBase.DATA_TYPE_ENUMERATED, Options:[CMItem.STATUS_HIDDEN,CMItem.STATUS_VISIBLE,CMItem.STATUS_SELECTABLE,CMItem.STATUS_EDITABLE],Default:CMItem.STATUS_SELECTABLE },// How the item appears to the user and how the user can interact with it
		
		// jjg - should this be in the view?
		PixelTolerance: { Name:"Pixel Tolerance",Type:CMBase.DATA_TYPE_FLOAT, Default:6 }, // distance from a point or polyline the user needs to click to select it.
		PositionOffset: { Name:"Position Offset Vector",Type:CMBase.DATA_TYPE_VECTOR, Default:null }, // x,y, and z values to offset the item
		Rotation: { Name:"Rotation Vector",Type:CMBase.DATA_TYPE_VECTOR, Default:null }, // pitch, roll, and yaw values (x,y,z) values to roatate the item in degrees
		Scale: { Name:"Scaling Vector",Type:CMBase.DATA_TYPE_VECTOR, Default:null }, // x,y, and z scaling values for the item's size
	},
	Style:
	{
		// standard HTML 5 settings except the defaults may change and sometimes the available settings will change between each settings group
		strokeStyle: { Name:"Line Style",Type:CMBase.DATA_TYPE_CSS_STYLE, Default:"rgb(0,0,0)" }, // Style to draw lines (HTML Canvas)
		lineWidth: { Name:"Line Width",Type:CMBase.DATA_TYPE_INTEGER, Default:1 }, // Pixel width of lines (HTML Canvas)
		lineCap: { Name:"Line Cap",Type:CMBase.DATA_TYPE_ENUMERATED, Options:["butt","round","square"],Default:"round" },// How to end lines (HTML Canvas)
		lineJoin: { Name:"Line Join",Type:CMBase.DATA_TYPE_ENUMERATED, Options:["bevel","round","miter"],Default:"round" }, // How to round line connections (HTML Canvas)
		fillStyle: { Name:"Fill Style",Type:CMBase.DATA_TYPE_CSS_STYLE, Default:"rgb(255,255,255)" }, // How to fill polygons (HTML Canvas)
		shadowColor: { Name:"Shadow Color",Type:CMBase.DATA_TYPE_COLOR, Default:"rgb(0,0,0)" }, // Color for shadows (HTML Canvas)
		shadowBlur: { Name:"Shadow Blur",Type:CMBase.DATA_TYPE_FLOAT, Default:1 }, // Pixels distance to blur the shadow (HTML Canvas)
		shadowOffsetX: { Name:"Shadow X",Type:CMBase.DATA_TYPE_FLOAT, Default:1 }, // Horizontal pixel distance to move shadow (HTML Canvas)
		shadowOffsetY: { Name:"Shadow Y",Type:CMBase.DATA_TYPE_FLOAT, Default:1 }, // Vertical pixel distance to move shadow (HTML Canvas)
		
		PatternImage: { Name:"Pattern Images",Type:CMBase.DATA_TYPE_IMAGE, Default:null }, // Replaces the fill style with an image (HTML Canvas)
		GradientType: { Name:"Gradient Type",Type:CMBase.DATA_TYPE_ENUMERATED, Options:["Linear","Radial"],Default:"Radial" }, // Type of gradient fill, requires coordinates and colors (HTML Canvas)
		GradientCoordinates: { Name:"Gradient Coordinates",Type:CMBase.DATA_TYPE_COORDINATES, Default:null }, // Pixel coordinates for the gradient (HTML Canvas)
		GradientRadius1: { Name:"Gradient Radius 1",Type:CMBase.DATA_TYPE_FLOAT, Default:0 }, // Pixel radius for radia gradient (HTML Canvas)
		GradientRadius2: { Name:"Gradient Radius 2",Type:CMBase.DATA_TYPE_FLOAT, Default:100 }, // Pixel radius for radia gradient (HTML Canvas)
		GradientColors: { Name:"Gradient Colors",Type:CMBase.DATA_TYPE_COLOR_ARRAY, Default:null }, // Colors for each gradient stop (HTML Canvas)
		
		globalAlpha: { Name:"Transparency",Type:CMBase.DATA_TYPE_FLOAT, Default:1 }, // 0 for transparent, 1 for opaque, between for translucent (HTML Canvas)
		globalCompositeOperation: { Name:"Composite Operation",Type:CMBase.DATA_TYPE_ENUMERATED, Options:["source-over","source-atop","source-in","source-out","destination-over","destination-atop","destination-in","destination-out","lighter","copy","xor"],Default:"source-over" }, // Determines how items are drawn on top of one another (HTML Canvas)
	},
	Text:
	{
		Text: { Name:"Text",Type:CMBase.DATA_TYPE_STRING, Default:null }, // text that will appear in the item
		Visible: { Name:"Text Visible",Type:CMBase.DATA_TYPE_BOOLEAN, Default:true }, // true to make theitem visible, false for hidden
//		DimensionsInPixels: { Name:"Dimensions in Pixel Units",Type:CMBase.DATA_TYPE_BOOLEAN, Default:true }, // true to have the font size and other dimensions be in pixel units, false for map-reference units
		
		font: { Name:"Font",Type:CMBase.DATA_TYPE_FONT, Default:"12px Arial" }, // CSS font metrics for the text (all are optional but order is required to be: "style variant weight size family" (e.g. "italic small-caps bold 12px arial")
		
		strokeStyle: { Name:"Line Style",Type:CMBase.DATA_TYPE_CSS_STYLE, Default:"rgb(0,0,0)" }, // color for the outline of the characters
		lineWidth: { Name:"Width",Type:CMBase.DATA_TYPE_INTEGER, Default:1 }, // width of the outline of the characters
		lineCap: { Name:"Line Cap",Type:CMBase.DATA_TYPE_ENUMERATED, Options:["butt","round","square"],Default:"round" }, // jjg - text does not have open line endings so remove this?
		lineJoin: { Name:"Line Join",Type:CMBase.DATA_TYPE_ENUMERATED, Options:["bevel","round","miter"],Default:"round" }, // how the line segments in the outlines of the characters join together
		fillStyle: { Name:"Fill Style",Type:CMBase.DATA_TYPE_CSS_STYLE, Default:"rgb(255,255,255)" }, // fill color for the characters
		shadowColor: { Name:"Shadow Color",Type:CMBase.DATA_TYPE_COLOR, Default:"rgb(0,0,0)" },
		shadowBlur: { Name:"Shadow Blur",Type:CMBase.DATA_TYPE_FLOAT, Default:1 },
		shadowOffsetX: { Name:"Shadow X",Type:CMBase.DATA_TYPE_FLOAT, Default:1 },
		shadowOffsetY: { Name:"Shadow Y",Type:CMBase.DATA_TYPE_FLOAT, Default:1 },
		
		globalAlpha: { Name:"Transparency",Type:CMBase.DATA_TYPE_FLOAT, Default:1 }, // 0 for transparent, 1 for opaque, between for translucent (HTML Canvas)
		globalCompositeOperation: { Name:"Composite Operation",Type:CMBase.DATA_TYPE_ENUMERATED, Options:["source-over","source-atop","source-in","source-out","destination-over","destination-atop","destination-in","destination-out","lighter","copy","xor"],Default:"source-over" }, // Determines how items are drawn on top of one another (HTML Canvas)
	},
};
CMItem.UniqueNumber=0;

CMItem.MESSAGE_CONTENT_CHANGED=CMBase.GetUniqueNumber();

//******************************************************************
// Constructor
//******************************************************************
/**
* Creates a new CMItem object.  This is an abstract class.
* @protected, @constructs
*/
function CMItem() 
{
	CMBase.call(this);
	
	// Properties
	this.UniqueNumber=CMItem.UniqueNumber;
	
	// CMBase does not have settings so this is the main property for settings
	this.TimeSlices=[
	{
	 	UniqueNumber:CMItem.UniqueNumber,
		Time:0,
		Settings:
		{
			Style:
			{
				strokeStyle:"rgb(0,0,0)",
				lineWidth:1,
				fillStyle:"rgb(255,255,255)"
			},
			Text:
			{
				strokeStyle:"rgb(255,255,255)",
				lineWidth:1,
				fillStyle:"rgb(0,0,0)"
			},
		}
	}];
	
	CMItem.UniqueNumber++;
	 
	this.LastStyle=null;
	this.LastTimeSlice=0;
}
CMItem.prototype=Object.create(CMBase.prototype); // inherit prototype functions from PanelBase()

CMItem.prototype.contructor=CMItem; // override the constructor to go to ours

//******************************************************************
// CMBase Settings Functions
//******************************************************************
/**
* Called when any settings are changed.
* @protected
*/
CMItem.prototype.SettingsChanged=function() 
{
	this.LastStyle=null; // force style to be recomputed
	
	this.Repaint();
}
/**
* Called to get the settings definitions. Subclasses should override to add their own settings groups.
* @protected, @override
*/

CMItem.prototype.GetSettingsDefinitions=function() 
{
	var Result={};
	
	for (Key in CMItem.SettingDefintions)
	{
		Result[Key]=CMItem.SettingDefintions[Key];
	}

	return(Result); 
}
/**
* Get a set of settings.  The TimeSlice must already exist within the object.
* This adds the TimeSlice parameter to the overriden CMBase.SetSettings() function.
* @public
* @param TimeSlice - optional value to set a property to transition at a specific time.
* @returns Settings - current settings object
*/
CMItem.prototype.GetSettings=function(TimeSlice) 
{
	var Result=null;
	
	var TimeSliceIndex=CMItem.FindTimeSliceIndex(this.TimeSlices,TimeSlice);
	
	if (TimeSliceIndex!=-1)
	{
		Result=this.TimeSlices[TimeSliceIndex].Settings;
	}
	return(Result);
}
/**
* Sets a set of settings for this object.  The TimeSlice must already exist within the object.
* This adds the TimeSlice parameter to the overriden CMBase.SetSettings() function.
* @public
* @param NewSettings - The settings to replace the existing settings.
* @param TimeSlice - Which time slice to set
*/
CMItem.prototype.SetSettings=function(NewSettings,TimeSlice) 
{
	var TimeSliceIndex=0;
	
	if (TimeSlice!=undefined) TimeSliceIndex=CMItem.FindTimeSliceIndex(this.TimeSlices,TimeSlice);
	
	if (TimeSliceIndex!=-1)
	{
		var TheSettings=this.TimeSlices[TimeSliceIndex].Settings;
		
		for (var GroupKey in NewSettings) // for each group the user is try to set settings for
		{
			var GroupSettings=this.GetGroupFromSettings(GroupKey,TheSettings);
/*			var GroupSettings=TheSettings[GroupKey];
			
			if (GroupSettings==undefined) // if the group is not in the current settings, see if the group is defined
			{
				var SettingsDefinitions=this.GetSettingsDefinitions();
				
				var GroupDefinitions=SettingsDefinitions[GroupKey];
				
				if (GroupDefinitions!=undefined) // if the group is defined, try to set the values with the keys
				{
					GroupSettings={};
					TheSettings[GroupKey]=GroupSettings;
				}
				else this.SettingsError(GroupKey);
			}
*/			// at this point we have a GroupSettings object or have generated an error
			var NewGroup=NewSettings[GroupKey];
			
			for (var SettingKey in NewGroup)
			{
				TheSettings[GroupKey][SettingKey]=NewGroup[SettingKey];
			}
		}
	}
	this.SettingsChanged();
}

/**
* Sets an individual setting value into the settings.  The only change from the CMBsae function is the addition of the TimeSlice parameter.
* @public
* @param Group - Group for the setting
* @param Key - on of the CMLayer.INFO enumerated types
* @param Value - value for the type (see the documentation for types for each of the properties)
*/
CMItem.prototype.SetSetting=function(Group,Key,Value,TimeSlice)
{
	// put the settings into a dictionary entry
	var TheSettings={};
	TheSettings[Group]={};
	TheSettings[Group][Key]=Value;
	
	// all settings go through SetSettings() // jjg - changes in the future
	this.SetSettings(TheSettings,TimeSlice);
	
	if ((Group=="Item")&&(Key=="Status"))
	{
		this.Repaint();
/*		if (this.GetSetting(Group,Key,TimeSlice)!=Value)
		{
//			if (Value==CMItem.STATUS_HIDDEN) this.SetVisible(false);
//			else this.SetVisible(true);
		}
*/	}
			
}
/**
* Get an individual value from the settings
* @public
* @param Group - Group for the setting
* @param Key - on of the CMLayer.INFO enumerated types
* @param Default - default value to use in none has been specified as of yet (optional)
* @returns Value - curnet property value.
*/
CMItem.prototype.GetSetting=function(Group,Key,Default,TimeSlice)
{
	var Result=null; //this.SettingDefintions[Group][Key].Default;
	
	if (Default!=undefined) Result=Default;
	
	var TimeSliceIndex=0;
	
	if (TimeSlice!=undefined) TimeSliceIndex=CMItem.FindTimeSliceIndex(this.TimeSlices,TimeSlice);
	
	if (TimeSliceIndex!=-1)
	{
		if (this.TimeSlices!=undefined)
		{
			// try to get the settings from the current object settings
			var GroupSettings=this.TimeSlices[TimeSliceIndex].Settings[Group];
			
			if (GroupSettings!=undefined) // group has not been defined yet
			{
				if (Key in GroupSettings) Result=GroupSettings[Key];
			}
			// if we could not find a setting value and the default was not provided, get the default from the definitions
			if (Result==null) 
			{
				var SettingsDefinitions=this.GetSettingsDefinitions();
				
				if (Group in SettingsDefinitions)
				{
					var GroupDefinitions=SettingsDefinitions[Group];
					
					if (Key in GroupDefinitions) 
					{
						Result=GroupDefinitions[Key].Default;
					}
					else this.SettingsError(Group,Key);
				}
				else this.SettingsError(Group);
			}
		}
	}
	return(Result);
}
//******************************************************************
// CMItem Settings Functions
//******************************************************************
/**
* Set a group of settings
* @public
* @param Group - Group for the setting
* @param Values - Object with groups settings
* @param TimeSlice - Optional time slice for where to insert the group
*/
CMItem.prototype.SetSettingGroup=function(Group,Values,TimeSlice)
{
	var TimeSliceIndex=0;
	
	if (TimeSlice!=undefined) TimeSliceIndex=CMItem.FindTimeSliceIndex(this.TimeSlices,TimeSlice);
	
	if (TimeSliceIndex!=-1)
	{
		this.TimeSlices[TimeSliceIndex].Settings[Group]=Values; // save them into the object
	}
	this.SettingsChanged();
}
/**
* Get a group of settings
* @public
* @param Group - Group for the setting
* @param Default - default value to use in none has been specified as of yet (optional)
* @returns Value - curnet property value.
*/
CMItem.prototype.GetSettingGroup=function(Group,Default,TimeSlice)
{
	var Result=null;
	if (Default!=undefined) Result=Default;
	
	var TimeSliceIndex=0;
	
	if (TimeSlice!=undefined) TimeSliceIndex=CMItem.FindTimeSliceIndex(this.TimeSlices,TimeSlice);
	
	if (TimeSliceIndex!=-1)
	{
		Result=this.TimeSlices[TimeSliceIndex].Settings[Group];
	}
	return(Result);
}
//******************************************************************
// Functions to select and unselect content
//******************************************************************
/**
* Subclasses should override this function to unselect any selected content
* @protected - only used within CanvasMap 
*/
CMItem.prototype.UnselectAll=function(SendMessageFlag) 
{
	if (this.Selected) // unselect this item
	{
		this.Selected=false;
		if (this.LayerInList!=undefined)
		{
			this.LayerInList.className="CM_LayerListItemClass";
		}
		
		if (SendMessageFlag) // call the scene to let everyone know the selection changed
		{
			var TheScene=this.GetParent(CMScene);
			TheScene.SelectionChanged(this);
		}
	}
}

//***********************************************************************************************
// Private Static Functions for managing time slices
//***********************************************************************************************
/**
* Finds an index to a specific time slice value
* @private
*/
CMItem.FindTimeSliceIndex=function(TimeSliceSettings,TimeSlice)
{
	var Result=-1;
	
	if (TimeSlice==undefined) // return the first entry
	{
		if (TimeSliceSettings.length>0) Result=0; // have at least one entry
	}
	else
	{
		for (var i=0;(i<TimeSliceSettings.length)&&(Result==-1);i++)
		{
			var TimeSliceSetting=TimeSliceSettings[i];
			
			if (TimeSliceSetting.Time==TimeSlice) Result=i;
		}
	}
	return(Result);
}
//***********************************************************************************************
// Protected functions for managing time slices
//***********************************************************************************************

/**
* Returns the time slices
* @protected
* @TheTimeSlices - array to add this objects time slices to
* @returns TheTimeSlices - the same array but with the time slices added.
*/
CMItem.prototype.GetTimes=function(TheTimeSlices)  
{ 
	TheTimeSlices=[];

	for (var i=0;i<this.TimeSlices.length;i++)
	{	
		var Time=this.TimeSlices[i].Time;
		
		CMUtilities.InsertIntoSortedArray(TheTimeSlices,parseFloat(Time));
	}
	return(TheTimeSlices); 
}
/**
* Adds a new time slice
* @protected
* @Time - the time to add
* @returns Index - index to the new time slice
*/
CMItem.prototype.InsertTime=function(Time)
{
	var TimeSlices=this.TimeSlices;
	
	var Index=-1;
	for (var i=0;i<TimeSlices.length;i++)
	{
		if (TimeSlices[i].Time==Time) // TimeSlice is already in the settings
		{
			Index=i; // return the index to the existing slice
		}
		else if (TimeSlices[i].Time>Time) // we are past the desired TimeSlice, insert the new one
		{
			Index=i;
			TimeSlices.splice(Index,0,Time);
		}
	}
	if (Index==-1) // missed the TimeSlice, add it at the end
	{
		Index=TimeSlices.length;
		TimeSlices.push({Settings:{},Time:Time});
	}
	if (Index!=-1) // inserted a TimeSlice at Index
	{
		var OldTimeSliceSettings=TimeSlices[Index-1].Settings;
		var NewTimeSliceSettings=TimeSlices[Index].Settings;
		
		for (ClassKey in OldTimeSliceSettings)
		{
			var TheSetting=OldTimeSliceSettings[ClassKey];
			
			TheSetting=CMUtilities.Clone(TheSetting);
			
			NewTimeSliceSettings[ClassKey]=TheSetting;
		}
	}
	this.Repaint();
	
	return(Index);
}
/**
* Removes a time slice
* @protected
* @Time - the time to delete
* @returns Index - index to the deleted time slice
*/
CMItem.prototype.DeleteTime=function(Time)
{
	var TimeSlices=this.TimeSlices;
	
	var Index=-1;
	for (var i=0;i<TimeSlices.length;i++)
	{
		if (TimeSlices[i].Time==Time) 
		{
			Index=i;
			TimeSlices.splice(Index,1); // remove the time slice
		}
	}
	this.Repaint();
	return(Index);
}
//******************************************************************
// Private Utility functions to aid with managing time slices
//******************************************************************
/**
* Get a tweened value between two values
* @private
* @param Value 1 - minimum value
* @param Value2 - maxumum value
* @param Default - default to use if neither Value1 or Value2 are defnied
* @param Factor - interpolation factor (1 for Value1, 0 for Value2, 0 to 1 is an interpolated value between the two
*/
CMItem.GetTweenFloatValue=function(Value1,Value2,Default,Factor) 
{
	var Result=Default;
	
	if (Value1!=undefined) // have value 1
	{
		if (Value2!=undefined) // also have value 2, do tweening
		{
			Result=(Value1*Factor)+(Value2*(1-Factor));
		}
		else // just have value1, use it
		{
			Result=Value1;
		}
	}
	else if (Value2!=undefined) // just have value 2, use it
	{
		Result=Value2;
	}
	return(Result);
}
/**
* Get a tweened value between two color values
* @private
* @param Value 1 - minimum value
* @param Value2 - maxumum value
* @param Default - default to use if neither Value1 or Value2 are defnied
* @param Factor - interpolation factor (1 for Value1, 0 for Value2, 0 to 1 is an interpolated value between the two
*/
CMItem.GetTweenColorValue=function(Value1,Value2,Default,Factor) 
{
	var Result=Default;
	
	var Colors=null;
	
	if (Value1!=undefined) // have value 1
	{
		var Colors1=CMUtilities.GetColorsFromAnyColor(Value1);
		
		if (Value2!=undefined) // also have value 2, do tweening
		{
			var Colors2=CMUtilities.GetColorsFromAnyColor(Value2);
			
			Colors={
				Red:Math.round((Colors1.Red*Factor)+(Colors2.Red*(1-Factor))),
				Green:Math.round((Colors1.Green*Factor)+(Colors2.Green*(1-Factor))),
				Blue:Math.round((Colors1.Blue*Factor)+(Colors2.Blue*(1-Factor)))
			}
		}
		else // just have value1, use it
		{
			Colors=Value1;
		}
	}
	else if (Value2!=undefined) // just have value 2, use it
	{
		Colors=Value2;
	}
	if (Colors!=null) Result="rgb("+Colors.Red+","+Colors.Green+","+Colors.Blue+")";
	
	return(Result);
}
/**
* Get a tweened value between two color values
*
* jjg - does not currently support gradients or patterns
*
* @private
* @param Value 1 - minimum value
* @param Value2 - maxumum value
* @param Default - default to use if neither Value1 or Value2 are defnied
* @param Factor - interpolation factor (1 for Value1, 0 for Value2, 0 to 1 is an interpolated value between the two
*/
CMItem.GetTweenStyleValue=function(Value1,Value2,Default,Factor) 
{
	var Result=CMItem.GetTweenColorValue(Value1,Value2,Default,Factor) ;

	return(Result);
}
//******************************************************************
// Protected functions for managing time slices
//******************************************************************
/**
* Removes a time slice
* @protected
* @Time - the time to delete
* @returns Index - index to the deleted time slice
*/
CMItem.GetTimeFactor=function(TimeSliceArray)
{
	var TimeSlice1=TimeSliceArray[0].Time;
	var TimeSlice2=TimeSliceArray[1].Time;
	
	var Range=TimeSlice2-TimeSlice1;
	var Factor=(Range-TimeSlice)/Range; // 0 at end, 1 at start (jjg - Where is TimeSlice defined?)
	
	return(Factor);
}


/**
* Find the appropriate time slices for the specified TimeSlice
* @protected - used by subclasses to find the time slice settings to use
*/
CMItem.prototype.GetBoundingTimeSlices=function(TimeSlice)
{
	if (TimeSlice==undefined)
	{
		var TheScene=this.GetParent(CMScene);
	
		TimeSlice=TheScene.GetTimeRange();
	}
	// find the settings to use for the style, either Settings1 or both Settings1 and Settings2 for tweening
	
	var TheTimeSliceSetting1=null;
	var TheTimeSliceSetting2=null;
//	var TheKeys=Object.keys(this.TimeSlices);
	
	for (var i=0;(i<this.TimeSlices.length)&&(TheTimeSliceSetting1==null);i++)
	{
		// get the key and make sure it is a number
		var ThisSlicesTime=parseFloat(this.TimeSlices[i].Time);
		
		if (ThisSlicesTime==TimeSlice) // current key exactly matches the desired time slice
		{
			TheTimeSliceSetting1=this.TimeSlices[i];
		}
		else if (ThisSlicesTime>TimeSlice) // current key is after desired time slice
		{
			if (i==0) // first key is after the desired time slice, use just the first key
			{
				TheTimeSliceSetting1=this.TimeSlices[0];
			}
			else // time slice is between two keys
			{
				TheTimeSliceSetting1=this.TimeSlices[i-1];
				TheTimeSliceSetting2=this.TimeSlices[i];
			}
		}
	}
	
	// if we did not find a match and there are timeslices, the desired time slice must be after the last key
	if ((TheTimeSliceSetting1==null)&&(this.TimeSlices.length>0))
	{
		TheTimeSliceSetting1=this.TimeSlices[this.TimeSlices.length-1];
	}
	
	// return the result as an array for fast access
	var Result=[TheTimeSliceSetting1,TheTimeSliceSetting2];
	
	return(Result);
}
/**
* Returns a tweened style based on the specified TimeSlice
* @protected
*/
CMItem.prototype.GetStyle=function(TheView,TimeSlice,Group) 
{
	if (TimeSlice==undefined) TimeSlice=0;

	if (Group==undefined) Group="Style";
	
//	if ((this.LastStyle==null)||(this.LastTimeSlice!=TimeSlice))
	{
		this.LastStyle=null;
		this.LastTimeSlice=TimeSlice;
		
		var Result=this.GetBoundingTimeSlices(TimeSlice);
		var TheTimeSlice1=Result[0];
		var TheTimeSliceSetting1=TheTimeSlice1.Settings;
		var TheTimeSlice2=Result[1];

		//*********************************************************************************************************
		// create the style to return
		
		if (TheTimeSlice1!=null) // have something
		{
			var TheStyle;
			
			if (TheTimeSlice2==null) // no tweening
			{
	//			for (TheGroupKey1 in TheTimeSliceSetting1)
				{
					var TheGroup1=TheTimeSliceSetting1[Group];
					
//					var TheGroup={};
					for (TheSettingKey1 in TheGroup1)
					{
						if (TheStyle===undefined) TheStyle={};
						
						var TheSetting1=TheGroup1[TheSettingKey1];
						
						TheStyle[TheSettingKey1]=TheSetting1;
					}
				}
			}
			else // have tweening
			{
				var TheTimeSliceSetting2=TheTimeSlice2.Settings;
				
				var Factor=CMItem.GetTimeFactor(Result);
				
	//			for (TheGroupKey1 in TheTimeSliceSetting1)
				{
					var TheGroup1=TheTimeSliceSetting1[Group];
					var TheGroup2=TheTimeSliceSetting2[Group];
					
					for (TheSettingKey1 in TheGroup1)
					{
						if (TheStyle===undefined) TheStyle={};
						
						var TheSetting1=TheGroup1[TheSettingKey1];
						var TheSetting2=TheGroup2[TheSettingKey1];
						
						if ((TheSettingKey1=="strokeStyle")||(TheSettingKey1=="fillStyle")) // styles are treated specially
						{
							TheStyle[TheSettingKey1]=CMItem.GetTweenStyleValue(TheSetting1,TheSetting2,"rgb(0,0,0)",Factor);
						}
						else if (TheSettingKey1=="shadowColor") // colors have to have each element tweened
						{
							TheStyle[TheSettingKey1]=CMItem.GetTweenColorValue(TheSetting1,TheSetting2,"rgb(0,0,0)",Factor);
						}
						else // must be a number
						{
							TheStyle[TheSettingKey1]=CMItem.GetTweenFloatValue(TheSetting1,TheSetting2,0,Factor) ;
						}
					}
				}
			}
			// convert ref dimensions to pixel dimensions (jjg - add tweening)
	
			for (key in TheStyle)
			{
				// have to adjust for reference unit dimensions
				if ((key=="lineWidth")||(key=="shadowOffsetX")||(key=="shadowOffsetY")||(key=="shadowBlur"))
				{
					var Test=12;
	//				TheStyle[key]=TheView.GetPixelWidthFromRefWidth(TheStyle[key]); // jjg - need to make this an option
				}
			}
			this.LastStyle=TheStyle;
		}
	}

	return(this.LastStyle);
}
//******************************************************************
// CMItem Settings helper functions
//******************************************************************
CMItem.prototype.GetRefTolerance=function(TheView)
{
	var PixelTolerance=this.GetSetting("Item","PixelTolerance",6);
	
	var RefTolerance=TheView.GetRefWidthFromPixelWidth(PixelTolerance);
	
	return(RefTolerance);
}

//******************************************************************
// Protected functions for managing the layer list
//******************************************************************

/**
* Called by the CMPanelLayerList to add this item to the layer list
*
* For now we assume the layer is changing each time this is called.
* @protected
*/
CMItem.prototype.AddToList=function(TheElement,Left,LayerInListTop,LayerListItemHeight)
{
	//***************************************************************************
	// create the overall div tag for the layer in the list
	
	var LayerInList=null;
	
//	if (this.LayerInList==undefined)
	{
		LayerInList=document.createElement('div');
		LayerInList.className="CM_LayerListItemClass";
		TheElement.appendChild(LayerInList);
		this.LayerInList=LayerInList;
	}
//	else
//	{
//		LayerInList=this.LayerInList;
//	}
	if (this.Selected) this.LayerInList.className="CM_LayerListItemClass_Selected";

	//***************************************************************************
	// add the check box
	
	var TheCheckBox=document.createElement('input');
	TheCheckBox.className="CM_LayerListCheckBoxClass";
	
	TheCheckBox.type="checkbox"
	TheCheckBox.TheLayer=this;
	TheCheckBox.checked=this.GetVisible(); // check if the layer is currently visible
	TheCheckBox.addEventListener('click', function()
	{
		if (this.checked)
		{
			this.TheLayer.SetVisible(true);
		}
		else // unchecked
		{
			this.TheLayer.SetVisible(false);
		}
	});

	CMUtilities.AbsolutePosition(TheCheckBox,Left+2,-6,30,LayerListItemHeight-6);
	
	// Set the position of the check boxes
	LayerInList.appendChild(TheCheckBox);

	//***************************************************************************
	// add the icon
	
	var TheIcon=this.GetIcon();
	
	if (CMUtilities.IsDefined(TheIcon))
	{
		CMUtilities.AbsolutePosition(TheIcon,Left+28,-5,16,16);
	
		// Set the position of the check boxes
	
		LayerInList.appendChild(TheIcon);
	}
	//***************************************************************************
	// add the name
	
	var TheLayerName=document.createElement('div');
	TheLayerName.className="CM_LayerListNameClass";
	
	var TheName=this.GetName();
	if (TheName===null) TheName="Untitled";
	TheLayerName.innerHTML=TheName;
	
	TheLayerName.TheLayer=this;
	TheLayerName.TheElement=TheElement;
	TheLayerName.LayerInList=LayerInList;
	TheLayerName.TheLayerList=this;

	CMUtilities.AbsolutePosition(TheLayerName,Left+48,0,170,LayerListItemHeight);
	
	//***************************************************************************
	// Mouse down to drag the layer
	//***************************************************************************
	
	this.DraggingDiv=null;
	
	document.addEventListener( "contextmenu", function(event) {
		event.preventDefault(); // keeps regular menu from appearing
		return(false); // old way to keep regular menu from appearing (not sure this is needed)
	});

	//***************************************************************************
	// event listener for when the user right clicks on the list
	// - regular click is for moving, right click for opening the menu
	
	TheLayerName.addEventListener('mousedown', function(event)
	{
		this.TheLayer.SetSelected(true);
			
		if (event.button==0) // left mouse button was pressed, move the layer in the list
		{
			// find the offset to the mouse click in the name
			
			this.DraggingDiv=document.getElementById("DraggingDiv");
			
			if (this.DraggingDiv==null)
			{
				this.DraggingDiv=document.createElement('div');
				this.DraggingDiv.className="CM_DraggingDivClass";
				this.DraggingDiv.id="DraggingDiv";
			}
			this.DraggingDiv.style.visibility="visible";
			
			this.TheElement.appendChild(this.DraggingDiv);
			
			this.TheElement.DraggingDiv=DraggingDiv;
			this.TheElement.DraggingLayer=this.TheLayer;
			
			// set the inital position of the div
			
			var TheElementPosition=$(this.TheElement).offset();
			
			CMUtilities.AbsolutePosition(this.DraggingDiv,0,event.clientY-TheElementPosition.top,200,0);
		}
		else // right mouse button was pressed, display the popup menu
		{
			//*******************************************************************
			// Create the popup menu if it has not been created already and remove it's contents
			// we use one LayerPopupMenu element for all layers and 
			// just change it's contents when it is selected
			
			var ThePopupMenu=CMUtilities.GetPopupMenu("CM_LayerPopupMenu",event.clientX,event.clientY);
			
			// Add the popup menu items
			this.TheLayer.FillPopupMenu(ThePopupMenu);
		}
		
		event.stopPropagation(); // stop the document from hidding a popup window
		event.preventDefault(); // keeps regular menu from appearing
		return(false); // old way to keep regular menu from appearing (not sure this is needed)
	});
	TheLayerName.addEventListener('mouseup', function(event)
	{
		event.preventDefault();
		return(false); // old way to keep regular menu from appearing (not sure this is needed)
	});
	
	// 

	LayerInList.appendChild(TheLayerName);
	
//	LayerInList.style.border="2px solid #ff0000"; // for debugging
	
	var LayerListWidth=jQuery(TheElement).outerWidth(false);
	
	CMUtilities.AbsolutePosition(LayerInList,Left+2,LayerInListTop,LayerListWidth,LayerListItemHeight);
	
	// these were all attempts to get the height of the element in the list after it was added
	// to use for spacing out the items and adding the dragging bar.  They all failed and absolute positioning was used instead.
//	var Height=LayerInList.css('height');
//	var LayerInListHeight=jQuery(LayerInList).outerHeight(false); // there are still some things that only jQuery does well.
//	var LayerListWidth=jQuery(TheElement).outerWidth(false); // there are still some things that only jQuery does well.
//	LayerInListTop+=LayerInList.getBoundingClientRect().height;
//	var Height=LayerInList.getBoundingClientRect().css('height');
}

//*******************************************************************************
// CMItem functions to be overriden by children
//*******************************************************************************

/**
* Reset the current style,
* @protected, @override
*/
/*
CMItem.prototype.ResetStyle=function() 
{
}*/
/**
* 
* @protected, @override
*/
CMItem.prototype.Repaint=function() 
{
	var Result=this.GetParent(CMScene);
	if (Result!=null) Result.Repaint();
}
/**
* 
* @protected, @override
*/
CMItem.prototype.Paint=function(TheView) 
{
}
/**
* 
* @protected, @override
*/
CMItem.prototype.PaintSelected=function(TheView) 
{
}
//*******************************************************************************
// CMItem public functions
//*******************************************************************************
/*
* Returns the current name of the layer
* @public, @deprecated
* @returns Name - the name of the layer
*/
CMItem.prototype.GetName=function()  
{ 
	var Result=this.GetSetting("Item","Name","");
	return(Result); 
}


/*
* Set the human-readable name for the layer.  This name will appear
* in the layer list.
* @public, @deprecated
* @param Name - sets the name of the layer.
*/
CMItem.prototype.SetName=function(Name) 
{
	this.SetSetting("Item","Name",Name);
//	this.Name=Name;
	//if (this.GetScene()!=null) this.GetScene().LayerSettingsChanged(this);
}


/**
* Gets the value of the selected flag for this object.
* @public
* @returns Selected flag
*/
CMItem.prototype.GetSelected=function() 
{
	return(this.Selected);
}
/**
* Sets this object to be the one the user has selected
* @public
* @returns Selected flag
*/
CMItem.prototype.SetSelected=function(New) 
{
	if (this.Selected!=New)
	{
		// get the scene which may be this object
		
		var TheScene;
		
		if (this instanceof CMScene)
		{
			TheScene=this;
		}
		else
		{
			TheScene=this.GetParent(CMScene);
		}
		
		if (New)  // selecting this object,  unselect previous content 
		{
			TheScene.UnselectAll(false); // unselect but do not send messages because we are about to select (keeps repaints down)
			this.Selected=true; // set the new selection
			
			if (this.LayerInList!=undefined)
			{
				this.LayerInList.className="CM_LayerListItemClass_Selected";
//				this.LayerInList.style.border="1px solid #000000"; 
			}
			TheScene.SelectionChanged(this); // notify everyone that the selectoin changed
		}
		else // unselecting, just notify everyone
		{
			this.Selected=false; // unselect
			this.LayerInList.className="CM_LayerListItemClass";
//			this.LayerInList.style.border="0px none";
			
			TheScene.SelectionChanged(this); // notify everyone that the selectoin chagned
		}
	}
}
/**
* jjg - SetVisible() being deprecated, moving this to SetSetting()
*/
CMItem.prototype.SetVisible=function(Flag) 
{
	var Status=this.GetSetting("Item","Status");
	
	if ((Flag)&&(Status==CMItem.STATUS_HIDDEN))
	{
		this.SetSetting("Item","Status",CMItem.STATUS_VISIBLE);
		this.Repaint();
	}
	else if ((Flag==false)&&(Status>=CMItem.STATUS_VISIBLE)) // works for true, false, and undefined
	{
		this.SetSetting("Item","Status",CMItem.STATUS_HIDDEN);
		this.Repaint();
	}
}
/**
* Returns the visible state of this item
* @public
* @returns true if the item is visible, false otherwise
*/
CMItem.prototype.GetVisible=function()
{
	var Result=true;
	
	var Status=this.GetSetting("Item","Status");
	
	if (Status==CMItem.STATUS_HIDDEN)
	{
		Result=false;
	}

	return(Result);
}
