/**************************************************************************************************
* CMItemHTML Class
* This class creates a DIV that can contain HTML (including SVG) data to be associated with a coordinate in 2D or 3D space.
* This was originally used to hold SVG data and then I realized this could be any HTML content.
*
* @module CM3ItemHTML
* @Copyright HSU, Jim Graham, 2019
**************************************************************************************************/
//**************************************************************************************************
// Constructor
//**************************************************************************************************

/**
 * Contructor
 */
CMItemHTML = function(Text) 
{
	CMItem.call(this);
	
	// Properties
	this.TheText=Text; // (jjg - setting?)

	this.Initialized=false;
};
CMItemHTML.prototype = new CMItem();

CMItemHTML.prototype.constructor = CMItemHTML;
//**************************************************************************************************
// Public Definitions
//**************************************************************************************************

CMItemHTML.SettingDefintions=
{
	HTML:
	{
		Text: { Name:"Text",Type:CMBase.DATA_TYPE_STRING, Default:"" }, // HTML text to set into the div
		ClassName: { Name:"Class Name",Type:CMBase.DATA_TYPE_STRING }, // HTML text to set into the div
		Position: { Name:"Type",Type:CMBase.DATA_TYPE_ENUMERATED, Options:CMLayer.LABEL_DIRECTIONS }, // Default is undefined which will center the div
		OffsetX: { Name:"Offset X",Type:CMBase.DATA_TYPE_FLOAT, Default:0 },
		OffsetY: { Name:"Offset Y",Type:CMBase.DATA_TYPE_FLOAT, Default:0 },
	},
}
//**************************************************************************************************
// Private functions
//**************************************************************************************************

/**
* @private
*/
CMItemHTML.prototype.Initialize=function()
{
	if (this.SVGElement==undefined)
	{
		//********************************************
		// draw some SVG text near the light
		var TheMainContainer=this.GetParent(CMMainContainer);
		if (TheMainContainer!=null)
		{
			var TheText=this.GetSetting("HTML","Text");
			
			var TheCanvasContainer=TheMainContainer.GetElement(CMMainContainer.CANVAS_CONTAINER);
	
			this.TheContainer=document.createElement("div");
			var TheContainer=this.TheContainer;
			TheContainer.style.position="relative"; // jjg move to settings
			TheContainer.style.top="10px";
			TheContainer.style.left="10px";
			TheContainer.style.width="300px";
			TheContainer.style.height="100px";
			TheContainer.style.zIndex=999;
			
			TheContainer.innerHTML=TheText;

			TheCanvasContainer.style.overflow="Hidden";
			TheCanvasContainer.appendChild(this.TheContainer);
			this.SVGElement=this.TheContainer;
			
			var TheScene=this.GetParent(CMScene);
			var TheView=TheScene.GetView(0);
			
			TheView.AddListener(CMView.MESSAGE_MOUSE_MOVE,this,function(TheView,ThisItem,TheEvent)
			{
				var TheObject=ThisItem.GetParent();
				
				if (TheObject.GetOGLObject()!=null)
				{
					var Result=TheView.GetScreenPosition(TheObject.GetOGLObject());
					
					// get position in camera (range from -1 to 1)
					var x=Result[0]; // 
					var y=Result[1];
					
					// get the width and height of the canvas container
					var TheCanvasContainer=TheMainContainer.GetElement(CMMainContainer.CANVAS_CONTAINER);
					var Width=TheCanvasContainer.clientWidth;
					var Height=TheCanvasContainer.clientHeight;
					
					x*=Width/2; // convert to pixel coordinates
					y*=Height/2;
					
					// move to upper left corner of the text
					x+=Width/2;
					y+=Height/2;
					
					x-=150;
					y-=50;
					
					ThisItem.SVGElement.style.top=y+"px";
					ThisItem.SVGElement.style.left=x+"px";
				}
			});
		}
	}
}
//******************************************************************
// CMBase Settings Functions
//******************************************************************

CMItemHTML.prototype.GetSettingsDefinitions=function() 
{
	var Result={};
	
	for (Key in CMItemHTML.SettingDefintions)
	{
		Result[Key]=CMItemHTML.SettingDefintions[Key];
	}

	return(Result); 
}
	
CMItem.prototype.CMItem_SetSettings=CMItem.prototype.SetSettings;

CMItemHTML.prototype.SetSettings=function(NewSettings,TimeSlice) 
{
	this.CMItem_SetSettings(NewSettings,TimeSlice);
	
	this.Repaint();
}

//**************************************************************************************************
// Public STItem Functions
//**************************************************************************************************

/**
* for 3D items, making them visible means removing the control item
*/
CMItemHTML.prototype.CMItem_SetVisible=CMItem.prototype.SetVisible;

CMItemHTML.prototype.SetVisible=function(Flag)
{
	if ((this.GetVisible()!=Flag)||(this.Initialized==false))
	{
		this.CMItem_SetVisible(Flag);
		
		this.Initialize();
		
		//this.AddOrRemoveObject(Flag);
	}
	
};


CMItemHTML.prototype.Paint=function(TheView) 
{
	this.Initialize();
}



//**************************************************************************************************
// Public functions
//**************************************************************************************************
/**
*
*//*
CMItemHTML.prototype.SetPosition=function(X,Y,Z)
{
	if (this.SVGElement==undefined) this.Initialize();
	
	if (X instanceof THREE.Vector3) 
	{
		this.Position=X;
	}
	else 
	{
		this.Position=new THREE.Vector3(X,Y,Z);
	}
	
	//********************************************
	// draw some SVG text near the light
	var TheMainContainer=this.GetParent(CMMainContainer);
	
	if (TheMainContainer!=null)
	{
		var TheCanvasContainer=TheMainContainer.GetElement(CMMainContainer.CANVAS_CONTAINER);
		
		// update the text within the object
		// find the new position for the SVG text
		var TheView=TheMainContainer.GetScene().GetView(0);
		
		var TheCamera=TheView.GetCamera();
		
		var pos = this.Position.clone();
		projScreenMat = new THREE.Matrix4();
		projScreenMat.multiplyMatrices( TheCamera.projectionMatrix, TheCamera.matrixWorldInverse );
		projScreenMat.multiplyVector3( pos );
		
		var x=( pos.x + 1 ) * TheCanvasContainer.offsetWidth / 2;
		var y= ( - pos.y + 1) * TheCanvasContainer.offsetHeight / 2;
		
		this.TheContainer.style.left=x+"px";
		this.TheContainer.style.top=y+"px";
	}
};*/
