/******************************************************************
* CMLayerDataset Class
*
* @module CMLayerDataset
* @Copyright HSU, Jim Graham, 2019
******************************************************************/

//******************************************************************
// Definitions
//******************************************************************
/**
* Below are the settings definitions.
* @public, @settings
*/
CMLayerDataset.SettingDefintions=
{
	Dataset: 
	{ 
		URL: { Name:"URL",Type:CMBase.DATA_TYPE_STRING, Default:null }, // Full or partial URL to the dataset to load into the layer
		Format: { Name:"Format",Type:CMBase.DATA_TYPE_ENUMERATED,Options:[CMDataset.GEOJSON,CMDataset.PYRAMID,CMDataset.PYRAMID_OPEN_FORMAT,CMDataset.RASTER,CMDataset.SQL],Default:CMDataset.GEOJSON }, // Type of data to load.
		NorthPoleColor: { Name:"NorthPoleColor",Type:CMBase.DATA_TYPE_COLOR,Default:null}, // Color used to fill in the north pole for pyramid layers 
		SouthPoleColor: { Name:"SouthPoleColor",Type:CMBase.DATA_TYPE_COLOR,Default:null }, // Color used to fill in the south pole for pyramid layers 
	},
};

//******************************************************************
// CMLayerDataset Constructor
//******************************************************************
/*
* Constructor
* @protected, @constructs
*/
function CMLayerDataset() 
{
	CMLayer.call(this);
	
	this.TimeSlices[0].Settings.Dataset=	
	{
	};
	// Properties
	this.TheDataset=null; // default
}
CMLayerDataset.prototype=Object.create(CMLayer.prototype); // inherit prototype functions from PanelBase()

CMLayerDataset.prototype.contructor=CMLayerDataset; // override the constructor to go to ours

//**************************************************************
// CMItem Functions
//**************************************************************
CMLayerDataset.prototype.CMLayer_SetParent=CMLayer.prototype.SetParent;

CMLayerDataset.prototype.SetParent=function(NewParent)
{
	this.CMLayer_SetParent(NewParent);
	
	if (this.TheDataset!=null) this.TheDataset.SetParent(this.GetParent(CMScene));
}

/**
* Unselects all features in the layer.  
* @public
*/
CMLayerDataset.prototype.CMLayer_UnselectAll=CMLayer.prototype.UnselectAll;

CMLayerDataset.prototype.UnselectAll=function(SendMessageFlag) 
{
	this.CMLayer_UnselectAll(SendMessageFlag);
	
	if ((this.TheDataset!=null)&&(this.TheDataset.GetSelectedFeature()!=-1)) // something is selected
	{ 
		this.TheDataset.UnselectAll(SendMessageFlag); // features are selected in the dataset
	}
}
CMLayerDataset.prototype.CMLayer_SetVisible=CMLayer.prototype.SetVisible;

CMLayerDataset.prototype.SetVisible=function(Flag) 
{
	this.CMLayer_SetVisible(Flag);
	
	if (this.TheDataset!=null) // something is selected
	{ 
		this.TheDataset.SetVisible(this,Flag);
	}
}

//**************************************************************
// CMItem Functions for painting
//**************************************************************

/*
* Paints a layer into the canvas.  This function is provided for 
* subclasses to override.  The code below shows the sequence of steps
* a layer class should take to paint itself into the canvas.
* @override
* @param TheView - the view to paint into.
*/
CMLayerDataset.prototype.Paint=function(TheView) 
{
	if ((this.IsVisible(TheView))&&(this.TheDataset!=null))
	{
		var TheStyle=this.GetStyle(TheView);
		
		if (TheStyle!==null) { TheView.SetStyle(TheStyle); }
		
		this.TheDataset.Paint(this,TheView,false);
		
		if (TheStyle!==null) { TheView.RestoreStyle(); }
	}
}
/**
* Just paint the selected features.  This is called after all the other features have
* been painted to paint the selected features on top
* @override
* @param TheView - the view to paint into.
*/
CMLayerDataset.prototype.PaintSelected=function(TheView) 
{
	if ((this.IsVisible(TheView))&&(this.TheDataset!=null))
	{
		// paint mouse over only if the style was set
		var TheStyle=this.GetStyle(TheView,undefined,"MouseOverStyle");
		
		if (CMUtilities.IsDefined(TheStyle)) 
		{ 
			TheView.SetStyle(TheStyle);
		
			this.TheDataset.Paint(this,TheView,false,true);
		
			TheView.RestoreStyle();
		}
		// paint the selected features only if the style was set
		var TheStyle=this.GetStyle(TheView,undefined,"SelectedStyle");
		
		if (CMUtilities.IsDefined(TheStyle)) 
		{ 
			TheView.SetStyle(TheStyle);
		
			this.TheDataset.Paint(this,TheView,true);
		
			TheView.RestoreStyle(); 
		}
	}
}

//**************************************************************
// CMBase Functions
//**************************************************************

CMLayerDataset.prototype.CMLayer_GetSettingsDefinitions=CMLayer.prototype.GetSettingsDefinitions;

CMLayerDataset.prototype.GetSettingsDefinitions=function() 
{
	var Result=this.CMLayer_GetSettingsDefinitions();
	
	for (Key in CMLayerDataset.SettingDefintions)
	{
		Result[Key]=CMLayerDataset.SettingDefintions[Key];
	}
	return(Result); 
}

//**************************************************************
// CMLayer Functions
//**************************************************************
/**
* Check if the feature is visible in the view.
* This should be called by subclasses but can also be called to limit a layer's bounds after loading.
* @public
* @param NewBounds - Object for Bounds with format {XMin,XMax,YMin,YMax}
*/
CMLayerDataset.prototype.SetBounds=function(NewBounds) 
{
	if (this.GetDataset()!=undefined) 
	{
		this.GetDataset().SetBounds(NewBounds);
	
		var TheScene=this.GetParent(CMScene);
		TheScene.SetBoundsDirty();
	}
}
/**
* Returns the bounds of the data within the layer.  Computed after loading the data.
* @public
* @returns Bounds - with format {XMin,XMax,YMin,YMax}
*/
CMLayerDataset.prototype.GetBounds=function() 
{
	var TheBounds=undefined;
	
	if (this.GetDataset()!=undefined) 
	{
		TheBounds=this.GetDataset().GetBounds();
		
		TheBounds=this.ProjectBounds(TheBounds);
	}
	return(TheBounds);
}

CMLayerDataset.prototype.SetProjector=function(NewProjector)
{
	this.TheProjector=NewProjector;
	if (this.TheDataset!=null) 
	{
		this.TheDataset.SetProjector(NewProjector); // jjg temp kludge
	}
}

CMLayerDataset.prototype.GetProjector=function()
{
	var Result=null;
	
	if (this.TheDataset!=null) 
	{
		Result=this.TheDataset.GetProjector(); // jjg temp kludge
	}
	return(Result);
}
//**************************************************************
// CMLayer functions for features
//**************************************************************

/**
* Sets the feature that is selected
* @public
* @param NewFeatureIndex - >=0 indicates a feature, -1 is for none.
*/
CMLayerDataset.prototype.SetSelectedFeature=function(New) 
{
	if ((this.TheDataset!=null)&&(New!=this.TheDataset.GetSelectedFeature()))
	{
		this.TheDataset.SetSelectedFeature(New); // sends message
		
		// call the scene to let everyone know the selection changed
		var TheScene=this.GetParent(CMScene);
		
		TheScene.SelectionChanged(this);
	}
}
/**
* Sets the current feature that the mouse is over
* @public
* @param NewFeatureIndex - index to the feature the mouse is over (typcially returned by In())
*/
CMLayerDataset.prototype.SetMouseOverFeature=function(New) 
{
	if ((this.TheDataset!=null)&&(this.TheDataset.GetMouseOverFeature()!=New)) // something is selected
	{ 
		this.TheDataset.SetMouseOverFeature(New);
	}
}
/**
* Returns the current feature that the mouse is over
* @public
* @returns FeatureIndex - index to the feature the mouse is over (typcially returned by In()), or -1 for none
*/
CMLayerDataset.prototype.ResetMouseOverFeature=function() 
{
	if ((this.TheDataset!=null)&&(this.TheDataset.GetMouseOverFeature()!=-1)) // something is selected
	{ 
		this.TheDataset.SetMouseOverFeature(-1);
	}
}
//**************************************************************
// CMLayer functions for feature interaction
//**************************************************************

CMLayerDataset.prototype.In=function(TheView,RefX,RefY) 
{
	var FeatureIndex=-1;
	
	if (this.TheDataset!=null)
	{
		// project the coordinates back to geographic if needed
		var Result=this.ProjectCoordinate(RefX,RefY);
		
		if (Result!=null)
		{
			RefX=Result[0];
			RefY=Result[1];
		
			var RefTolerance=this.GetRefTolerance(TheView);
			
			var NumFeatures=this.TheDataset.GetNumFeatures();
			
			for (var i=0;( i < NumFeatures)&&(FeatureIndex==-1); i++) 
			{
				var Status=this.GetFeatureSetting("Item","Status",i,true);
				
				if (Status>=CMItem.STATUS_VISIBLE)
				{
					var Result=this.TheDataset.InFeature(TheView,RefX,RefY,i,RefTolerance);

					if (Result) 
					{
						FeatureIndex=i;
					}
				}
			}
			var FeatureIndex=this.TheDataset.In(TheView,RefX,RefY,RefTolerance);
		}
	}
	return(FeatureIndex);
};
//******************************************************************
// CMLayer Search Functions
//******************************************************************

CMLayerDataset.prototype.GetSearchResults=function(SearchPhrase,ResultsPanel) 
{
	if ((this.IsVisible())&&(this.TheDataset!=null))
	{
		this.TheDataset.GetSearchResults(SearchPhrase,ResultsPanel);
	}
}
/*
* returns the icon for the layer list
* @override
* @protected
* @returns - an element that can be placed next to the layer in the layer list (should be 16x16)
*/
CMLayerDataset.prototype.CMLayer_GetIcon=CMLayer.prototype.GetIcon;

CMLayerDataset.prototype.GetIcon=function()
{
	var TheIcon=this.CMLayer_GetIcon();
	
	if (this.TheDataset!=null)
	{
		TheIcon=this.TheDataset.GetIcon(this,TheIcon);
	}
	return(TheIcon);
}

//**************************************************************
// CMLayer functions
//**************************************************************
/*
* Called to obtain the data for the layer from a URL.
*
* @override, @public
*/
CMLayerDataset.prototype.RequestData=function() 
{
	var URL=this.GetSetting("Dataset","URL",null);
	var Format=this.GetSetting("Dataset","Format",CMDataset.GEOJSON);
	this.SetURL(URL,Format);
}
//**************************************************************
// CMLayerDataset functions for managing the dataset
//**************************************************************
/*
* Called to obtain the data for the layer from a URL.
* Depricated, use:
*	- SetSetting("Dataset","URL",URL);
*	- SetSetting("Dataset","Format",Format);
*	- RequestData()
*
* @protected
* @param URL - the URL to use to obtain data
* @param DataSetType - optional data set type (GeoJSON is the default)
*/
CMLayerDataset.prototype.SetURL=function(URL,DataSetType) 
{
	var TheScene=this.GetParent(CMScene);
	
	// create the new data set or get an existing one
	if (this.TheDataset==null)
	{
		var TheDataset=CMDataset.GetDataObject(URL,DataSetType);
		this.SetDataset(TheDataset);
	}
									 
	// Call the layer's OnLoad() function when the dataset loads the data
	this.TheDataset.AddListener(CMDataset.MESSAGE_DATASET_TILE_LOADED,this,function(TheDataset,ThisLayer,AdditionalInfo)
	{
		//ThisLayer.OnLoad(); this is just a tile loading so we just repaint the scene
		ThisLayer.GetScene().Repaint();
	});
									 
	// Call the layer's OnLoad() function when the dataset loads the data
	this.TheDataset.AddListener(CMDataset.MESSAGE_DATASET_LOADED,this,function(TheDataset,ThisLayer,AdditionalInfo)
	{
		ThisLayer.OnLoad();
		ThisLayer.GetScene().LayerListChanged(); // jjg - a bit of a hack to get the layer list to repaint once we have the data (i.e. can determine if the GeoJSON data is string).
		// setup any properties that were set by attributes
		if (ThisLayer.SettingsAttributes!=null)
		{
			// go through all the properties that come from attributes
			for (var SettingsGroupKey in ThisLayer.SettingsAttributes)
			{
				var SettingsGroup=ThisLayer.SettingsAttributes[SettingsGroupKey];
				
				for (var SettingsKey in SettingsGroup)
				{
					var AttributeName=SettingsGroup[SettingsKey];
					
					var Index=TheDataset.GetAttributeIndexFromHeading(AttributeName);
					
					if (Index==-1) CMMainContainer.Error("Sorry, the attribute "+AttributeName+" was not found");
					else TheDataset.LoadAttributeColumn(Index);
				}
			}
		}
	});
	// Make sure the scene is repained when the selection changes
	this.TheDataset.AddListener(CMDataset.MESSAGE_DATASET_SELECTION_CHANGED,this,function(TheDataset,ThisLayer,AdditionalInfo)
	{
		ThisLayer.GetScene().SelectionChanged(TheDataset);
//		ThisLayer.GetScene().Repaint();
	});
	
	// Make sure the scene is repained when the mouse over changes
	this.TheDataset.AddListener(CMDataset.MESSAGE_DATASET_MOUSE_OVER_FEATURE_CHANGED,this,function(TheDataset,ThisLayer,AdditionalInfo)
	{
		ThisLayer.GetScene().Repaint();
	});
	/*this.TheDataset.AddListener(CMDataset.MESSAGE_DATASET_TILE_LOADED,this,function(TheDataset,ThisLayer,AdditionalInfo)
	{
		ThisLayer.OnLoad();
		ThisLayer.GetScene().Repaint();
	});
	*/
	// add a listener for when the attributes change
	// if there are settings that are based on an attribute, we need to update the settings here.
	this.TheDataset.AddListener(CMDataset.MESSAGE_ATTRIBUTES_CHANGED,this,function(TheDataset,ThisLayer,AdditionalInfo)
	{
		// setup any properties that were set by attributes
		if (ThisLayer.SettingsAttributes!=null)
		{
			// go through all the properties that come from attributes
			for (var SettingsGroupKey in ThisLayer.SettingsAttributes)
			{
				var SettingsGroup=ThisLayer.SettingsAttributes[SettingsGroupKey];
				
				for (var SettingsKey in SettingsGroup)
				{
					var AttributeName=SettingsGroup[SettingsKey];
				
					var ColumnIndex=TheDataset.GetAttributeIndexFromHeading(AttributeName);
					
					if (ColumnIndex==-1)
					{
						var TheMainContainer=ThisLayer.GetParent(CMMainContainer);
						CMMainContainer.Error("Sorry, the attribute "+AttributeName+" in "+ThisLayer.GetName()+" is not available");
					}
					else
					{
						var NumRows=TheDataset.GetNumAttributeRows();
						
						for (var j=0;j<NumRows;j++)
						{
							if (NumRows!=TheDataset.GetNumAttributeRows())
							{
								var test=12;
							}
							var Value=TheDataset.GetAttributeCell(ColumnIndex,j);
						
							if ((Value!==undefined)&&(Value!==""))
							{
								if ((SettingsGroupKey=="IconImage")&&(SettingsKey=="URL"))
								{
									var TheImage;  //=JSON.parse(Value); // image object with the URL
									TheImage=new Image(); 
									TheImage.Loaded=false;
									TheImage.TheLayer=this;
									
									TheImage.onload=function() 
									{ 
										This.Loaded=true;
				
										This.TheLayer.Repaint(); 
									};
									TheImage.src=Value; 
								}
								else if ((SettingsGroupKey=="Mark")&&(SettingsKey=="Type"))
								{
									if (Value==="MARK_CIRCLE") Value=CMLayer.MARK_CIRCLE;
									if (Value==="MARK_TRIANGLE") Value=CMLayer.MARK_TRIANGLE;
									if (Value==="MARK_SQUARE") Value=CMLayer.MARK_SQUARE;
									if (Value==="MARK_STAR") Value=CMLayer.MARK_STAR;
								}
								else if ((SettingsGroupKey=="Mark")&&(SettingsKey=="Size"))
								{
									Value=parseInt(Value);
								}
								else // all others are HTML 5 Canvas Styles
								{
									Value=Value;
								}
								ThisLayer.SetFeatureSetting(SettingsGroupKey,SettingsKey,j,Value);
								
								if (ThisLayer.CallbackFunction!=undefined) ThisLayer.CallbackFunction(ThisLayer.CallbackParameters);
							}
						}
					}
				}
			}
			ThisLayer.SendMessageToListeners(CMBase.MESSAGE_SETTINGS_CHANGED,null);
		}
	});
	
	// add a projector if one is provided
//	if (TheProjector!=undefined) this.TheDataset.SetProjector(TheProjector);
	
	// load the main dataset
	this.TheDataset.SetURL(URL);

}
/*
* Set the vector data from a GeoJSON object
* @public
*/
CMLayerDataset.prototype.SetData=function(TheData,DataSetType) 
{
	// create the new data set or get an existing one
	var TheScene=this.GetParent(CMScene);
	
	var NewDataset=CMDataset.GetDataObject(null,DataSetType);
	
	this.SetDataset(NewDataset);
//	this.TheDataset.SetParent(TheScene); 
	
//	this.GetDataset(); // allocates the dataset
	
	// add a projector if one is provided
	//if (TheProjector!=undefined) this.TheDataset.SetProjector(TheProjector);
	
	this.TheDataset.SetData(TheData);
};
/*
* Gets the dataset from this object.  Allocates the dataset if it has not already
* been allocated.
* @public
*/
CMLayerDataset.prototype.GetDataset=function() 
{
	if (this.TheDataset==null)
	{
		var Format=this.GetSetting("Dataset","Format",null);
		
		if (Format!=null)
		{
			var TheDataset=CMDataset.GetDataObject(null,Format);
			
			this.SetDataset(TheDataset);
		}
	}
	
	return(this.TheDataset);
};
/*
* sets the dataset from this object
* @public
*/
CMLayerDataset.prototype.SetDataset=function(NewDataset) 
{
	this.TheDataset=NewDataset;
	
	if (this.TheDataset!=null) 
	{
		this.TheDataset.SetParent(this.GetParent(CMScene));
		
		this.TheDataset.SetVisible(this,this.GetVisible());
		
		this.TheDataset.AddListener(CMDataset.MESSAGE_DATASET_LOADED,this,function(TheDataset,ThisLayer,AdditionalInfo)
		{
			ThisLayer.OnLoad();
		});
		if (this.TheProjector!=undefined) this.TheDataset.SetProjector(this.TheProjector);
	}
};
