/******************************************************************************************************************
* CMLayerGrid Class
*
* Displays a raster based on the numeric values in a JSON file.
*
* @module CMDatasetPyramidOpenFormat
* @Copyright HSU, Jim Graham, 2019
******************************************************************************************************************/
//******************************************************************
// Constructor
//******************************************************************
function CMLayerGrid() 
{
	CMLayer.call(this);

	// Properties
	this.TheImage=null;
}
CMLayerGrid.prototype=Object.create(CMLayer.prototype); // inherit prototype functions from PanelBase()

CMLayerGrid.prototype.contructor=CMLayerGrid; // override the constructor to go to ours
//******************************************************************
// CMLayer Class
//******************************************************************
/*
* Called to obtain the data for the layer from a URL.
* Depricated, use:
*	- SetSetting("Dataset","URL",URL);
*	- SetSetting("Dataset","Format",Format);
*	- RequestData()
*
* TheView is required so the layer can be repainted when data is received. 
* @override, @public
* @param URL - the URL to use to obtain data
* @param DataSetType - optional data set type (GeoJSON is the default)
*/
CMLayerGrid.prototype.RequestData=function() 
{
	var URL=this.GetSetting("Dataset","URL",null);
	var Format=this.GetSetting("Dataset","Format",CMDataset.GEOJSON);
	this.SetURL(URL,Format);
}
/*
* Depricated from public servers
* Called to obtain the data for the layer from a URL.
* Currently,only GeoJSON is supported as the transfer type.
* @private
*/
CMLayerGrid.prototype.SetURL=function(URL) 
{
	var TheRequest=new XMLHttpRequest();
	TheRequest.open("GET",URL,true);
	TheRequest.TheURL=URL;
	TheRequest.TheLayer=this;
	
	TheRequest.onreadystatechange=function() 
	{
		if( this.readyState == 4)  // done
		{
			if( this.status == 200) // OK
			{
				// get the JSON object from the response
				
				var TheText=TheRequest.responseText;

				var TheGeoJSONObject=JSON.parse(TheText);
				
				// find the parameters for the data
				
				this.TheLayer.NumBands=TheGeoJSONObject.NumBands;
				
				this.TheLayer.NumColumns=TheGeoJSONObject.WidthInPixels;
				this.TheLayer.NumRows=TheGeoJSONObject.HeightInPixels;
				
				this.TheLayer.MinPixelValues=TheGeoJSONObject.MinPixelValues;
				this.TheLayer.MaxPixelValues=TheGeoJSONObject.MaxPixelValues;
			
				this.TheLayer.TheData=TheGeoJSONObject.Data;
				
				// get the bounds 
				
				this.TheLayer.TheBounds=TheGeoJSONObject.Bounds; // 
				
				this.TheLayer.OnLoad();
			}
			else alert("HTTP error "+this.status+" "+this.statusText+" ("+this.TheURL+")");
		}
	}
	TheRequest.send();
}

//******************************************************************
// CMLayerGrid Functions
//******************************************************************

CMLayerGrid.prototype.In=function(TheView,RefX,RefY) 
{
	return(-1);
};

/*
* Paints a layer into the canvas
*/
CMLayerGrid.prototype.Paint=function(TheView) 
{
	if ((this.TheData!=undefined)&&(this.TheImageData==null))
	{
		var Range=this.MaxPixelValues-this.MinPixelValues;
		var Factor=255.0/Range;
		
		var TheCanvasElement=TheView.GetCanvasElement();
		
		var TheContext=TheCanvasElement.getContext("2d");
		var TheImageData=TheContext.createImageData(this.NumColumns,this.NumRows);
		
		var Index=0;
		for (var y=0;y<this.NumRows;y++)
		{
			for (var x=0;x<this.NumColumns;x++)
			{
				var Value=this.TheData[y][x];
				
				Value=(Value-this.MinPixelValues)*Factor; // convert to 0 to 255
				
				TheImageData.data[Index+0]=Value;
				TheImageData.data[Index+1]=Value;
				TheImageData.data[Index+2]=Value;
				TheImageData.data[Index+3]=255;
				
				Index+=4;
			}
		}
		this.TheImageData=TheImageData;
	}
	
	if ((this.IsVisible(TheView))&&(this.TheImageData!=null))
	{
		// Get the drawing context from our <canvas> and
		// set the fill to determine what color our map will be.
		var TheContext=TheView.GetContext();
		
		var TheBounds=this.TheBounds;
		
		var TheStyle=this.GetStyle(TheView);
		
		if (TheStyle!=undefined) TheView.SetStyle(TheStyle);
		
		//TheContext.putImageData(this.TheImageData,100,100);	
		
		if (TheBounds!=null)
		{
			RefX=TheBounds.XMin;
			RefY=TheBounds.YMax;
			RefWidth=TheBounds.XMax-TheBounds.XMin;
			RefHeight=TheBounds.YMin-TheBounds.YMax;
		
			var Result=TheView.GetPixelFromRef(RefX,RefY);
			var XInPixels1=Math.round(Result.PixelX);
			var YInPixels1=Math.round(Result.PixelY);
			
			var PixelWidth=TheView.GetPixelWidthFromRefWidth(RefWidth);
			var PixelHeight=TheView.GetPixelHeightFromRefHeight(RefHeight);
			
			PixelWidth=Math.round(PixelWidth);
			PixelHeight=Math.round(PixelHeight);
			
			var TheCanvasElement=TheView.GetCanvasElement();
			var TheContext=TheCanvasElement.getContext("2d");
			TheContext.putImageData(this.TheImageData,XInPixels1,YInPixels1,0,0,PixelWidth,PixelHeight);	
			
//			this.TheContext.drawImage(TheImage,XInPixels1,YInPixels1,PixelWidth,PixelHeight);		
//			TheView.PaintRefImageScaled(this.TheImage,this.TheBounds);
		}
		else // draw the raster in the upper left corner of the canvas
		{
			TheContext.drawImage(this.TheImageData,0,0,this.TheImage.width,this.TheImage.height);
		}
		if (TheStyle!=undefined) TheView.RestoreStyle();
	}
}

