﻿/******************************************************************************************************************
* CMLayerItems
*
* @module CMLayerItems
* @Copyright HSU, Jim Graham, 2019
******************************************************************************************************************/
//******************************************************************
// CMToolHandler (move later?)
//******************************************************************

function CMToolHandler(ObjectType)
{
	this.ObjectType=ObjectType;
} 
CMToolHandler.prototype.MouseDown=function(TheView,RefX,RefY,TheEvent) 
{
	var Used=false;
	
//	if (this.TheLayer.Dragging==false) // not creating something already
	{
		var ItemIndex=-1;
		
		var NewObject=null;
		
		switch (this.ObjectType)
		{
		case "Rectangle":
			NewObject=new CMItemRect(CMItemRect.RECTANGLE);
			ItemIndex=this.TheLayer.AddChild(NewObject);
			NewObject.SetControlBounds(RefX,RefX,RefY,RefY);
			break;
		case "RoundedRectangle":
			NewObject=new CMItemRect(CMItemRect.ROUNDED_RECTANGLE);
			ItemIndex=this.TheLayer.AddChild(NewObject);
			NewObject.SetControlBounds(RefX,RefX,RefY,RefY);
			break;
		case "Oval":
			NewObject=new CMItemRect(CMItemRect.OVAL);
			ItemIndex=this.TheLayer.AddChild(NewObject);
			NewObject.SetControlBounds(RefX,RefX,RefY,RefY);
			break;
		case "Polygon":
			NewObject=new CMItemPoly();
			ItemIndex=this.TheLayer.AddChild(NewObject);
			
			var Xs=[RefX,RefX];
			var Ys=[RefY,RefY];
			var Smoothing=[false,false];
			NewObject.SetControlPoints(0,Xs,Ys,Smoothing);
//			NewObject.SetSettings("Curve","Smoothing",false);
			break;
		case "Curve":
			NewObject=new CMItemPoly();
			ItemIndex=this.TheLayer.AddChild(NewObject);
			
			var Xs=[RefX,RefX];
			var Ys=[RefY,RefY];
			var Smoothing=[true,true];
			NewObject.SetControlPoints(0,Xs,Ys,Smoothing);
			break;
		case "Arrow":
			NewObject=new CMItemPolyArrow();
			ItemIndex=this.TheLayer.AddChild(NewObject);
			
			var Xs=[RefX,RefX];
			var Ys=[RefY,RefY];
			var Smoothing=[true,true];
			NewObject.SetControlPoints(0,Xs,Ys,Smoothing);
			break;
		}
	
		NewObject.StartCreating(RefX,RefY);
		NewObject.SetSelected(true);
		NewObject.SetSetting("Item","Status",CMItem.STATUS_EDITABLE);
		
		this.TheLayer.SelectedItemIndex=ItemIndex;
		
		Used=true;
	}
	
	return(Used);
}
CMToolHandler.prototype.MouseMove=function(TheView,RefX,RefY) 
{
	return(false);
}
CMToolHandler.prototype.MouseUp=function(TheView,RefX,RefY) 
{
	return(false);
}

//******************************************************************
// CMLayerItems Constructor
//******************************************************************
/*
* Constructor
* @public, @constructs
*/
function CMLayerItems() 
{
	CMLayer.call(this);

	// Properties
	this.SelectedItemIndex=-1; // jjg remove?
	
	this.ToolGroupElement=null;
	
	this.SetSetting("Item","Status",CMItem.STATUS_EDITABLE);
}

CMLayerItems.prototype=Object.create(CMLayer.prototype); // inherit prototype functions

CMLayerItems.prototype.contructor=CMLayerItems; // override the constructor to go to ours

//******************************************************************
// CMLayerItems private functions
//******************************************************************
/**
* @private
*/
CMLayerItems.prototype.Unselected=function() 
{
	if (this.ToolGroupElement!=null)
	{
		var TheCanvasMap=this.GetParent(CMMainContainer);
		var TheToolPanel=TheCanvasMap.GetToolPanel();
		
		TheToolPanel.RemoveTool("CMLayerObjects_RectImage");
		TheToolPanel.RemoveTool("CMLayerObjects_RoundedRectImage");
		TheToolPanel.RemoveTool("CMLayerObjects_OvalImage");
		TheToolPanel.RemoveTool("CMLayerObjects_CurveImage");
		TheToolPanel.RemoveTool("CMLayerObjects_ArrowImage");
		
		TheToolPanel.RemoveToolGroupElement(this.ToolGroupElement);
		this.ToolGroupElement=null;
		
		var TheView=TheCanvasMap.GetView();
		
		TheView.SetToolHandler(null);
	}
}

//******************************************************************
// CMBase Functions
//******************************************************************
// override the layer UnselectAll() function
CMLayerItems.prototype.CMLayer_UnselectAll=CMLayer.prototype.UnselectAll;

CMLayerItems.prototype.UnselectAll=function(SendMessage) 
{
	this.CMLayer_UnselectAll(SendMessage);
	
	var NumChildren=this.GetNumChildren();
	
	for (var i=0;i<NumChildren;i++)
	{
		var TheChild=this.GetChild(i);
		
		TheChild.UnselectAll(SendMessage);
	}
	this.Unselected();
}
/**
* Called when the layer is selected
*/
CMLayerItems.prototype.CMLayer_SetSelected=CMLayer.prototype.SetSelected;

CMLayerItems.prototype.SetSelected=function(New) 
{
	if (New!=this.GetSelected()) // selection has changed
	{
		this.CMLayer_SetSelected(New);

		var TheCanvasMap=this.GetParent(CMMainContainer);
		var TheToolPanel=TheCanvasMap.GetToolPanel();
		
		if (New)
		{
			if (TheToolPanel!=null)
			{
				// add the rect tool
				
				var ImageFolder=TheCanvasMap.GetSetting("MainContainer","ImageFolder");
				
				var RectTool=TheToolPanel.AddTool("CMLayerObjects_RectImage",ImageFolder+"Icon_Rect_Default.png",
					ImageFolder+"Icon_Rect_Selected.png");
				
				CMLayerItems.SetupTool(RectTool,this,"Rectangle");
				
				// add the rRoundRectTool
				
				var RoundRectTool=TheToolPanel.AddTool("CMLayerObjects_RoundedRectImage",ImageFolder+"Icon_RoundedRect_Default.png",
					ImageFolder+"Icon_RoundedRect_Selected.png");
				
				CMLayerItems.SetupTool(RoundRectTool,this,"RoundedRectangle");
				
				// add the oval
				
				var OvalTool=TheToolPanel.AddTool("CMLayerObjects_OvalImage",ImageFolder+"Icon_Oval_Default.png",
					ImageFolder+"Icon_Oval_Selected.png");
				
				CMLayerItems.SetupTool(OvalTool,this,"Oval");
				
				// add the polygon tool
				
				var PolygonTool=TheToolPanel.AddTool("CMLayerObjects_PolygonImage",ImageFolder+"Icon_Polygon_Default.png",
					ImageFolder+"Icon_Polygon_Selected.png");
				
				CMLayerItems.SetupTool(PolygonTool,this,"Polygon");
				
				// add the curve tool
				
				var CurveTool=TheToolPanel.AddTool("CMLayerObjects_CurveImage",ImageFolder+"Icon_Curve_Default.png",
					ImageFolder+"Icon_Curve_Selected.png");
				
				CMLayerItems.SetupTool(CurveTool,this,"Curve");
				
				// add the arrow tool
				
				var ArrowTool=TheToolPanel.AddTool("CMLayerObjects_ArrowImage",ImageFolder+"Icon_Arrow_Default.png",
					ImageFolder+"Icon_Arrow_Selected.png");
				
				CMLayerItems.SetupTool(ArrowTool,this,"Arrow");
				
				// make the tools into a group
				this.ToolGroupElement=TheToolPanel.MakeToolGroup([RectTool,RoundRectTool,OvalTool,CurveTool,ArrowTool,PolygonTool]);
			}
			//else
			{
				//TheToolPanel.AddToolGroupElement(this.ToolGroupElement);
			}
		}
		else // delete the tool from the tool bar
		{
			this.Unselected();
		}
		
	}
}
/**
* Sets up a tool to be displayed in the menu bar.
* This is a helper function.
* @ TheTool - Tool element
* @private
*/
CMLayerItems.SetupTool=function(TheTool,TheLayer,ObjectType)
{
	TheTool.TheLayer=TheLayer;
	TheTool.ObjectType=ObjectType;
	
	TheTool.Original_onclick=TheTool.onclick;
	
	TheTool.onclick=function() 
	{ 
		TheTool.Original_onclick();
		
		var TheCanvasMap=this.TheLayer.GetParent(CMMainContainer);
		var TheToolPanel=TheCanvasMap.GetToolPanel();
		
		// setup the tool handler in the view
		
		var TheView=TheCanvasMap.GetView();
		var TheToolHandler=new CMToolHandler(this.ObjectType);
		TheToolHandler.TheLayer=this.TheLayer;
		
		TheView.SetToolHandler(TheToolHandler);
	}
	TheTool.UnselectFunction=function()
	{
		var TheCanvasMap=this.TheLayer.GetParent(CMMainContainer);
		var TheView=TheCanvasMap.GetView();
		
		// reset the tool handler
		TheView.SetToolHandler(null);
	}

}
CMLayerItems.prototype.CMLayer_GetTimeSlices=CMLayer.prototype.GetTimes;

CMLayerItems.prototype.GetTimes=function(TheTimeSlices) 
{
	TheTimeSlices=this.CMLayer_GetTimeSlices(TheTimeSlices);
	
	var NumChildren=this.GetNumChildren();
	
	for (var i=0;i<NumChildren;i++)
	{
		var TheChild=this.GetChild(i);
		
		TheTimeSlices=TheChild.GetTimes(TheTimeSlices);
	}
	return(TheTimeSlices);
}
//******************************************************************
// CMLayer Mouse event handling
//******************************************************************

/*
* returns the feature index for the coordinate in projected space
* returns -1 if the coordinate is not in a feature
*/
CMLayerItems.prototype.In=function(TheView,RefX,RefY) 
{
	var ItemIndex=-1;
	
	var NumChildren=this.GetNumChildren();
	
	if ((this.IsVisible(TheView))&&(NumChildren>0))
	{
		var RefTolerance=this.GetRefTolerance(TheView);
			
		// Loop over the features
		for (var i=0;( i <NumChildren)&&(ItemIndex==-1); i++) 
		{
			var TheChild=this.GetChild(i);
			
			var Part=TheChild.InPart(TheView,RefX,RefY,RefTolerance);
			
			if (Part!=-1) ItemIndex=i;
		}
	}
	return(ItemIndex);
};

CMLayerItems.prototype.MouseDown=function(TheView,RefX,RefY,TheEvent) 
{
	var Used=false;
	
	if ((this.IsVisible(TheView))&&(this.GetSetting("Item","Status")>=CMItem.STATUS_SELECTABLE)) // check if we where clicked in
	{
/*		if ((Used==false)&&(this.Creating))
		{
			var TheItem=this.TheItems[this.SelectedItemIndex];
			
			var NumPoints=TheItem.Xs.length;
			
			if (TheEvent.detail==2) // double click
			{
				this.Creating=false;
				this.Dragging=false;
				TheItem.Xs.pop(); // remove the last point which was added by the second button click
				TheItem.Ys.pop(); // remove the last point which was added by the second button click
			}
			else // add a point
			{
				TheItem.Xs.splice(NumPoints-1,0,RefX);
				TheItem.Ys.splice(NumPoints-1,0,RefY);
			}
			Used=true;
			this.Repaint();
		}
*/		if ((Used==false)&&((TheView.GetTool()==CMView.TOOL_INFO)||(TheView.GetTool()==CMView.TOOL_SELECT)))
		{
			var ItemIndex=this.In(TheView,RefX,RefY);
			
			if (ItemIndex!=-1)
			{
				var TheChild=this.GetChild(ItemIndex);
				
				Used=TheChild.MouseDown(TheView,RefX,RefY,TheEvent);
				
				if (this.GetChild(ItemIndex).GetSelected())
				{
					this.SelectedItemIndex=ItemIndex;
					Used=true;
				}
/*				this.Dragging=true;
				this.SelectedPart=this.InPart(TheView,RefX,RefY,ItemIndex);
				this.SetAnchor(RefX,RefY,ItemIndex,this.SelectedPart) ;
				this.SelectedItemIndex=ItemIndex;
*/				
			}
		}
	}
	return(Used);
};
CMLayerItems.prototype.MouseMove=function(TheView,RefX,RefY,TheEvent) 
{
	var Used=false;
	
	if (this.GetSetting("Item","Status")>=CMItem.STATUS_SELECTABLE)
	{
		var NumChildren=this.GetNumChildren();
		
		for (var i=0;(i<NumChildren)&&(Used==false); i++) 
		{
			var TheChild=this.GetChild(i);
			
			if (TheChild.GetSelected())
			{
				Used=TheChild.MouseMove(TheView,RefX,RefY,TheEvent);
			}
		}
	}
	return(Used);
};
CMLayerItems.prototype.MouseUp=function(TheView,RefX,RefY,TheEvent)
{
	var Used=false;
	
	if (this.GetSetting("Item","Status")>=CMItem.STATUS_SELECTABLE)
	{
		var NumChildren=this.GetNumChildren();
		
		for (var i=0;( i <NumChildren)&&(Used==false); i++) 
		{
			var TheChild=this.GetChild(i);
			
			if (TheChild.GetSelected())
			{
				Used=TheChild.MouseUp(TheView,RefX,RefY,TheEvent);
			}
		}
	}
	return(Used);
};

CMLayerItems.prototype.ShowInfoWindow=function(FeatureIndex,TheView,RefX,RefY) 
{
	var TheFeatures=this.TheData.features;

	var TheFeature=TheFeatures[FeatureIndex];
	
	var Properties=TheFeature.properties;
	
	var TheHTML=this.GetFeatureSetting("Layer","InfoText",FeatureIndex,null);
	
	if (TheHTML!=null)
	{
		var InfoWindow=TheView.CreateInfoWindow("CMLayerItems.InfoWindow",RefX,RefY,this.GetInfoWindowWidth(),30,TheHTML);
		
		CMMainContainer.SetPopupWindow(InfoWindow);
	}
};

//******************************************************************
// CMLayerItems Painting Functions
//******************************************************************
/*
* Paints a layer into the canvas
* This is a little complicated because the geometries can contain
* polylines, polygons or points. 
*/
CMLayerItems.prototype.Paint=function(TheView) 
{
	var NumChildren=this.GetNumChildren();
	
	if ((this.IsVisible(TheView))&&(NumChildren>0))
	{
		var TheStyle=this.GetStyle(TheView);
		
		if (TheStyle!=undefined) TheView.SetStyle(TheStyle);
			
		for (var i=0;i<NumChildren;i++)
		{
			var TheChild=this.GetChild(i);
			
			TheChild.Paint(TheView);
		}
		if (TheStyle!=undefined) TheView.RestoreStyle();
	}
}
/*
* Paints a layer into the canvas
* This is a little complicated because the geometries can contain
* polylines, polygons or points. 
*/
CMLayerItems.prototype.PaintSelected=function(TheView) 
{
	if (this.IsVisible(TheView))
	{
		// paint the mouse over if any
		
		TheView.SaveStyle();
		
		var NumChildren=this.GetNumChildren();
		
		for (var i=0;(i<NumChildren); i++) 
		{
			var TheChild=this.GetChild(i);
			
			TheChild.PaintSelected(TheView);
		}

		TheView.RestoreStyle(); 
	}
}

//******************************************************************
// Public CMLayerItems functions
//******************************************************************

CMLayerItems.prototype.CMLayer_AddChild=CMLayer.prototype.AddChild;

CMLayerItems.prototype.AddChild=function(TheObject) 
{
	var Index=this.CMLayer_AddChild(TheObject);
	
//	this.TheItems.push(TheObject);
//	TheObject.SetParent(this);
	this.Repaint();
	
	this.GetParent(CMScene).LayerContentChanged(this);
	
	return(Index);
}

CMLayerItems.prototype.GetSelectedItemIndex=function() 
{
	return(this.SelectedItemIndex);
}