/***************************************************************************************
* CMMainContainer Class
*
* Main class to contain the other elements and objects that make up a CMMainContainer.
*
* The elements for a CMMainContainer are held in an array, 
* @module CMMainContainer
* @Copyright HSU, Jim Graham, 2019
***************************************************************************************/
//***************************************************************************************
// Static functions
//***************************************************************************************
// Functions to manage popup windows
//
// There can only be one popup window on the screen at a time.
//***************************************************************************************
/**
* Disable the context menu (regular menu that pops up when the user right-clicks on items in the browser.
* This only appears to work at the document level
*/
document.oncontextmenu = function(event)
{
	event.preventDefault();
	event.stopPropagation();
}
/*
* This function is inserted into event processing so we can hide popup windows when the
* user clicks anywhere in the document.
*/
document.onmousedown=function(TheEvent) 
{ "use strict";
	
	var TargetContainedInPopupWindow=false;
	var TargetIsPopupWindow=false;
	
	// see if the event is in a popup window
	for (var i=0;i<CMMainContainer.PopupWindow.length;i++) 
	{
		TargetContainedInPopupWindow=jQuery.contains( CMMainContainer.PopupWindow[i], TheEvent.target ) ;
		
		if (TheEvent.target===CMMainContainer.PopupWindow[i])
		{
			 TargetIsPopupWindow=true;
		}
	}
	
	if ((TargetContainedInPopupWindow===false)&&(TargetIsPopupWindow===false))
	{
	   CMMainContainer.HidePopupWindows();	
	}
};
document.onmousemove=function(TheEvent) 
{ "use strict";
	if (CMMainContainer.DraggingItem!=undefined)
	{
		if (!TheEvent) { TheEvent=window.event; }
	
		var Coordinate=CMUtilities.GetElementCoordinate(TheEvent.clientX,TheEvent.clientY,CMMainContainer.DraggingItem);
		var PixelX=Coordinate.x;
		var PixelY=Coordinate.y;
		
		var rect = CMMainContainer.DraggingItem.getBoundingClientRect();
		
		var Top=rect.top;
		
		var Height=PixelY;
		var Width=PixelX;
	
		if (CMMainContainer.Dragging=='s')
		{
			CMMainContainer.DraggingItem.style.height=Height+"px";
			CMMainContainer.DraggingItem.TheCanvasMap.Resize();
		}
		else if (CMMainContainer.Dragging=='e')
		{
			CMMainContainer.DraggingItem.style.width=Width+"px";
			CMMainContainer.DraggingItem.TheCanvasMap.Resize();
		}
		
		
	}
};
document.onmouseup=function(TheEvent) 
{ "use strict";
	if (CMMainContainer.DraggingItem!=undefined)
	{
		CMMainContainer.DraggingItem=undefined;
		CMMainContainer.Dragging=undefined;
		document.body.style.cursor="default";
	}
}

/**
* Sets the current popup window and hides any existing windows
* @public
* @param NewPopupWindow - the DOM element with the visible window
*/
CMMainContainer.SetPopupWindow=function(NewPopupWindow)
{ "use strict";
	CMMainContainer.HidePopupWindows();
	CMMainContainer.PopupWindow.push(NewPopupWindow);
};
/**
* Adds a new popup window.  Existing popup windows will remain open
*/
CMMainContainer.AddPopupWindow=function(NewPopupWindow)
{ "use strict";
	//CMMainContainer.HidePopupWindows();
	CMMainContainer.PopupWindow.push(NewPopupWindow);
};

/**
* Sets the current popup window and hides any existing windows
* @public
*/
CMMainContainer.HidePopupWindows=function() 
{ "use strict";
	for (var i=0;i<CMMainContainer.PopupWindow.length;i++)
	{
		CMMainContainer.PopupWindow[i].style.visibility="hidden";
	}
	CMMainContainer.PopupWindow=[];
};

//***************************************************************************************
// Definitions
// The following definitions should be treated as "static" and should not be 
// modified by users of CMMainContainer
//***************************************************************************************

/**
* Defniitions for the DOM elements within a CMMainContainer
* Indexes into the CMMainContainer.ELEMENT_DEFS array
* @public, @enum
*/
CMMainContainer.MAP_CONTAINER="MapContainer"; // main container with all the panels
CMMainContainer.TOOL_CONTAINER="ToolContainer"; // tools above map
CMMainContainer.TOOL_EDIT="ToolEdit"; // arrow tool
CMMainContainer.TOOL_INFO="ToolInfo"; /// "i" tool
CMMainContainer.TOOL_PAN="ToolPan"; // hand tool
CMMainContainer.CANVAS_CONTAINER="CanvasContainer"; // container for the canvas object
CMMainContainer.CANVAS="Canvas"; 
CMMainContainer.LAYER_LIST="LayerList"; // list of layers contained within the TAB_CONTAINER
CMMainContainer.MAP_FOOTER="MapFooter"; // footer with coordinates, SRS, and authors
CMMainContainer.MAP_COORDINATES="MapCoordinates";
CMMainContainer.MAP_SRS="SRS"; // 
CMMainContainer.MAP_CREDITS="Credits"; // 
CMMainContainer.NAVIGATION="Navigation"; // navigvation buttons (jjg no longer used)
CMMainContainer.BACKGROUND_LIST="BackgroundList";
CMMainContainer.SEARCH_PANEL="SearchPanel";
CMMainContainer.VERTICAL_TAB_CONTAINER="TabContainer";
CMMainContainer.HORIZIONAL_TAB_CONTAINER="HorizontalTabContainer";
CMMainContainer.SETTINGS_PANEL="SettingsPanel";
CMMainContainer.TIME_EDITOR_PANEL="TimeEditPanel";
CMMainContainer.ATTRIBUTE_PANEL="AttributePanel";
CMMainContainer.TIME_SLIDER_PANEL="TimeSliderPanel";
CMMainContainer.DATA_PANEL="DataPanel";

/**
* Definitions for the mobile device gesture events
*/
CMMainContainer.GESTURE_ZOOM=0.2; // how much to add to the zoom on a pinch event
CMMainContainer.GESTURE_PAN=8; // how many pixels to move the map on a pan event

// Global variable to count the total number of maps created on one web page.
// This value is saved as the index into each CMMainContainer and then incremented each time a map is created. 
// The value is also added to the end of all ElementIDs to make them unique for each map.

CMMainContainer.NumMaps=0; 

// keeps track of a popup windows that are currently displayed so it can be hidden if the
// user clicks anywhere in the document.

CMMainContainer.PopupWindow=[];

/**
* Definitions for the elements within the CMMainContainer.  
* Default for the HTML tag is DIV.
* @private
*/
CMMainContainer.ELEMENT_DEFS=
{
	MapContainer:{
		ID:"CM_MapContainer",
		ClassName:"CM_MapContainer",
	},
	ToolContainer:{
		ID:"CM_ToolContainer",
		ClassName:"CM_ToolContainer",
	},
	ToolEdit:{
		ID:"CM_ToolEdit",
		ClassName:"CM_Tool",
	},
	ToolInfo:{
		ID:"CM_ToolInfo",
		ClassName:"CM_Tool",
	},
	ToolPan:{
		ID:"CM_ToolPan",
		ClassName:"CM_Tool",
	},
	CanvasContainer:{
		ID:"CM_CanvasContainer",
		ClassName:"CM_CanvasContainer",
	},
	Canvas:{
		ID:"CM_Canvas",
		ClassName:"CM_Canvas",
		HTMLTag:"CANVAS",
	},
	LayerList:{
		ID:"CM_LayerList",
		ClassName:"CM_ButtonContent",
	},
	MapFooter:{
		ID:"CM_MapFooter",
		ClassName:"CM_MapFooter",
	},
	MapCoordinates:{
		ID:"CM_MapCoordinates",
		ClassName:"CM_Credits",
	},
	SRS:{
		ID:"CM_SRS",
		ClassName:"CM_Credits",
	},
	Credits:{
		ID:"CM_Credits",
		ClassName:"CM_Credits",
	},
	Navigation:{
		ID:"CM_Navigation",
		ClassName:"CM_Navigation",
	},
	BackgroundList:{
		ID:"CM_BackgroundList",
		ClassName:"CM_ButtonContent",
	},
	SearchPanel:{
		ID:"CM_SearchPanel",
		ClassName:"CM_ButtonContent",
	},
	TabContainer:{
		ID:"CM_TabContainer",
		ClassName:"CM_TabContainer",
	},
	HorizontalTabContainer:{
		ID:"CM_HorizontalTabContainer",
		ClassName:"CM_HorizontalTabContainer",
	},
	SettingsPanel:{
		ID:"CM_SettingsPanel",
		ClassName:"CM_SettingsPanel",
	},
	TimeEditPanel:{
		ID:"CM_TimeEditPanel",
		ClassName:"CM_TimeEditPanel",
	},
	AttributePanel:{
		ID:"CM_AttributePanel",
		ClassName:"CM_AttributePanel",
	},
	TimeSliderPanel:{
		ID:"CM_TimeSliderPanel",
		ClassName:"CM_TimeSliderPanel",
	},
	DataPanel:{
		ID:"CM_DataPanel",
		ClassName:"CM_DataPanel",
	},
}

/**
* Below are the settings definitions.
* @public, @settings
*/
CMMainContainer.SettingDefintions=
{
	MainContainer:
	{
		AllowMouseEvents: { Name:"Allow Mouse Events",Type:CMBase.DATA_TYPE_BOOLEAN, Default:true },
		MobileSupported: { Name:"Mobile Supported",Type:CMBase.DATA_TYPE_BOOLEAN, Default:false },
		Scene3D: { Name:"3D Scene",Type:CMBase.DATA_TYPE_BOOLEAN, Default:false },
		ImageFolder: { Name:"Image Folder",Type:CMBase.DATA_TYPE_STRING, Default:"../Images/" },
		CoordinateUnits: { Name:"Coordinate Units",Type:CMBase.DATA_TYPE_ENUMERATED,Options:[CMUtilities.COORDINATE_UNITS_DD,CMUtilities.COORDINATE_UNITS_DMS,CMUtilities.COORDINATE_UNITS_METERS,CMUtilities.COORDINATE_UNITS_FEET,CMUtilities.COORDINATE_UNITS_PIXELS,CMUtilities.COORDINATE_UNITS_ZOOM],Default:CMUtilities.COORDINATE_UNITS_DMS},
	},
};
//***************************************************************************************
// Constructors
//***************************************************************************************
/**
* Constructor for the main CMMainContainer object
* @public, @constructs
*/
function CMMainContainer() 
{ "use strict";
	CMBase.call(this);

	//*****************************
	// settings (move into settings?)
	
	this.ExistingElements=true; // true when we need to search for existing elements in the DOM
	
	this.Settings=
	{
		MainContainer:
		{
		},
	};
	
	//*****************************
	// other properties
	
	this.Index=CMMainContainer.NumMaps; // set the index for this map
	CMMainContainer.NumMaps++; // increment the index for the next map
	
	this.TheScene=null; // child?
	
	// Array for the elements in the CMMainContainer.  The array entrie will be initialized to undefined
	// when first used.  The user can set the entries to null to prevent an element from being created.
	this.Elements=null;
	
	this.Test=false; // flag for Jim to test stuff without breaking the map
	
	// panel objects for managing the map elements
	
	this.ToolPanel=null;
	this.PanelFooter=null;

	this.PanelSettings=null;
	this.PanelLayerList=null;
	this.PanelBackgrounds=null;
	this.PanelSearch=null;
}
CMMainContainer.prototype=Object.create(CMBase.prototype); // inherit prototype functions from PanelBase()

CMMainContainer.prototype.contructor=CMMainContainer; // override the constructor to go to ours

CMMainContainer.prototype.SetTest=function(New) { this.Test=New; };

//***************************************************************************************
// Private functions 
//***************************************************************************************
/*
* Takes over the mouse wheel.  CMMainContainer adds this as an event handler on initialization.
* @private
*/
CMMainContainer.MouseWheel=function(TheEvent)
{ "use strict";
	var Result; // return result is undefined typically
	
	CMMainContainer.HidePopupWindows(); // static function
	
	var TheEvent=window.event || TheEvent // grab the event passed in or globally (IE compatibility)
	
	var Result=this.TheCanvasMap.TheScene.GetView(0).MouseWheel(TheEvent);
		
	return(Result);
};
/*
* called by Initialize() below to initialize the entries in the Elements[] array that have
* not already been intialized by the user.
* @private
*/
CMMainContainer.prototype.Private_GetElements=function()
{
	// make sure the element dictionary has been created
	
	if (this.Elements==null) this.Elements={};
	
	for (var key in CMMainContainer.ELEMENT_DEFS) // get or create each of the elements
	{
		if (this.Elements[key]===undefined) // the element has not been initlized
		{
			var TheEntry=CMMainContainer.ELEMENT_DEFS[key];
			
			var ElementID=TheEntry.ID+"_"+this.Index;
			
			if (this.ExistingElements) // look for an existing element in the DOM
			{
				this.Elements[key]=document.getElementById(ElementID);
			}
			// the horizontal tab container is optional
			if (key!=CMMainContainer.HORIZIONAL_TAB_CONTAINER)
			{
				if (this.Elements[key]==undefined) // the element has not been created, create it now
				{
					var HTMLTag=CMMainContainer.ELEMENT_DEFS[key].HTMLTag;
					if (HTMLTag==undefined) HTMLTag="DIV";
					
					this.Elements[key]=document.createElement(HTMLTag);
					this.Elements[key].id=ElementID;
				}
				// if the element has a CSS class, set it now
				if (CMMainContainer.ELEMENT_DEFS[key].ClassName!=undefined)  // set the class
				{
					if (this.Elements[key].className=="")
					{
						this.Elements[key].className=CMMainContainer.ELEMENT_DEFS[key].ClassName;
					}
				}
			}
		}
	}
};
//******************************************************************
// CMBase Settings Functions
//******************************************************************

CMMainContainer.prototype.GetSettingsDefinitions=function() 
{
	var Result={};
	
	for (Key in CMMainContainer.SettingDefintions)
	{
		Result[Key]=CMMainContainer.SettingDefintions[Key];
	}

	return(Result); 
}
//***************************************************************************************
// CMMainContainer functions to be called before Initialize() is called.  These functions setup
// connections with the DOM objects and set the behavior of the map.
//***************************************************************************************
/**
* Sets the folder containing the images for the CMMainContainer interface
* Deprecated, use SetSetting("MainContainer","MobileSupported",MobileSupported) instead
* @public
* @param ImageFolder - path to the folder with the images
*/
CMMainContainer.prototype.SetImageFolder=function(ImageFolder)
{  "use strict";
	this.SetSetting("MainContainer","ImageFolder",ImageFolder);
//	this.ImageFolder=ImageFolder; 
};
/** 
* Changes the coordinate units
* Deprecated, use SetSetting("MainContainer","CoordinateUnits",CoordinateUnits) instead
* @public
* @param CoordinateUnits - one of the coordinate definitions:
* - CMUtilities.COORDINATE_UNITS_DD=0;
* - CMUtilities.COORDINATE_UNITS_DMS=1;
* - CMUtilities.COORDINATE_UNITS_METERS=2;
* - CMUtilities.COORDINATE_UNITS_FEET=3;
* - CMUtilities.COORDINATE_UNITS_PIXELS=4; // displays the pixel level coordinates for debugging
* - CMUtilities.COORDINATE_UNITS_ZOOM=5; // displays the zoom level for debugging
*/
CMMainContainer.prototype.SetCoordinateUnits=function(CoordinateUnits) 
{ 
	this.SetSetting("MainContainer","CoordinateUnits",CoordinateUnits);
	
	if (this.PanelFooter!=null)
	{
		this.PanelFooter.CoordinateUnits=CoordinateUnits;
	}
};

//***************************************************************************************
// Element functions
/*
* Function to provide an existnig element in place of the one that CMMainContainer 
* will create by default.  The user can also specify "null" which will prevent
* the element from being created.
* @public
* @param ElementIndex - CMMainContainer element definition for the element to set (i.e. CMMainContainer.MAP_CONTAINER).
* @param Element - the element to replace the standard CMMainContainer element
*/
CMMainContainer.prototype.SetElement=function(ElementIndex,Element)
{ 
	if (this.Elements==null) this.Elements={};
	
	if (typeof(Element)=="string") // an ID was specified, find the existing element.
	{
		Element=document.getElementById(Element);
	}
	// save the element to replace the default one.
	
	this.Elements[ElementIndex]=Element; 
};
/*
* Get the element from CMMainContainer based on the predefined indexes
* @public
* @param ElementIndex - CMMainContainer element definition for the element to set (i.e. CMMainContainer.MAP_CONTAINER).
* @returns - TheElement the DOM element for the specified element definition or NULL if unavailable.
*/
CMMainContainer.prototype.GetElement=function(ElementIndex)
{ 
	return(this.Elements[ElementIndex]); 
};

/**
* Helper function to turn off all the elements except the main map
*/
CMMainContainer.prototype.SimpleMap=function()
{ 
	this.SetElement(CMMainContainer.TOOL_CONTAINER,null); // turn off the tool bar below the title
	this.SetElement(CMMainContainer.NAVIGATION,null); // turn off the nagivation controls in the map 
	this.SetElement(CMMainContainer.VERTICAL_TAB_CONTAINER,null); // turn off the tab controls to the upper right of the map
	this.SetElement(CMMainContainer.LAYER_LIST,null); // hide the list of layers that is below the tab controls
	this.SetElement(CMMainContainer.BACKGROUND_LIST,null); // hide the background list
	this.SetElement(CMMainContainer.SEARCH_PANEL,null); // hide the search panel
	this.SetElement(CMMainContainer.SETTINGS_PANEL,null); // hide the search panel
	this.SetElement(CMMainContainer.DATA_PANEL,null); // hide the search panel
	this.SetElement(CMMainContainer.MAP_FOOTER,null); // hide the map footer at the bottom of the map
};

//***************************************************************************************
// Private functions
//***************************************************************************************

/*
* This is the function to call to initialize the CMMainContainer.  It sets up the member variables,
* links the objects together, and sets up the event handlers.
* @public 
* @param TheView - Optional 2D or 3D view.  If undefined, a 2D view is created, if true, a 3D view is created.
*/
CMMainContainer.prototype.Initialize=function(TheView)
{
	//if (AllowMouseEvents==undefined) AllowMouseEvents=true;
	
	// get the elements from the web page unless they have already been specified
	this.Private_GetElements();
		
	var ImageFolder=this.GetSetting("MainContainer","ImageFolder");
			
	//*****************************************************
	// setup the scene and view
	
	var TheGeo=null;
	
	if ((TheView==undefined)||(TheView instanceof CMView2D)) // no view specified or provided a 2D view
	{
		this.TheScene=new CMScene(this);
		TheGeo=new CMGeo(this.TheScene);
	
		if (TheView==undefined) TheView=TheView=new CMView2D(); // default
//		this.TheScene.AddView(TheView);
//		TheView.Setup(this.Elements[CMMainContainer.CANVAS]);
	}
	else // create a 3D view
	{
		this.TheScene=new CM3Scene();
		TheGeo=new CM3Geo(this.TheScene);
		
		if (((TheView instanceof CM3View)==false) ) //&& ((TheView instanceof CM3Camera)==false)
		{
			TheView=new CM3View( );
		}
	}
	this.TheScene.SetParent(this);
	
	this.TheScene.AddView(TheView);
	this.TheScene.AddChild(TheGeo);
	
	TheView.Setup(this.Elements[CMMainContainer.CANVAS],this.Elements[CMMainContainer.CANVAS_CONTAINER]);
	
	// This is the map container which is provided by the user
	
	var MapContainer=this.Elements[CMMainContainer.MAP_CONTAINER];
	MapContainer.TheCanvasMap=this;
	MapContainer.OriginalMouseDown=MapContainer.onmousedown;

	var CanvasContainer=this.Elements[CMMainContainer.CANVAS_CONTAINER];
	if  (CMUtilities.IsDefined(CanvasContainer)) 
	{
		MapContainer.appendChild(CanvasContainer);
	}
	var TheCanvasElement=this.Elements[CMMainContainer.CANVAS];
	if  (CMUtilities.IsDefined(TheCanvasElement)) 
	{
		CanvasContainer.appendChild(TheCanvasElement);
	}

	//******************************************************
	// tools
	
	var ToolContainer=this.Elements[CMMainContainer.TOOL_CONTAINER];
	if  (CMUtilities.IsDefined(ToolContainer))
	{
		MapContainer.appendChild(ToolContainer);
	}
	
	//**************************************************
	// vertical buttons
	
	var TabContainer=this.Elements[CMMainContainer.VERTICAL_TAB_CONTAINER]; // VERTICAL_TAB_CONTAINER is the content container
	if  (CMUtilities.IsDefined(TabContainer)) 
	{
		this.TabPanel=new CMPanelButtons(this);
		
		this.TabPanel.SetElement(TabContainer);
	
		MapContainer.appendChild(TabContainer);
		
		// panel for layers
		
		var LayerListElement=this.Elements[CMMainContainer.LAYER_LIST];
		if  (CMUtilities.IsDefined(LayerListElement)) 
		{
			this.PanelLayerList=new CMPanelLayerList(this);
		
			var TheLayerListTab=this.TabPanel.AddTab("Layers",ImageFolder+"Icon_Layers.png",undefined,LayerListElement,MapContainer);
			
			this.PanelLayerList.SetElement(LayerListElement); // connect the settings panel to it's DIV element
		}
		
		// panel for background layers
		
		var PanelBackgroundsElement=this.Elements[CMMainContainer.BACKGROUND_LIST];
		if  (CMUtilities.IsDefined(PanelBackgroundsElement)) 
		{
			this.PanelBackgrounds=new CMPanelBackgrounds(this);
		
			var TheBackgroundsTab=this.TabPanel.AddTab("Background",ImageFolder+"Icon_Background_Default.png",null,PanelBackgroundsElement,MapContainer);
			
			this.PanelBackgrounds.SetElement(PanelBackgroundsElement); // connect the settings panel to it's DIV element
		}
		
		// panel for search
		
		var SearchPanelElement=this.Elements[CMMainContainer.SEARCH_PANEL];
		if  (CMUtilities.IsDefined(SearchPanelElement)) 
		{
			this.PanelSearch=new CMPanelSearch(this);
		
			var TheSearchTab=this.TabPanel.AddTab("Search",ImageFolder+"Icon_Search.png",null,SearchPanelElement,MapContainer);
			
			this.PanelSearch.SetElement(SearchPanelElement); // connect the settings panel to it's DIV element
		}
	}

	//************************************************************************
	// Add the tools container
	// The positioning of the tools is set within the CMMainContainer CSS file	
	var ToolContainer=this.GetElement(CMMainContainer.TOOL_CONTAINER);
	if (CMUtilities.IsDefined(ToolContainer))
	{
		// create the tool panel object and link it to its DIV element
		
		this.ToolPanel=new CMPanelTool(this);
		
		this.ToolPanel.SetElement(ToolContainer);
		
		// add the select/arrow tool

		var EditTool=this.GetElement(CMMainContainer.TOOL_EDIT);
		
		if (CMUtilities.IsDefined(EditTool))
		{
			var TheTool=this.ToolPanel.AddTool(EditTool,ImageFolder+"IconArrow_20w_Default.png",
				ImageFolder+"IconArrow_20w_Selected.png",this,
				'Click to get information on features or drag the map',function(TheCanvasMap) 
			{ 
				TheCanvasMap.SelectTool(CMView.TOOL_SELECT); // create a new select tool
			});
			TheTool.CMMainContainer=this;
		}
		
		// add the select/arrow tool
		
		var PanTool=this.GetElement(CMMainContainer.TOOL_PAN);
		
		if (CMUtilities.IsDefined(PanTool))
		{
			var TheTool=this.ToolPanel.AddTool(PanTool,ImageFolder+"IconHand_20w_Default.png",
				ImageFolder+"IconHand_20pixels_Selected.png",this,
				'Click to get information on features or drag the map',function(TheCanvasMap) 
			{ 
				TheCanvasMap.SelectTool(CMView.TOOL_HAND); // create a new select tool
			});
			TheTool.CMMainContainer=this;
		}
		
		// add the select/arrow tool
		
		var InfoTool=this.GetElement(CMMainContainer.TOOL_INFO);
		
		if (CMUtilities.IsDefined(InfoTool))
		{
			var TheTool=this.ToolPanel.AddTool(InfoTool,ImageFolder+"Icon_I_Default.png",
				ImageFolder+"Icon_I_Selected.png",this,
				'Click to get information on features or drag the map',function(TheCanvasMap) 
			{ 
				TheCanvasMap.SelectTool(CMView.TOOL_INFO); // create a new select tool
			});
			TheTool.CMMainContainer=this;
		}
		this.ToolPanel.MakeToolGroup([EditTool,PanTool,InfoTool]);
		
		// add zoom in tool
		
		var ZoomInButton=document.createElement("DIV");
		ZoomInButton.id="CMZoomIn_0";
		ZoomInButton.className="CM_Tool";
		
		var TheTool=this.ToolPanel.AddTool(ZoomInButton,ImageFolder+"Icon_ZoomIn_Small_17H.png",
			ImageFolder+"Icon_ZoomIn_Small_17H.png",this,
			'Click to zoom into the map',function(TheCanvasMap) 
		{ 
			this.CMMainContainer.GetScene().GetView(0).ZoomIn();
		});
		TheTool.CMMainContainer=this;
		
		// add home button
		
		var HomeButton=document.createElement("DIV");
		HomeButton.id="CMHome_0";
		HomeButton.className="CM_Tool";
		
		var TheTool=this.ToolPanel.AddTool(HomeButton,ImageFolder+"Icon_HomeExtent_small_17H.png",
			ImageFolder+"Icon_HomeExtent_small_17H.png",this,
			'Click to zoom to the full extent of the map',function(TheCanvasMap) 
		{ 
			this.CMMainContainer.GetScene().GetView(0).ZoomToMaxBounds();
		});
		TheTool.CMMainContainer=this;
		
		// add zoom out tool
		
		var ZoomOutButton=document.createElement("DIV");
		ZoomOutButton.id="CMZoomOut_0";
		
		var TheTool=this.ToolPanel.AddTool(ZoomOutButton,ImageFolder+"Icon_ZoomOut_Small_17H.png",
			ImageFolder+"Icon_ZoomOut_Small_17H.png",this,
			'Click to zoom out of the map',function(TheCanvasMap) 
		{ 
			this.CMMainContainer.GetScene().GetView(0).ZoomOut();
		});
		TheTool.CMMainContainer=this;
		
		this.ToolPanel.MakeToolGroup([ZoomInButton,HomeButton,ZoomOutButton]);
	}
	//*****************************************************************************
	// footer
	
	var MapFooter=this.Elements[CMMainContainer.MAP_FOOTER];
	if  (CMUtilities.IsDefined(MapFooter)) 
	{
		// create the tool panel object and link it to its DIV element
		
		this.PanelFooter=new CMPanelFooter(this);
		
		this.PanelFooter.SetElement(MapFooter);
		
		MapContainer.appendChild(MapFooter);
	}
	// horiziontal tab container
	
 	var HoriziontalTabContainerElement=this.Elements[CMMainContainer.HORIZIONAL_TAB_CONTAINER];
	if  (CMUtilities.IsDefined(HoriziontalTabContainerElement)) 
	{
		// create the horizontal tab panel container
		
		this.HoriziontalTabContainer=new CMPanelTabs(this);
		this.HoriziontalTabContainer.SetElement(HoriziontalTabContainerElement);
		
		// time editor
		var TimeEditorPanelElement=this.Elements[CMMainContainer.TIME_EDITOR_PANEL];
		if  (CMUtilities.IsDefined(TimeEditorPanelElement)) 
		{
			this.PanelTimeline=new CMPanelTime(this);
			this.PanelTimeline.SetElement(TimeEditorPanelElement);
		
			var TimeEditorTab=this.HoriziontalTabContainer.AddTab("Timeline","Timeline",null,TimeEditorPanelElement);
		
			// link the settings panel to the TimeEditorPanal (jjg - use messages instead?)
			
			var TheSettingsPanel=this.GetPanelSettings();
			this.PanelTimeline.SetSettingsPanel(TheSettingsPanel); // connect the settings panel to the time editor panel
		}
		// attributes
		
	 	var AttributePanelElement=this.Elements[CMMainContainer.ATTRIBUTE_PANEL];
		if  (CMUtilities.IsDefined(AttributePanelElement)) 
		{
			var TheAttributePanel=new CMPanelAttributes(this);
			TheAttributePanel.SetElement(AttributePanelElement);
		
			var AttributesTab=this.HoriziontalTabContainer.AddTab("Attributes","Attributes",null,AttributePanelElement);
		}
		// settings
		
	 	var SettingsPanelElement=this.Elements[CMMainContainer.SETTINGS_PANEL];
		if  (CMUtilities.IsDefined(SettingsPanelElement)) 
		{
			var TheSettingsPanel=new CMPanelSettings(this);
			TheSettingsPanel.SetElement(SettingsPanelElement);
		
			var SettingsTab=this.HoriziontalTabContainer.AddTab("Settings","Settings",null,SettingsPanelElement);
		}
		
		// data
		
	 	var DataPanelElement=this.Elements[CMMainContainer.DATA_PANEL];
		if  (CMUtilities.IsDefined(DataPanelElement)) 
		{
			var TheDataPanel=new CMPanelData(this);
			TheDataPanel.SetElement(DataPanelElement);
		
			var DataTab=this.HoriziontalTabContainer.AddTab("Data","Data",null,DataPanelElement);
		}
		
	}
	
	//*****************************************************
	// call jQuery to add the mouse handles to the canvas element
	
	if (this.GetSetting("MainContainer","AllowMouseEvents",true))
	{
		TheView.AddMouseEventHandlers();
		TheView.AddKeyEventHandlers();
	}
	//*************************************************************************************
	//add events for the map container
	
	MapContainer.addEventListener("mousemove",function(TheEvent) 
	{
		if (!TheEvent) { TheEvent=window.event; }
	
		var Coordinate=CMUtilities.GetElementCoordinate(TheEvent.clientX,TheEvent.clientY,this);
		var PixelX=Coordinate.x;
		var PixelY=Coordinate.y;
		
		var Height=this.offsetHeight;
		var Width=this.offsetWidth;
		
		if (Math.abs(Height-PixelY)<10) 
		{
			this.style.cursor="s-resize";
		}
		else if (Math.abs(Width-PixelX)<10) 
		{
			this.style.cursor="e-resize";
		}
		else 
		{
			this.style.cursor="default";
		}
	});
	MapContainer.addEventListener("mousedown",function(TheEvent) 
	{
		if (!TheEvent) { TheEvent=window.event; }
	
		var Coordinate=CMUtilities.GetElementCoordinate(TheEvent.clientX,TheEvent.clientY,this);
		var PixelX=Coordinate.x;
		var PixelY=Coordinate.y;
		
		var Height=this.offsetHeight;
		var Width=this.offsetWidth;
		
		if (Math.abs(Height-PixelY)<10) 
		{
			CMMainContainer.DraggingItem=this;
			CMMainContainer.Dragging='s';
			document.body.style.cursor="s-resize";
		}
		else if (Math.abs(Width-PixelX)<10) 
		{
			CMMainContainer.DraggingItem=this;
			CMMainContainer.Dragging='e';
			document.body.style.cursor="e-resize";
		}
		else 
		{
			document.body.style.cursor="default";
		}
	});
		
	//*********************************************************
	// setup gesture support using hammer.js
	
	var CanvasContainer=this.GetElement(CMMainContainer.CANVAS_CONTAINER);

	if (this.GetSetting("MainContainer","MobileSupported"))
	{
		TheView.AddMobileEvents();
	}
};
//***************************************************************************************
// CMBase Functions
//***************************************************************************************

/**
* Sets an individual setting value into the settings
* @public
* @param Group - Group for the setting
* @param Key - on of the CMLayer.INFO enumerated types
* @param Value - value for the type (see the documentation for types for each of the properties)
*/
CMMainContainer.prototype.SetSetting=function(Group,Key,Value)
{
	try
	{
		this.Settings[Group][Key]=Value; // save them into the object
	}
	catch(err)
	{
		CMMainContainer.Error("Sorry, we could not find the group '"+Group+"' or the Key '"+Key+"' in this objects settings");
	}
//	this.SettingsChanged();
}
/**
* Get an individual value from the settings
* @public
* @param Group - Group for the setting
* @param Key - on of the CMLayer.INFO enumerated types
* @param Default - default value to use in none has been specified as of yet (optional)
* @returns Value - curnet property value.
*/
CMMainContainer.prototype.GetSetting=function(Group,Key,Default)
{
	var Result=CMMainContainer.SettingDefintions[Group][Key].Default;
	
	if (Default!=undefined) Result=Default;
	
	var TheGroup=this.Settings[Group];
	
	if (TheGroup!=undefined)
	{
		if (Key in TheGroup) Result=TheGroup[Key];
	}
	return(Result);
}
//***************************************************************************************
// CMMainContainer functions that can be called after initialize()
//***************************************************************************************

//***************************************************************************************
// Gets and sets

/**
* Get the scene object that contains the layers for the map
* @public
* @returns TheScene - the current scene for this map.
*/
CMMainContainer.prototype.GetScene=function() { return(this.TheScene); };

/**
* Get the view for the CMMainContainer.  When using more than one view, 
* get the scene and then use the Scene's GetNumViews() and GetView(Index)
* functions.
*
* @public
* @returns TheView - the current view for this scene.
*/
CMMainContainer.prototype.GetView=function() { return(this.TheScene.GetView(0)); };

CMMainContainer.prototype.GetToolPanel=function() { return(this.ToolPanel); };

CMMainContainer.prototype.GetPanelFooter=function() { return(this.PanelFooter); };

CMMainContainer.prototype.GetPanelSettings=function() { return(this.PanelSettings); };

CMMainContainer.prototype.SetPanelSettings=function(NewPanelSettings) { this.PanelSettings=NewPanelSettings; };

CMMainContainer.prototype.GetPanelTimeline=function() { return(this.PanelTimeline); };


/**
* Set the projection for coordinate conversion
* @public
* @param TheProjector - the projector to use to project coordinates
*/
CMMainContainer.prototype.SetProjector=function(TheProjector) { this.TheScene.SetProjector(TheProjector); };
CMMainContainer.prototype.GetProjector=function() { return(this.TheScene.GetProjector()); };

/**
* Set the current tool selected for interacting with the map
* @public
* @param ViewToolDef - sets the current tool (e.g. CMView.TOOL_INFO)
*/
CMMainContainer.prototype.SelectTool=function(ViewToolDef) 
{ 
	if (this.TheScene.GetView(0)==null) { alert("Sorry, CMMainContainer.SelectTool() cannot be called until after Initialize() is called()"); }
	this.TheScene.GetView(0).SetTool(ViewToolDef);
};
//***************************************************************************************
// Add layers

/*
* Add a layer to the map.  The layer will be on top of other layers
* @public
* @param NewLayer - CMLayer object to add to the map
*/
CMMainContainer.prototype.AddLayer=function(NewLayer) 
{ 
	var LayerIndex=-1;
	
	if (this.TheScene==null) { alert("Sorry, CMMainContainer.AddLayer() cannot be called until after Initialize() is called()"); }
	else 
	{
		LayerIndex=this.TheScene.AddLayer(NewLayer);
	}
	return(LayerIndex);
};
/*
* Add background to the list
* @public
* @param NewLayer - CMLayer object to add to the backgrounds.  Only one background will
*  appear and it will always be behind the other layers.
*/
CMMainContainer.prototype.AddBackground=function(NewLayer) 
{ 
	if (this.TheScene==null) { alert("Sorry, CMMainContainer.AddBackground() cannot be called until after Initialize() is called()"); }
	else 
	{
		this.TheScene.AddBackground(NewLayer);
	}
};
/*
* Add background to the list
* @public
* @param NewLayer - CMLayer object to add to the backgrounds.  Only one background will
*  appear and it will always be behind the other layers.
*/
CMMainContainer.prototype.AddBackgrounds=function(Backgrounds,SelectedIndex) 
{ 
	var TheScene=this.GetScene();
	
	var Layer_World;
	
	for (var i=0;i<Backgrounds.length;i++)
	{
		Layer_World=new CMLayerDataset();
		
		TheScene.AddBackground(Layer_World);
		Layer_World.SetName(Backgrounds[i].Name);
		
		// only make the layer hidden if it is not selected
		if (i!=SelectedIndex) Layer_World.SetSetting("Item","Status",CMItem.STATUS_HIDDEN);

		if (Backgrounds[i].NorthPoleColor!=undefined) Layer_World.SetSetting("Dataset","NorthPoleColor",Backgrounds[i].NorthPoleColor);
		if (Backgrounds[i].SouthPoleColor!=undefined) Layer_World.SetSetting("Dataset","SouthPoleColor",Backgrounds[i].SouthPoleColor);
		Layer_World.SetSetting("Layer","Metadata",Backgrounds[i].Metadata);
		Layer_World.SetSetting("Dataset","URL",Backgrounds[i].URL);
		Layer_World.SetSetting("Dataset","Format",Backgrounds[i].Format);
		
		Layer_World.RequestData();
	}
	
	TheScene.SetSelectedBackgroundIndex(SelectedIndex);
}

//********************************************************************************
// StartMap function
//********************************************************************************

/*
* StartMap() should be called after all the layers are added to start up the map and
* fill out the layer list.  Layers can be added and removed later as well but the user
* may see them being added.
* @public
* @param ResizeFlag - If true, the map will be resized right away.  Otherwise, the size
*  of the canvas map container will be used for the canvas.
*/
CMMainContainer.prototype.StartMap=function(ResizeFlag) 
{
	if ((typeof(CM3Scene)!="undefined")&&(this.TheScene instanceof CM3Scene))
	{
		this.TheScene.Start();
	}
	// select layers tab to start
	
	if  (CMUtilities.IsDefined(this.TabPanel)) 
	{
//		this.TabPanel.SelectTab("Layers");
	}
	// setup resize
	
	if (ResizeFlag) 
	{
		this.Resize();
	}
	else 
	{
		// the canvas aspect ratio is not correct unless we set the "width" and "height" of the element
		// rather than the style.  This must be done or it will display maps distorted
		
		this.TheScene.Resize();
	}
};

//*************************************************************************************
// Functions called by GUI widgets
//*************************************************************************************
/*
* called to zoom the map in by 2x
* @public
*/
CMMainContainer.prototype.ZoomIn=function() 
{
	if (this.TheScene.GetView(0)==null) { alert("Sorry, CMMainContainer.ZoomIn() cannot be called until after Setup() is called()"); }
	else this.TheScene.GetView(0).ZoomIn();
};
/*
* called to zoom the map out by 2x
* @public
*/
CMMainContainer.prototype.ZoomOut=function() 
{
	if (this.TheScene.GetView(0)==null) { alert("Sorry, CMMainContainer.ZoomOut() cannot be called until after Setup() is called()"); }
	else this.TheScene.GetView(0).ZoomOut();
};
/**
* Zooms the map to the maximum specified value
* @public
*/
CMMainContainer.prototype.ZoomToMax=function() 
{
	var SceneBounds=this.TheScene.GetBounds();
	
	if (SceneBounds!=null)
	{
		this.TheScene.GetView(0).ZoomToBounds(SceneBounds);
	}
};
/*
* called to zoom the map to a specific area of the scene.
* @public
* @param TheBounds - boundary object { XMin,XMax,YMin,YMax}
*/
CMMainContainer.prototype.ZoomToBounds=function(TheBounds) 
{ 
	if (this.TheScene.GetView(0)==null) { alert("Sorry, CMMainContainer.ZoomToBounds() cannot be called until after Setup() is called()"); }
	else this.TheScene.GetView(0).ZoomToBounds(TheBounds); 
};
/**
* Zooms to the specified zoom level
* @public
* @param ZoomLevel - level to zoom to, 1=1 pixel per map unit, 2=pixels per map unit, etc.)
*/
CMMainContainer.prototype.ZoomTo=function(ZoomLevel) 
{ 
	if (this.TheScene.GetView(0)==null) { alert("Sorry, CMMainContainer.ZoomTo() cannot be called until after Setup() is called()"); }
	else this.TheScene.GetView(0).ZoomTo(ZoomLevel); 
};
/**
* Centers the map at RefX,RefY in the first view
* @public
* @param RefX - Longitude or easting for the center of the map
* @param RefY - Latitude or northing for the center of the map
*/
CMMainContainer.prototype.SetRefCenter=function(RefX,RefY) 
{ 
	if (this.TheScene.GetView(0)==null) { alert("Sorry, CMMainContainer.SetRefCenter() cannot be called until after Setup() is called()"); }
	else this.TheScene.GetView(0).SetRefCenter(RefX,RefY); 
};

/**
* Called to resize the map when the window size changes.  
* The only reason this is needed is because
* we have to use JavaScript to resize the Canvas HTML element, CSS does not work.
* @public
*/
CMMainContainer.prototype.Resize=function() 
{
	CMMainContainer.HidePopupWindows();
	
	this.TheScene.Resize();
};
CMMainContainer.Error=function(Message) 
{
	throw(Message);
//	alert("Message");
};
