/******************************************************************************************************************
* CMNorthArrow
* JS class for creating north arrows that can point north.
*
* @module CMNorthArrow
* @Copyright HSU, Jim Graham, 2019
******************************************************************************************************************/
//***************************************************************************************
// Global Definitions
//***************************************************************************************
CMNorthArrow.SettingDefintions=
{
	NorthArrow:
	{
		Scale: { Name:"Scale",Type:CMBase.DATA_TYPE_FLOAT, Default:1 },
		ClassName: { Name:"Class Name",Type:CMBase.DATA_TYPE_STRING, Default:"CM_NorthArrow" },
		URL: { Name:"URL",Type:CMBase.DATA_TYPE_STRING, Default:null },
	},
}

//***************************************************************************************
// Constructors
//***************************************************************************************
/*
* 
*/
function CMNorthArrow() 
{
	CMItem.call(this);
	
	// settings
	this.TimeSlices[0].Settings.NorthArrow=	
	{
	};
	// other properties
	this.AngleInRadians=0;
	
	this.OriginalWidth=0;
	this.OriginalHeight=0;
}

CMNorthArrow.prototype=Object.create(CMItem.prototype); // inherit prototype functions from CMBase()

CMNorthArrow.prototype.contructor=CMNorthArrow; // override the constructor to go to ours


//******************************************************************
// Private functions
//******************************************************************
/**
* @private
*/
CMNorthArrow.prototype.SetupAngle=function(TheView)
{
	this.AngleInRadians=0;
	
	if (this.TheContainer!=undefined)
	{
		var TheMainContainer=this.GetParent(CMMainContainer);
		var TheProjector=TheMainContainer.GetProjector();
		
		var TheNorthArrow=this.TheContainer.childNodes[0];
		
		var TheRect=this.TheContainer.getBoundingClientRect();
		
		var CenterX=this.TheContainer.offsetLeft+(this.TheContainer.offsetWidth)/2;
		var CenterY=this.TheContainer.offsetTop+(this.TheContainer.offsetHeight)/2;
		
		var Easting=TheView.GetRefXFromPixelX(CenterX);
		var Northing1=TheView.GetRefYFromPixelY(CenterY-5);
		var Northing2=TheView.GetRefYFromPixelY(CenterY+5);
		
		if (isNaN(Easting)||isNaN(Northing1)||isNaN(Northing2))
		{
			this.AngleInRadians=0;
		}
		else
		{
			var Result1=TheProjector.ProjectToGeographic(Easting,Northing1);
			var Result2=TheProjector.ProjectToGeographic(Easting,Northing2);
			
			if ((Result1!=null)&&(Result2!=null))
			{
				var angleRadians = Math.atan2( Result2[0] - Result1[0],Result2[1] - Result1[1]);
			
				this.AngleInRadians=Math.PI-angleRadians;
			}
		}
	}
}
/**
* Sets the transform to rotate the north arrow based on the map projection and location
* @private
*/
CMNorthArrow.prototype.SetTransform=function()
{
	if (this.TheContainer!=undefined)
	{
		var TheGraphic=this.TheContainer.childNodes[0];
		
		if (TheGraphic!=undefined)
		{
			var AngleInDegrees=this.AngleInRadians*180/Math.PI;
			
			var Width=TheGraphic.clientWidth;
			var Height=TheGraphic.clientHeight;
			
			var Scale=this.GetSetting("NorthArrow","Scale",1);
			
			// get the amount to translate the graphic to keep it within the container
			var TranslateX=this.OriginalWidth/2*(Scale-1);//this.OriginalWidth/Scale; // 0 at Scale=1, 32 at Scale=2, 64 at Scale=3, 96 at Scale=4
			var TranslateY=this.OriginalHeight/2*(Scale-1);//this.OriginalHeight/Scale;
			
			var Transform=" translate("+TranslateX+","+TranslateY+") scale("+Scale+") rotate("+AngleInDegrees+")";
			
			TheGraphic.setAttribute("transform",Transform );
			
			// setup the width and height of the container
			var Width=Width*Scale;
			var Height=Height*Scale;
			
	//		this.TheContainer.style.top=TranslateY+"px";
	//		this.TheContainer.style.left=TranslateX+"px";
			
//			this.TheContainer.style.zIndex = "1000";
//			this.TheContainer.style.border = "thick solid #0000FF";
			
			this.TheContainer.style.width=Width+"px";
			this.TheContainer.style.height=Height+"px";
		}
	}
}
/**
* Sets up the north arrow the first time it is used
* @private
*/

CMNorthArrow.prototype.SetupNorthArrow=function()
{
	var TheMainContainer=this.GetParent(CMMainContainer);
	var TheCanvasContainer=TheMainContainer.GetElement(CMMainContainer.CANVAS_CONTAINER);
	
	if (this.TheContainer==undefined)
	{
		// add the DIV container
		
		var TheContainer=document.createElement("div");
		TheContainer.className=this.GetSetting("NorthArrow","ClassName","CM_NorthArrow");
		
		TheCanvasContainer.appendChild(TheContainer);
		
		this.TheContainer=TheContainer;
//		this.TheContainer.style.zIndex = 99999999;
//		this.TheContainer.style.border = "thick solid #ff00FF";

		TheContainer.innerHTML=this.SVGText;
		
		var TheGraphic=this.TheContainer.childNodes[0];
		
		this.OriginalWidth=TheGraphic.clientWidth;
		this.OriginalHeight=TheGraphic.clientHeight;
		
		this.SetTransform();
		
		// setup the container to match the svg graphic
		
		// add the SVG element to the Div container
/*		
		var TheSVGElement=document.createElement("svg");
		TheSVGElement.width=Width;
		TheSVGElement.height=Height;
		TheSVGElement.setAttribute("transform", "scale(4)");
		
		TheContainer.appendChild(TheSVGElement);
		
		this.SVGElement=TheSVGElement;
		
		// add the text item
		
		var TheTextElement=document.createElement("text");
		TheTextElement.x="120";
		TheTextElement.y="150";
		TheTextElement.stroke="black";
		TheTextElement.strokeWidth="1";
		TheTextElement.fill="white";
		TheTextElement.pointerEvents="visible";
		
		TheTextElement.innerHTML=this.SVGText;
		TheTextElement.setAttribute("transform", "scale(4)");
		
		TheSVGElement.appendChild(TheTextElement);
		this.TheTextElement=TheTextElement;
*/		
	}
	
/*	var TheArray=CMNorthArrow.Coordinates;
	
	var ctx=TheView.TheContext;
	
	ctx.translate(this.x,this.y);
	ctx.rotate(this.AngleInRadians);
	
	ctx.moveTo(+TheArray[0][0],TheArray[0][1]);
										  
	for (var i=0;i<TheArray.length;i++)
	{
		ctx.lineTo(TheArray[i][0],TheArray[i][1]);
	}
	ctx.fill();
	ctx.stroke();
	
	ctx.rotate(-this.AngleInRadians);
	ctx.translate(-this.x,-this.y);
*/	
}
//******************************************************************
// CMItem functions
//******************************************************************
CMNorthArrow.prototype.Paint=function(TheView)
{
	if (this.TheContainer!=undefined)
	{
		this.SetupAngle(TheView);
		
		this.SetTransform();
	}
}
//******************************************************************
// CMNorthArrow functions
//******************************************************************
CMNorthArrow.prototype.RequestData=function() 
{
	var URL=this.GetSetting("NorthArrow","URL");
	this.SetURL(URL);
}

/*
* Called to obtain the data for the layer from a URL.
* Currently,only GeoJSON is supported as the transfer type.
* @private
*/
CMNorthArrow.prototype.SetURL=function(URL) 
{
	this.URL=URL;
	
	var TheRequest=new XMLHttpRequest(); // wait staff at the resturant
	TheRequest.open("GET",URL,true); // the URL is what we ordered
	TheRequest.TheURL=URL;
	TheRequest.ThisNorthArrow=this;
			
	TheRequest.onreadystatechange=function() 
	{
		if (this.readyState == 4)  // done
		{
			if (this.status == 200) // OK
			{
				this.ThisNorthArrow.SVGText=TheRequest.responseText;
				
				this.ThisNorthArrow.SetupNorthArrow();
			}
			else alert("HTTP error "+this.status+" "+this.statusText+" ("+this.TheURL+")");
		}
	}
	TheRequest.send();
};





