/******************************************************************************************************************
* SetupSettingsPanel
*
* @module SetupSettingsPanel
* @Copyright HSU, Jim Graham, 2019
******************************************************************************************************************/
/*-- Changes check box size and margin --*/
CMPanelBackgrounds.LAYER_LIST_ITEM_HEIGHT=24;
CMPanelBackgrounds.LAYER_POPUP_MENU_ITEM_HEIGHT=24;

//******************************************************************
// Constructor
//******************************************************************

function CMPanelBackgrounds(TheCanvasMap) 
{
	CMBase.call(this);

	if (TheCanvasMap==undefined) alert("Sorry, the CMPanelBackgrounds requires a CanvasMap object on construction");
	
	this.SetParent(TheCanvasMap);
	
	var TheScene=TheCanvasMap.GetScene();
	TheScene.AddListener(CMScene.MESSAGE_BACKGROUNDS_CHANGED,this,function(TheScene,TheListener,AdditionalInfo)
	{
		TheListener.Setup(TheScene);
	});
	
	// Properties
	this.TheElement=null;
	
	this.LayerListItemHeight=CMPanelBackgrounds.LAYER_LIST_ITEM_HEIGHT;
	this.LayerPopupMenuItemHeight=CMPanelBackgrounds.LAYER_POPUP_MENU_ITEM_HEIGHT;
}

CMPanelBackgrounds.prototype=Object.create(CMBase.prototype); // inherit prototype functions from CMBase()

CMPanelBackgrounds.prototype.contructor=CMPanelBackgrounds; // override the constructor to go to ours
//******************************************************************
// Private Functions
//******************************************************************

//******************************************************************
// Functions
//******************************************************************

CMPanelBackgrounds.prototype.SetElement=function(TheElement)
{
	this.TheElement=TheElement;
}
/**
* Setup the properties panel for a selected object
*/
CMPanelBackgrounds.prototype.Setup=function(TheScene)
{
	var TheElement=this.TheElement;
	
	// first, remove all the existing elements from the element
	while (TheElement.firstChild) // while there is a first element in the dialog
	{
		// removing the first element moves the next element to the first position
		// so this little loop will remove all the elements from another element
		TheElement.removeChild(TheElement.firstChild);
	}
	
	var Left=TheElement.style.left;
	var Top=TheElement.style.top;
	
	// TheElement.style.borderColor="#cccccc"; // border color for the layers list. style moved to CanvasMap.css.
	Left=0;
	Top=6;
	var TheGeo=TheScene.GetChild(0,CMGeo);
	
	for (var i=0;(i<TheGeo.GetNumBackgrounds());i++)
	{
		var TheBackground=TheGeo.GetBackground(i);
		
		var LayerInListTop=Top+(i*this.LayerListItemHeight);
		
		// create the overall div tag for the layer in the list
		
		var LayerInList=document.createElement('div');
		LayerInList.className="CM_BackgroundListItemClass";
	
		var LayerListWidth=jQuery(TheElement).outerWidth(false);
	
		CMUtilities.AbsolutePosition(LayerInList,Left+2,LayerInListTop,LayerListWidth,this.LayerListItemHeight);
		
		TheElement.appendChild(LayerInList);
	
		//*******************************************************
		// add the check box
		
		var TheCheckBox=document.createElement('input');
		TheCheckBox.className="CM_BackgroundListRadioButtonClass";
		TheCheckBox.name="Background";
		TheCheckBox.value=TheBackground.Name;
		TheCheckBox.type="radio";
//		TheCheckBox.style.width="20%";
		
		TheCheckBox.TheGeo=TheGeo;
		TheCheckBox.TheBackground=TheBackground;
		TheCheckBox.ThisIndex=i;
		
		TheCheckBox.addEventListener('click', function()
		{
			if (this.checked)
			{
				this.TheGeo.SetSelectedBackgroundIndex(this.ThisIndex);
			}
		});
		TheCheckBox.checked=false;
		
		if (TheGeo.SelectedBackgroundIndex==i) TheCheckBox.checked=true;
		
		// debugging
	
		CMUtilities.AbsolutePosition(TheCheckBox,Left+2,-6,14,this.LayerListItemHeight);
		
		// Sets the position of the check boxes
		
		LayerInList.appendChild(TheCheckBox);

		//****************************************************************
		// add the name
		
		var TheLayerName=document.createElement('div');
		TheLayerName.className="CM_BackgroundListNameClass";
		
		var Name=TheBackground.GetSetting("Item","Name","Unnamed"); 
		
		TheLayerName.innerHTML=Name;
		
		//***************************************************************************
		// event listener for when the user right clicks on the list
		// - regular click is for moving, right click for opening the menu
		
		TheLayerName.TheLayer=TheBackground;
		TheLayerName.addEventListener('mousedown', function(event)
		{
			if (event.button!=0) // right mouse button was pressed, display the popup menu
			{
				//*******************************************************************
				// Create the popup menu if it has not been created already and remove it's contents
				// we use one LayerPopupMenu element for all layers and 
				// just change it's contents when it is selected
				
				var TheMetadata=this.TheLayer.GetSetting("Layer","Metadata");
				
				if (TheMetadata!=undefined)
				{
					var ThePopupMenu=CMUtilities.GetPopupMenu("CM_LayerPopupMenu",event.clientX,event.clientY);
				
					this.TheLayer.AddMetadataMenuItem(ThePopupMenu);
				}
			}
			
			event.stopPropagation(); // stop the document from hidding a popup window
			event.preventDefault(); // keeps regular menu from appearing
			return(false); // old way to keep regular menu from appearing (not sure this is needed)
		});
		//*******************************************************************
		CMUtilities.AbsolutePosition(TheLayerName,Left+30,0,150,this.LayerListItemHeight);
	
		//*******************************************************************
		LayerInList.appendChild(TheLayerName);
	}
};