/******************************************************************************************************************
* CMPanelButtons
*
* @module CMPanelButtons
* @Copyright HSU, Jim Graham, 2019
******************************************************************************************************************/
//******************************************************************
// Constructor
//******************************************************************
function CMPanelButtons(TheCanvasMap) 
{
	CMBase.call(this);

	if (TheCanvasMap==undefined) alert("Sorry, the CMPanelButtons requires a CanvasMap object on construction");
	
	this.SetParent(TheCanvasMap);
	
	// Properties
	this.Tabs=[];
	this.TabContents=[];
}

CMPanelButtons.prototype=Object.create(CMBase.prototype); // inherit prototype functions from CMBase()

CMPanelButtons.prototype.contructor=CMPanelButtons; // override the constructor to go to ours

//******************************************************************
// private Functions
//******************************************************************
/**
* @private
*/
CMPanelButtons.prototype.GetTabIndexFromName=function(TabName)
{
	var Result=-1;
	
	for (var i=0;i<this.Tabs.length;i++)
	{
		if (this.Tabs[i].Name==TabName) // this tab is being selected
		{
			Result=i;
		}
	}
	return(Result);
}

//******************************************************************
// Functions
//******************************************************************
/**
* Set the element that contains the tab panel
*/
CMPanelButtons.prototype.SetElement=function(TheElement)
{
	this.TheElement=TheElement;

	// add the container for the tabs at the top of the div
	
	this.TabContainer=document.createElement("DIV");
	this.TabContainer.className="CM_PanelButtons";
  
	this.TheElement.appendChild(this.TabContainer);
}

/**
* Add a tab to the tab panel.
* @public
* @param ContentID - The id for the content that will be associated with the tab.
* @param Name - The name that will appear in the tabs
* @param ToolTip - Optional tools tip that will appear when the user hovers over the tab.
* @returns - The tab content assocaited with the tab.
*/
CMPanelButtons.prototype.AddTab=function(ContentID,ImagePath,ToolTip,TabContent,MapContainer)
{
	if  (this.TheElement!=null) // tab container
	{
		// create the tab button element
		var ButtonElement=document.createElement("button");
		ButtonElement.id=ContentID+"_tab";
		
		ButtonElement.className="tablinks";
		
		if (ToolTip!=undefined) ButtonElement.title=ToolTip;
		ButtonElement.innerHTML="<img src='"+ImagePath+"'></img>";
		
		ButtonElement.Selected=false;
		ButtonElement.TabPanel=this;
		ButtonElement.ContentID=ContentID;
		
		ButtonElement.onclick=function()
		{
			this.TabPanel.SelectTab(this.ContentID);
			
			//CMMainContainer.HidePopupWindow(); // global
		}
		this.Tabs.push(ButtonElement);
		
		this.TabContainer.appendChild(ButtonElement);
		
		// setup the content container
		if (CMUtilities.IsDefined(TabContent)==false)
		{
			TabContent=document.createElement("DIV");
			TabContent.id=ContentID;
			TabContent.className="CM_ButtonContent";
		}
					
		TabContent.style.visibility="hidden";
		//TabContent.style.zIndex=999999;

		this.TabContents.push(TabContent);
		
		MapContainer.appendChild(TabContent);
	}
	return(TabContent);
}
/**
* Set the currently selected tab
* @public
* @param TheTabButton - the button object that was returned by AddTab()
*/
CMPanelButtons.prototype.SelectTab=function(ContentID)
{
	ContentID=ContentID+"_tab";
	
	for (var i=0;i<this.Tabs.length;i++)
	{
		if (this.Tabs[i].id==ContentID) // this tab is being selected
		{
			//this.TabContents[i].style.display="block";
			this.TabContents[i].style.visibility="visible";
			CMMainContainer.SetPopupWindow(this.TabContents[i]);
			this.Tabs[i].className = this.Tabs[i].className += " active";
		}
		else
		{
			//this.TabContents[i].style.display="none";
			this.TabContents[i].style.visibility="hidden";
			this.Tabs[i].className = this.Tabs[i].className.replace(" active", "");
		}
	}
}
/***
* Returns a tab content element so it can be filled
* @public
* @param TabName - name of the tab whose content will be returned
*/
CMPanelButtons.prototype.GetTabContentElement=function(TabName)
{
	var Result=null;
	
	var Index=this.GetTabIndexFromName(TabName);
	
	if (Index!=-1) Result=this.TabContents[Index];

	return(Result);
}

/***
* Removes a table from tab container
* @public
* @param TabName - name of the tab whose content will be returned
*/
CMPanelButtons.prototype.RemoveTab=function(TabName)
{
	var Index=this.GetTabIndexFromName(TabName);
	
	if (Index!=-1) // found the tool
	{
		var ButtonElement=this.Tabs[Index];
		
		// remove the tool from the tool panel
		ButtonElement.parentNode.removeChild(ButtonElement);
		
		// remove the tool from the array
		this.Tabs.splice(Index,1);
		this.TabContents.splice(Index,1);
	}
}

