/******************************************************************************************************************
* CMPanelLayerList
*
* @module CMPanelLayerList
* @Copyright HSU, Jim Graham, 2019
******************************************************************************************************************/
/*-- Changes check box size and margin --*/
CMPanelLayerList.LAYER_LIST_ITEM_HEIGHT=28;
CMPanelLayerList.LAYER_POPUP_MENU_ITEM_HEIGHT=24;

//******************************************************************
// Constructor
//******************************************************************

/**
* @private
*/
function CMPanelLayerList(TheCanvasMap) 
{
	CMBase.call(this);

	if (TheCanvasMap==undefined) alert("Sorry, the CMPanelLayerList requires a CanvasMap object on construction");
	
	this.SetParent(TheCanvasMap);
	
	var TheScene=TheCanvasMap.GetScene();
	TheScene.AddListener(CMScene.MESSAGE_LAYER_LIST_CHANGED,this,function(TheScene,TheListener,AdditionalInfo)
	{
		TheListener.AddLayerList(TheScene);
	});
	
	// jjg -  move to settings
	this.LayerListItemHeight=CMPanelLayerList.LAYER_LIST_ITEM_HEIGHT;
	this.LayerPopupMenuItemHeight=CMPanelLayerList.LAYER_POPUP_MENU_ITEM_HEIGHT;
	
	// Properties
	this.TheElement=null;
}

CMPanelLayerList.prototype=Object.create(CMBase.prototype); // inherit prototype functions from CMBase()

CMPanelLayerList.prototype.contructor=CMPanelLayerList; // override the constructor to go to ours
//******************************************************************
// Private Functions
//******************************************************************
//******************************************************************
// CMScene functions to create and manage the layer list
// 
// This is a relatively complicated set of code to allow users
// to move, edit, and access properties in the layer list.
// This provides a layer list similar to a GIS application that
// is not appropriate for most web apps.
//******************************************************************
/**
* @private
*/
CMPanelLayerList.prototype.AddLayerToList=function(TheElement,LayerIndex,Left,LayerInListTop,TheScene)
{
	var TheLayer=TheScene.GetChild(0,CMGeo).GetChild(LayerIndex,CMLayer);
	
//	try
	{
		TheLayer.AddToList(TheElement,Left,LayerInListTop,this.LayerListItemHeight);
	}
/*	catch (Error)
	{
		throw(Error);
	}
*/}

//******************************************************************
// Functions
//******************************************************************

CMPanelLayerList.prototype.SetElement=function(TheElement)
{
	this.TheElement=TheElement;
}
/**
* Setup the properties panel for a selected object
*/
CMPanelLayerList.prototype.Setup=function(TheObject)
{
	var SettingsPopup=this.TheElement;
	
	while (SettingsPopup.hasChildNodes()) 
	{
		SettingsPopup.removeChild(SettingsPopup.lastChild);
	}
}

/**
* Add the layer list to the specified DOM element.
* This will typically be a div tag in the map page.
* Override to provide your own layer list.
* @protected, @override
* @param TheElement
*/
CMPanelLayerList.prototype.AddLayerList=function(TheScene) 
{
	TheElement=this.TheElement;
	TheElement.LayerListPanel=this;
	
	// first, remove all the existing elements from the element
	while (TheElement.firstChild) // while there is a first element in the dialog
	{
		// removing the first element moves the next element to the first position
		// so this little loop will remove all the elements from another element
		TheElement.removeChild(TheElement.firstChild);
	}
	
	var Left=TheElement.style.left;
	var Top=TheElement.style.top;
	
	// TheElement.style.borderColor="#cccccc"; // border color for the layers list. style moved to CanvasMap.css.
	Left=0;
	Top=0;
	var LayerInListTop=0;
	
	var TheGeo=TheScene.GetChild(0,CMGeo);
	
	for (var i=0;(i<TheGeo.GetNumChildren(CMLayer));i++)
	{
		var LayerInListTop=Top+(i*this.LayerListItemHeight);
		
		this.AddLayerToList(TheElement,i,Left,LayerInListTop,TheScene);
	}
	//**************************************************************************
	//TheElement.style.border="2px solid #00ff00";
		// moving the cursor for the layer
		
	TheElement.addEventListener('mousemove', function(event)
	{
		if (this.DraggingDiv!=null)
		{
			{
				// set the inital position of the div
				
				var TheElementPosition=$(this).offset();
				
				CMUtilities.AbsolutePosition(this.DraggingDiv,0,event.clientY-TheElementPosition.top,200,0);
			}
			event.preventDefault();
			return(false); // old way to keep regular menu from appearing (not sure this is needed)
		}
	});
	//*****************************************************
	// mouse released to move the layer to a new location
	
	TheElement.addEventListener('mouseup', function(event)
	{
		if (this.DraggingDiv!=null) // the user is dragging a layer in the list
		{
			event.preventDefault();
			
			var TheCanvasMap=this.LayerListPanel.GetParent();
			var TheScene=TheCanvasMap.GetScene();
			
			var TheElementPosition=$(this).offset();
			var NewY=event.clientY-TheElementPosition.top;
			var NewIndex=Math.floor(NewY/this.LayerListPanel.LayerListItemHeight);
			TheScene.MoveLayer(this.DraggingLayer,NewIndex);
			
			this.DraggingDiv.style.visibility= "hidden";
			this.DraggingDiv=null;
			
			this.LayerListPanel.AddLayerList(TheScene);
		}
		return(false); // old way to keep regular menu from appearing (not sure this is needed)
	});
	TheElement.addEventListener('mouseleave', function(event)
	{
		if (this.DraggingDiv!=null)
		{
			this.DraggingDiv.style.visibility= "hidden";
			this.DraggingDiv=null;
			event.preventDefault();
		}
	});
};