/******************************************************************************************************************
* CMProj4Grid
*
* Projects data based on a grid.  This class uses a URL to a Proj4 grid file as created
* by the associated Python script.  This file is used when the user just wants to specify
* a single projection grid.  Otherwise, CMProj4WebService should be used which inherits
* from this class and allows various settings to be specified and a web service call will
* then produce a new grid for those settings.
*
* @module CMProj4Grid
* @Copyright HSU, Jim Graham, 2019
******************************************************************************************************************/
/**
* Below are the settings definitions.
* @public, @settings
*/

CMProj4Grid.SettingDefintions=
{
	Proj4Grid:
	{
		URL: { Name:"URL",Type:CMBase.DATA_TYPE_STRING, Default:"/Archive/ProjectionGrids/robin/5_5/-179_179_-90_90/90.js" }, 
		LongitudeOfOrigin: { Name:"Longitude Of Origin",Type:CMBase.DATA_TYPE_FLOATING, Default:0 }, 
	},
};

//***************************************************************************************
// Constructor
//***************************************************************************************
/*
* Constructor
* @public, @constructs
*/
function CMProj4Grid() 
{
	CMProjector.call(this);

	// other properties
	this.TheProjection; // this is the projection grid data (really a JSON object)
}
CMProj4Grid.prototype=Object.create(CMProjector.prototype); // inherit prototype functions from CMProjector()

CMProj4Grid.prototype.contructor=CMProj4Grid; // override the constructor to go to ours

//***************************************************************************************
//	CMBase functions
//***************************************************************************************

CMProj4Grid.prototype.GetSettingsDefinitions=function() 
{
	var Result={};
	
	for (Key in CMProj4Grid.SettingDefintions)
	{
		Result[Key]=CMProj4Grid.SettingDefintions[Key];
	}

	return(Result); 
}
CMProj4Grid.prototype.CMProjector_SettingsChanged=CMProjector.prototype.SettingsChanged;

CMProj4Grid.prototype.SettingsChanged=function() 
{
	this.CMProjector_SettingsChanged();
	
	this.SendMessageToListeners(CMBase.MESSAGE_SETTINGS_CHANGED);
}

//***************************************************************************************
// CMProjector Functions
//***************************************************************************************

CMProj4Grid.prototype.ProjectFromGeographic=function(Lon,Lat,Elevation)
{
	var Result=null;
	
	if (Lon==undefined) throw("Sorry, the coordinate sent to CMProj4Grid.ProjectFromGeographic() must be defined.");
																													   
	this.Initialize();
	
	if (this.TheProjection!=null)
	{
		// adjust the longitude based on the selected origin
		var OriginalLon=Lon;
		
		var LongitudeOfOrigin=this.GetSetting("Proj4Grid","LongitudeOfOrigin",0);
		
		Lon-=LongitudeOfOrigin;	
		
		while (Lon<-180) Lon+=360;
		while (Lon>180) Lon-=360;
		
		if (Lat>this.TheProjection.LatitudeMax) Lat=this.TheProjection.LatitudeMax;
		if (Lat<this.TheProjection.LatitudeMin) Lat=this.TheProjection.LatitudeMin;
		if (Lon<this.TheProjection.LongitudeMin) Lon=this.TheProjection.LongitudeMin;
		if (Lon>this.TheProjection.LongitudeMax) Lon=this.TheProjection.LongitudeMax;
		
		if ((Lon>this.TheProjection.LongitudeMax)||(Lon<this.TheProjection.LongitudeMin))
		{
			j=12;
		}
		//
		var Eastings=this.TheProjection.Eastings;
		var Northings=this.TheProjection.Northings;
		
		var LonRange=this.TheProjection.LongitudeMax-this.TheProjection.LongitudeMin;
		var LatRange=this.TheProjection.LatitudeMax-this.TheProjection.LatitudeMin;
		
		var NumRows=Eastings.length;
		var NumColumns=Eastings[0].length;
		
		var LonFloatIndex=(Lon-this.TheProjection.LongitudeMin)/LonRange*(NumColumns-1);
		var LatFloatIndex=(Lat-this.TheProjection.LatitudeMin)/LatRange*(NumRows-1);
		
		var LonFactor=LonFloatIndex%1;
		var LatFactor=LatFloatIndex%1;
		
		var LonIndex=Math.floor(LonFloatIndex);
		var LatIndex=Math.floor(LatFloatIndex);
		
		try
		{
			var Easting=0;
			var Northing=0;
			
			if ((LonFactor==0)&&(LatFactor==0))
			{
				Easting=Eastings[LatIndex][LonIndex];
				Northing=Northings[LatIndex][LonIndex];
			}
			else if ((LatFactor==0))
			{
				var EastingSW=Eastings[LatIndex][LonIndex];
				var EastingSE=Eastings[LatIndex][LonIndex+1];
				
				Easting=
					EastingSW*(1-LonFactor)+
					EastingSE*(LonFactor);
			
				var NorthingSW=Northings[LatIndex][LonIndex];
				var NorthingSE=Northings[LatIndex][LonIndex+1];
				
				Northing=
					NorthingSW*(1-LonFactor)+
					NorthingSE*(LonFactor);
			}
			else if ((LonFactor==0))
			{
				var EastingSW=Eastings[LatIndex][LonIndex];
				var EastingNW=Eastings[LatIndex+1][LonIndex];
				
				Easting=
					EastingSW*(1-LatFactor)+
					EastingNW*(LatFactor);
			
				var NorthingSW=Northings[LatIndex][LonIndex];
				var NorthingNW=Northings[LatIndex+1][LonIndex];
				
				Northing=
					NorthingSW*(1-LatFactor)+
					NorthingNW*(LatFactor);
			}
			else
			{
				var EastingSW=Eastings[LatIndex][LonIndex];
				var EastingNW=Eastings[LatIndex+1][LonIndex];
				var EastingSE=Eastings[LatIndex][LonIndex+1];
				var EastingNE=Eastings[LatIndex+1][LonIndex+1];
				
				Easting=
					EastingSW*(1-LonFactor)*(1-LatFactor)+
					EastingSE*(LonFactor)*(1-LatFactor)+
					EastingNW*(1-LonFactor)*(LatFactor)+
					EastingNE*(LonFactor)*(LatFactor);
			
				var NorthingSW=Northings[LatIndex][LonIndex];
				var NorthingNW=Northings[LatIndex+1][LonIndex];
				var NorthingSE=Northings[LatIndex][LonIndex+1];
				var NorthingNE=Northings[LatIndex+1][LonIndex+1];
				
				Northing=
					NorthingSW*(1-LonFactor)*(1-LatFactor)+
					NorthingSE*(LonFactor)*(1-LatFactor)+
					NorthingNW*(1-LonFactor)*(LatFactor)+
					NorthingNE*(LonFactor)*(LatFactor);
			}
			
			if ((isNaN(Northing)==false)&&(isNaN(Easting)==false))
			{
				Result=[Easting,Northing,Elevation];
			}
		}
		catch (err)
		{
			var j=12;
		}
	}
	return(Result);
};

CMProj4Grid.prototype.ProjectToGeographic=function(Easting,Northing,Elevation)
{
	var Result=null;
	
	this.Initialize();
	
	if (this.TheProjection!=null)
	{
		var NumRows=this.TheProjection.NumRows;
		var NumColumns=this.TheProjection.NumColumns;
		
		var Latitudes=this.TheProjection.Latitudes;
		var Longitudes=this.TheProjection.Longitudes;
		
		var EastingRange=this.TheProjection.EastingMax-this.TheProjection.EastingMin;
		var NorthingRange=this.TheProjection.NorthingMax-this.TheProjection.NorthingMin;
		
		var NumRows=Longitudes.length;
		var NumColumns=Longitudes[0].length;
		
		var EastingFloatIndex=(Easting-this.TheProjection.EastingMin)/EastingRange*(NumColumns-1);
		var NorthingFloatIndex=(Northing-this.TheProjection.NorthingMin)/NorthingRange*(NumRows-1);
		
		var EastingFactor=EastingFloatIndex%1;
		var NorthingFactor=NorthingFloatIndex%1;
		
		var EastingIndex=Math.floor(EastingFloatIndex);
		var NorthingIndex=Math.floor(NorthingFloatIndex);
		
		var LongitudeOfOrigin=this.GetSetting("Proj4Grid","LongitudeOfOrigin",0);
		
		try
		{
			var Longitude=0;
			var Latitude=0;
			var FoundValue=false;
			
			if ((EastingFactor==0)&&(NorthingFactor==0))
			{
				if ((NorthingIndex>=0)&&(NorthingIndex<NumRows)&&(EastingIndex>=0)&&(EastingIndex<NumColumns))
				{
					Longitude=Longitudes[NorthingIndex][EastingIndex];
					Latitude=Latitudes[NorthingIndex][EastingIndex];
					
					FoundValue=true;
				}
			}
			else if ((NorthingFactor==0))
			{
				if ((NorthingIndex>=0)&&(NorthingIndex<NumRows)&&(EastingIndex>=0)&&(EastingIndex<NumColumns-1))
				{
					var LongitudeSW=Longitudes[NorthingIndex][EastingIndex];
					var LongitudeSE=Longitudes[NorthingIndex][EastingIndex+1];
					
					Longitude=
						LongitudeSW*(1-EastingFactor)+
						LongitudeSE*(EastingFactor);
				
					var NorthingSW=Latitudes[NorthingIndex][EastingIndex];
					var NorthingSE=Latitudes[NorthingIndex][EastingIndex+1];
					
					Northing=
						NorthingSW*(1-EastingFactor)+
						NorthingSE*(EastingFactor);
					
					FoundValue=true;
				}
			}
			else if ((EastingFactor==0))
			{
				if ((NorthingIndex>=0)&&(NorthingIndex<NumRows-1)&&(EastingIndex>=0)&&(EastingIndex<NumColumns))
				{
					var LongitudeSW=Longitudes[NorthingIndex][EastingIndex];
					var LongitudeNW=Longitudes[NorthingIndex+1][EastingIndex];
					
					Longitude=
						LongitudeSW*(1-NorthingFactor)+
						LongitudeNW*(NorthingFactor);
				
					var NorthingSW=Latitudes[NorthingIndex][EastingIndex];
					var NorthingNW=Latitudes[NorthingIndex+1][EastingIndex];
					
					Northing=
						NorthingSW*(1-NorthingFactor)+
						NorthingNW*(NorthingFactor);
					
					FoundValue=true;
				}
			}
			else
			{
				if ((NorthingIndex>=0)&&(NorthingIndex<NumRows-1)&&(EastingIndex>=0)&&(EastingIndex<NumColumns-1))
				{
					var LongitudeSW=Longitudes[NorthingIndex][EastingIndex];
					var LongitudeNW=Longitudes[NorthingIndex+1][EastingIndex];
					var LongitudeSE=Longitudes[NorthingIndex][EastingIndex+1];
					var LongitudeNE=Longitudes[NorthingIndex+1][EastingIndex+1];
					
					Longitude=
						LongitudeSW*(1-EastingFactor)*(1-NorthingFactor)+
						LongitudeSE*(EastingFactor)*(1-NorthingFactor)+
						LongitudeNW*(1-EastingFactor)*(NorthingFactor)+
						LongitudeNE*(EastingFactor)*(NorthingFactor);
				
					var LatitudesSW=Latitudes[NorthingIndex][EastingIndex];
					var LatitudesNW=Latitudes[NorthingIndex+1][EastingIndex];
					var LatitudesSE=Latitudes[NorthingIndex][EastingIndex+1];
					var LatitudesNE=Latitudes[NorthingIndex+1][EastingIndex+1];
					
					Latitude=
						LatitudesSW*(1-EastingFactor)*(1-NorthingFactor)+
						LatitudesSE*(EastingFactor)*(1-NorthingFactor)+
						LatitudesNW*(1-EastingFactor)*(NorthingFactor)+
						LatitudesNE*(EastingFactor)*(NorthingFactor);
					
					FoundValue=true;
				}
			}
			
			if ((FoundValue)&&(isNaN(Longitude)==false)&&(isNaN(Latitude)==false)&&(Longitude<1000000)&&(Latitude<1000000))
			{
				// shift the lonigtude based on the origin
				Longitude+=LongitudeOfOrigin;
				
				while (Longitude<-180) Longitude+=360;
				while (Longitude>180) Longitude-=360;
						
				Result=[Longitude,Latitude,Elevation];
			}
			else
			{
				var j=12;
			}
		}
		catch(err)
		{
			throw(err);
		}
	}
	return(Result);
};

//***************************************************************************************
// CMProjector Functions
//***************************************************************************************

/**
* Returns clipping bounds that can be used to clip geographic data for the current
* projection settings
* @public
* @returns - an array of clipping polys [Xs[],Ys[]]
*/
CMProj4Grid.prototype.GetClippingPolys=function()
{
	var Result=null;
	
	this.Initialize();

	if (this.TheProjection!=null)
	{
		var LongitudeOfOrigin=this.GetSetting("Proj4Grid","LongitudeOfOrigin",0);
	//	var LatitudeOfOrigin=this.GetSetting("Projection","lat_0",0);
		var LatitudeOfOrigin=0;
		
		var XMin=this.TheProjection.LongitudeMin+LongitudeOfOrigin;
		var XMax=this.TheProjection.LongitudeMax+LongitudeOfOrigin;
		var YMin=this.TheProjection.LatitudeMin+LatitudeOfOrigin;
		var YMax=this.TheProjection.LatitudeMax+LatitudeOfOrigin;
			
		var Pad=0.00001; // this is required to keep drawn polygons from breaking across the clipping areas
		
		Result=CMProjector.GetGeographicClippingPolys(XMin,XMax,YMin,YMax,Pad);
	}
	return(Result);
}

CMProj4Grid.prototype.GetClass=function() { return(CMProj4Grid); }
//***************************************************************************************
// Private CMProj4Grid Functions
//***************************************************************************************
/**
* Allows subclasses to get the projection grid
* @protected
*/
CMProj4Grid.prototype.GetProjectionGrid=function() 
{ 
	return(this.TheProjection); 
}
/**
* Allows subclasses to set the projection grid
* @protected
*/
CMProj4Grid.prototype.SetProjectionGrid=function(NewGrid) 
{ 
	this.TheProjection=NewGrid; 
}

//***************************************************************************************
//CMProj4WebService Functions
//***************************************************************************************
/*
* Requests a grid specified by a URL
* @public 
*/
CMProj4Grid.prototype.RequestGrid=function(Callback)
{
	var URL=TheProjector.GetSetting("Proj4Grid","URL");
	
	var TheRequest=new XMLHttpRequest();
	TheRequest.Callback=Callback;
	TheRequest.TheProjector=this;
	
	TheRequest.onreadystatechange=function() 
	{
		if ((this.readyState == 4)&&( this.status == 200)) // OK
		{
			this.TheProjector.TheProjection=JSON.parse(TheRequest.responseText);
			this.Callback();
		}
	}
	TheRequest.open("GET",URL,true);
	TheRequest.send();
}
/**
* Start of getting the projection bounds based on uncertinaty
* @public 
*/
CMProj4Grid.prototype.GetProectionBounds=function()
{
	var Lons=[-10,0,0,-10];
	var Lats=[0,0,-10,-10];
	
	var Distance=10;
	
	var X,Y0;
	var X1,Y1;
	var X2,Y2;
	var X3,Y3;
	
	var Index1=0;
	while (Index1<Lons.length)
	{
		var NumPoints=Lons.length;
		
		// get the coordinates for the line and the previous and next coordinates
		var Index0=Index1-1;
		if (Index0<0) Index0=NumPoints-1;
		
		var Index2=Index1+1;
		if (Index2>=NumPoints) Index2=(Index0+1)-NumPoints;
		
		var Index3=Index1+2;
		if (Index3>=NumPoints) Index3=(Index0+2)-NumPoints;
		
		X0=Lons[Index0];
		Y0=Lats[Index0];
		X1=Lons[Index1];
		Y1=Lats[Index1];
		X2=Lons[Index2];
		Y2=Lats[Index2];
		X3=Lons[Index3];
		Y3=Lats[Index3];
		// 
		if (Y1==Y2) // flat line
		{
			if (X2>X1) // running left to right (top segment)
			{
				if (Y0<=Y1) // previous is flat or down
				{
					if (Y3<=Y2) // potential tab sticking up
					{
						// add the cell above by adding two points at Index2 (seond coordinate)
						Lons.splice(Index2, 0, X1); // add a new coordinate above the current one
						Lats.splice(Index2, 0,Y1+Distance); 
						Lons.splice(Index2+1, 0, X2); // add a new coordinate above the current one
						Lats.splice(Index2+1, 0, Y2+Distance); // use +1 because Index2 has moved
						Index1+=2; // move past the new points
					}
					else // potential notch to the upper left
					{
						Lats[Index1]=Lats[Index1]+Distance; // move the first coordinate up one space
						
					}
				}
				else // previuos is up
				{
					if (Y3<=Y2) // potential notch to the upper right
					{
					}
				}
			}
		}
		Index1++;		
	}
	
	var Projected=this.ProjectPolyFromGeographicWithoutClip([Lons,Lats]);
	
	var Result={
		Lons:Lons,
		Lats:Lats,
		Eastings:Projected[0],
		Northings:Projected[1]
	};
	return(Result);
}
