/******************************************************************************************************************
* CMScene
*
* @module CMScene
* @Copyright HSU, Jim Graham, 2019
******************************************************************************************************************/
//******************************************************************
// Global definitions
//******************************************************************

/**
* This is the main rendering loop for all scenes and views.
* The function runs continuously.
* @private
*/
function CMRenderScenes() 
 {
	 if (RenderingScene==false)
	 {
		RenderingScene=true; // keep us from being reentrant
		 
		for (var i=0;i<CMScene.TheScenes.length;i++)
		{
			var TheScene=CMScene.TheScenes[i];
		
			var TheView=TheScene.GetView(0);
			TheScene.Paint(TheView);
		}
		RenderingScene=false;
	 }
	requestAnimationFrame(CMRenderScenes);
}
var RenderingScene=false;

// get the scenes started rendering

requestAnimationFrame(CMRenderScenes);

// global variables for tracking the scenes
CMScene.TheScenes=[];

CMScene.NumScenes=0;

//******************************************************************
// Global CMScene definitions
//******************************************************************

/**
* Message definitions
*/
CMScene.MESSAGE_LAYER_LIST_CHANGED=CMBase.GetUniqueNumber();
CMScene.MESSAGE_LAYER_CONTENT_CHANGED=CMBase.GetUniqueNumber();
CMScene.MESSAGE_LAYER_SETTINGS_CHANGED=CMBase.GetUniqueNumber();
CMScene.MESSAGE_BACKGROUNDS_CHANGED=CMBase.GetUniqueNumber();
CMScene.MESSAGE_TIME_SLICES_CHANGED=CMBase.GetUniqueNumber();
CMScene.MESSAGE_TIME_RANGE_CHANGED=CMBase.GetUniqueNumber();
CMScene.MESSAGE_SELECTION_CHANGED=CMBase.GetUniqueNumber();

//******************************************************************
// CMScene Class
//******************************************************************
/*4
* Constructor for the CMScene class
* @public, @constructs
*/
function CMScene(TheCanvasMap) 
{
	CMItem.call(this);

	this.SetParent(TheCanvasMap);
	
	// Global Static variables
	CMScene.TheScenes.push(this);
	CMScene.NumScenes++;
	
	// Properties
	this.Views=[]; // jjg - move to children?
	
 	this.MinTime=0; 
	
	// private properties
	this.Times=[0];
	
	this.TheBounds=null;

	this.NumRepaintBlocks=0;
	this.NeedRepaint=false; // needs to move into the view in the future
	
	// provide good default settings
	this.SetSetting("Style","fillStyle","rgb(210,220,255)"); // 

	this.SetName("Scene_"+CMScene.NumScenes);
}

CMScene.prototype=Object.create(CMItem.prototype); // inherit prototype functions from PanelBase()

CMScene.prototype.contructor=CMScene; // override the constructor to go to ours

//******************************************************************
//  Protected functions
//******************************************************************
/*
* Allows layers to get the view that they are drawn in.
* @protected
* @param ViewIndex - allows future expansion to have mulitple views in one scene
*/
CMScene.prototype.GetView=function(ViewIndex) 
{
	var Result=null;
	
	if (ViewIndex==undefined) ViewIndex=0;
	
	Result=this.Views[ViewIndex];
	
	return(Result);
}
CMScene.prototype.GetNumViews=function() 
{
	var Result=this.Views.length;
	
	return(Result);
}
/*
* Allos the view to be replaced
*/
CMScene.prototype.SetView=function(ViewIndex,NewView) 
{
	NewView.SetParent(this);
	
	this.Views[ViewIndex]=NewView;
}

//******************************************************************
// CMBase Functions
//******************************************************************

CMScene.prototype.GetTimes=function(TheTimeSlices) 
{
	return(this.Times);
}
//******************************************************************
// Background Management
//******************************************************************
/*
* Adds a background layer to the scene
* @public
* @param TheLayer
* @returns LayerIndex 
*/
CMScene.prototype.AddBackground=function(TheLayer) 
{
	var Index=this.GetChild(0,CMGeo).AddBackground(TheLayer);
	
	return(Index);
}
/*
* Sets which background will be selected for painting
* @public
* @param BackgroundIndex - 
*/
CMScene.prototype.SetSelectedBackgroundIndex=function(New)
{
	var Index=this.GetChild(0,CMGeo).SetSelectedBackgroundIndex(New);
}


//******************************************************************
// public CMScene functions for Geo Management
//******************************************************************
/*CMScene.prototype.SetGeo=function(NewGeo,Index)
{
	if (Index==undefined) Index=0;
	
	if (this.Geos==null) { this.Geos=[]; }
	
	this.Geos[Index]=NewGeo;
};*/
CMScene.prototype.GetGeo=function(Index)
{
	if (Index==undefined) Index=0;
	
	return(this.GetChild(Index,CMGeo));
};
//******************************************************************
// public CMScene functions for layer Management
//******************************************************************
/*
* Adds a layer to the scene
* @public
* @param TheLayer
* @returns LayerIndex 
*/
CMScene.prototype.AddLayer=function(TheLayer,GeoIndex) 
{
	if (GeoIndex==undefined) GeoIndex=0;
	
	var LayerIndex=this.GetChild(GeoIndex,CMGeo).AddLayer(TheLayer);
	
	return(LayerIndex);
}

// these get moved to CMBase as child functions
/*
* Return the index for the specified layer.  If the 
* layer does not appear in the CMScene, return -1
* @public
* @param TheLayer - 
* @returns LayerIndex
*/
CMScene.prototype.GetLayerIndex=function(TheLayer,GeoIndex) 
{
	if (GeoIndex==undefined) GeoIndex=0;
	
	var Result=this.GetChild(GeoIndex,CMGeo).GetLayerIndex(TheLayer);
	return(Result);
}
/*
* Returns the layer at the specified index.
* @public
* @param Index - 
* @returns TheLayer
*/
CMScene.prototype.GetLayer=function(LayerIndex,GeoIndex) 
{
	if (GeoIndex==undefined) GeoIndex=0;
	
	var Result=this.GetChild(GeoIndex,CMGeo).GetLayer(LayerIndex);
	
	return(Result);
}

/*
* Swap the layer with the one above it.
* @public
* @param Index - 
*/
CMScene.prototype.MoveLayerUp=function(Index,GeoIndex) 
{
	if (GeoIndex==undefined) GeoIndex=0;
	
	var Result=this.GetChild(GeoIndex,CMGeo).MoveLayerUp(Index);
}
/*
* Swap the specified layer with the one below it.
* @public
* @param Index - 
*/
CMScene.prototype.MoveLayerDown=function(Index,GeoIndex) 
{
	if (GeoIndex==undefined) GeoIndex=0;
	
	var Result=this.GetChild(GeoIndex,CMGeo).MoveLayerDown(Index);
}

/**
* Remove the layer at the specified index from the list of layers
* @public
* @param Index - 
*/
CMScene.prototype.DeleteLayer=function(Index,GeoIndex) 
{
	if (GeoIndex==undefined) GeoIndex=0;
	
	var Result=this.GetChild(GeoIndex,CMGeo).DeleteLayer(Index);
	return(Result);
}
/**
* Moves a layer to a new position in the layer list 
* @public
* @param TheLayer - 
* @param NewIndex - 
*/
CMScene.prototype.MoveLayer=function(TheLayer,NewIndex,GeoIndex) 
{
	if (GeoIndex==undefined) GeoIndex=0;
	
	this.GetChild(GeoIndex,CMGeo).MoveLayer(TheLayer,NewIndex);
}
//******************************************************************
// Search
//******************************************************************
/**
* Get the search results from each layer.  The layers will insert 
* elements into the SearchResults element
*
* Called by CanvasMap
* @protected
*/
CMScene.prototype.GetSearchResults=function(TheText,SearchResults) 
{
	this.GetChild(0,CMGeo).GetSearchResults(TheText,SearchResults);

}
//******************************************************************
// Map Elements
//******************************************************************
/**
* Add a new map element to the scene (e.g. a scale bar)
* @public
* @param TheElement - 
*/
CMScene.prototype.AddMapElement=function(TheElement) 
{
	this.Views[0].AddMapElement(TheElement);
}

//******************************************************************
// Messages from the views that are then sent to the layers
//******************************************************************
/**
* Called by CanvasMap to add a view for this scene.
* @protected
*/
CMScene.prototype.AddView=function(TheView) 
{
	var ViewIndex=this.Views.length;
	
	this.SetView(ViewIndex,TheView);
	
	this.SetTimeRange(0);
}

/**
* Sets the minimum and maximum time for elements in the map.
* This is under development and currently layers need to
* check the time before painting into the view.
* @public
* @param MinTime - lowest allowed value for the time
*/
CMScene.prototype.SetTimeRange=function(MinTime)
{
	this.MinTime=MinTime;
	
	this.SendMessageToListeners(CMScene.MESSAGE_TIME_RANGE_CHANGED);
}
/**
* Gets the current setting for the time range.
* @public
* @returns time range as a JSON object with MinTime, MaxTime
*/
CMScene.prototype.GetTimeRange=function()
{
	return(this.MinTime);
}

//******************************************************************
// Event handlers
//******************************************************************
/*
* Not used
*/
CMScene.prototype.In=function(TheView,RefX,RefY,TheEvent) 
{
	var Result=null;
	
	// in operates the opposite direction as painting
	
	var NumGeos=this.GetNumChildren(CMGeo);
	
	for (var i=NumGeos-1;(i>=0)&&(Result==null);i--)
	{
		var TheGeo=this.GetChild(i,CMGeo);
		
		var Result=TheGeo.In(TheView,RefX,RefY,TheEvent);
		
		if (Result!=null) Result.GeoIndex=i;
	}
	return(Result);
};

/**
* Called when there is a mouse down event received by CanvasMap
* @public
* @override
* @param TheView - the view that recieved the event
* @param RefX - reference horiziontal coordinate value
* @param RefY - reference vertical coordinate value
*//*
CMScene.prototype.MouseDown=function(TheView,RefX,RefY,TheEvent) 
{
	var Used=false;
	
	var NumGeos=this.GetNumChildren(CMGeo);
	
	for (var i=NumGeos-1;(i>=0)&&(Used==false);i--)
	{
		var TheGeo=this.GetChild(i);
		
		Used=TheGeo.MouseDown(TheView,RefX,RefY,TheEvent);
	}
	return(Used);
};*/
/**
* Called when there is a mouse move event received by CanvasMap
* @public
* @override
* @param TheView - the view that recieved the event
* @param RefX - reference horiziontal coordinate value
* @param RefY - reference vertical coordinate value
*//*
CMScene.prototype.MouseMove=function(TheView,RefX,RefY,TheEvent) 
{
	var Used=false;
	
	var NumGeos=this.GetNumChildren(CMGeo);
	
	for (var i=NumGeos-1;(i>=0)&&(Used==false);i--)
	{
		var TheGeo=this.GetChild(i);
		
		Used=TheGeo.MouseMove(TheView,RefX,RefY,TheEvent);
	}
	return(Used);
};*/
/**
* Called when there is a mouse up event received by CanvasMap
* @public
* @override
* @param TheView - the view that recieved the event
* @param RefX - reference horiziontal coordinate value
* @param RefY - reference vertical coordinate value
*//*
CMScene.prototype.MouseUp=function(TheView,RefX,RefY,TheEvent) 
{
	var Used=false;
	
	var NumGeos=this.GetNumChildren(CMGeo);
	
	for (var i=NumGeos-1;(i>=0)&&(Used==false);i--)
	{
		var TheGeo=this.GetChild(i);
		
		Used=TheGeo.MouseUp(TheView,RefX,RefY,TheEvent);
	}
	return(Used);
};
*/
//******************************************************************
// Painting
//******************************************************************
/**
* Called when the window is resized
* @public
* @override
* @param TheView - the view that recieved the event
*/
CMScene.prototype.Resize=function(TheView) 
{
	var NumGeos=this.GetNumChildren(CMGeo);
	
	for (var i=0;i<NumGeos;i++)
	{
		var TheGeo=this.GetChild(i,CMGeo);
		
		TheGeo.Resize(TheView);
	}
	// resize the views
	
	for (var i=0;i<this.Views.length;i++)
	{
		this.Views[i].Resize();
	}
	this.Repaint();
}
//*****************************************************************************
// Below is the code to repaint the views as needed
// To do:
// - Move the "NeedPaint" flag into the view
//*****************************************************************************

CMScene.prototype.Paint=function(TheView) 
{
	var NeedRepaint=this.NeedRepaint;
	
	if ((typeof(CM3Scene)!== 'undefined')&&(this instanceof CM3Scene)) NeedRepaint=true;
	
	if ((this.NumRepaintBlocks==0)&&(NeedRepaint))
	{
		this.NeedRepaint=false;
		
		this.IncrementRepaintBlocks();
		
		// let the view know we are starting
		
		TheView.PaintStart();
		
		// clear the background
		
		var Style=this.GetStyle(TheView,0);
		
		if (Style!=undefined)
		{
			TheView.SetStyle(Style);
			TheView.PaintBackground();
			TheView.RestoreStyle();
		}
		
		// then the other layers
		
		var NumChildren=this.GetNumChildren();
	
		for (var i=0;i<NumChildren;i++)
		{
			var TheChild=this.GetChild(i);
		
			TheChild.Paint(TheView);
		}
		
		//
		TheView.PaintEnd();
		
		//
		
		this.DecrementRepaintBlocks();
		
		if (this.NeedRepaint) this.Repaint();
	}

}
/**
* Called by the layers to repaint the scene
* @public
* @override
* @param TheView - the view that recieved the event
*/
CMScene.prototype.Repaint=function() 
{
	if (this.Views!=null)
	{
		for (var i=0;i<this.Views.length;i++)
		{
			this.NeedRepaint=true;
//			this.Paint(this.Views[i]); // jjg for now
		}
	}
}
/**
* Called to block repainting when a large number of settings are made that would all
* generate repaints.
* @public
*/
CMScene.prototype.IncrementRepaintBlocks=function()
{
	this.NumRepaintBlocks++;
}
/**
* Calls to IncrementRepaintBlocks must be paired with calls to this function.
* generate repaints.
* @public
*/
CMScene.prototype.DecrementRepaintBlocks=function()
{
	this.NumRepaintBlocks--;
}
//******************************************************************
// Layer List functions
//******************************************************************
/**
* Called from a scene to indicate that a layer's settings changed.
* @protected
* @param TheLayer
*/
CMScene.prototype.LayerListChanged=function()
{
	this.SendMessageToListeners(CMScene.MESSAGE_LAYER_LIST_CHANGED,this);
}
CMScene.prototype.LayerContentChanged=function(ItemThatChanged)
{
	this.SendMessageToListeners(CMScene.MESSAGE_LAYER_CONTENT_CHANGED,ItemThatChanged);
}
CMScene.prototype.LayerSettingsChanged=function(ItemThatChanged)
{
	this.SendMessageToListeners(CMScene.MESSAGE_LAYER_SETTINGS_CHANGED,ItemThatChanged);
	this.Repaint();
}
CMScene.prototype.SelectionChanged=function(ItemThatChanged)
{
	this.SendMessageToListeners(CMScene.MESSAGE_SELECTION_CHANGED,ItemThatChanged);
	this.Repaint();
}
CMScene.prototype.BackgroundListChanged=function()
{
	this.SendMessageToListeners(CMScene.MESSAGE_BACKGROUNDS_CHANGED,this);
}

//******************************************************************
// Additional public functions
//******************************************************************
/**
* Set the projection for coordinate conversion
* @public
* @param TheProjector - the projector to use to project coordinates
*/
CMScene.prototype.SetProjector=function(TheProjector) 
{ 
	var TheGeo=this.GetChild(0,CMGeo);
	
	if (TheGeo!=null) TheGeo.SetProjector(TheProjector);
	else alert("Sorry, the Geo must be setup before the projector can be set");
}
CMScene.prototype.GetProjector=function() 
{ 
	var TheProjector=null;
	
	var TheGeo=this.GetChild(0,CMGeo);
	
	if (TheGeo!=null) TheProjector=TheGeo.GetProjector();
							
	return(TheProjector); 
}

/**
* Called by layers to force the bounds to be recomputed.
* @protected
*/
CMScene.prototype.SetBoundsDirty=function() 
{ 
	this.TheBounds=null;
}
/**
* Gets the bounds of all the geo bounds combined.
* @public
*/
CMScene.prototype.GetBounds=function() 
{ 
	var NumGeos=this.GetNumChildren(CMGeo);
	
	if ((NumGeos>0)&&(this.TheBounds==null)) // bounds is dirty and we have layer data
	{
		for (var i=0;i<NumGeos;i++)
		{
			var TheGeo=this.GetChild(i,CMGeo);
			
			var TheGeoBounds=TheGeo.GetBounds();
			
			if (TheGeoBounds!=null)
			{
				if (this.TheBounds==null) // create for the first time
				{
					this.TheBounds=CMUtilities.CloneBounds(TheGeoBounds);
				}
				else // add this features bounds to the current bounds
				{
					CMUtilities.AddToBounds(this.TheBounds,TheGeoBounds);
				}
			}
		}
	}
	return(this.TheBounds);
}
/**
* Insert a Time in the appropraite location in the array
* @public 
* @param Time - the time slice to insert
*/
CMScene.prototype.InsertTime=function(Time) 
{
	Time=parseFloat(Time); // make sure we have a number

	CMUtilities.InsertIntoSortedArray(this.Times,Time);
	
	this.SendMessageToListeners(CMScene.MESSAGE_TIME_SLICES_CHANGED);
}
/**
* Remove the entry for the specified Time
* @public 
* @param Time - the time slice to delete
*/
CMScene.prototype.DeleteTime=function(Time) 
{ 
	Time=parseFloat(Time); // make sure we have a number

	var Index=this.Times.indexOf(Time);
	if (Index!=-1)
	{
		this.Times.splice(Index,1);
	}
	this.SendMessageToListeners(CMScene.MESSAGE_TIME_SLICES_CHANGED);
}


/**
* Unselect all information in the layers
* @public
*/
CMScene.prototype.CMItem_UnselectAll=CMItem.prototype.UnselectAll;

CMScene.prototype.UnselectAll=function(SendMessageFlag) 
{
	this.CMItem_UnselectAll(SendMessageFlag);
	
	var NumGeos=this.GetNumChildren(CMGeo);
	
	for (var i=0;i<NumGeos;i++)
	{
		var TheGeo=this.GetChild(i);
		
		TheGeo.UnselectAll(SendMessageFlag);
	}

}

